@@ -364,6 +364,58 @@ class AwsCredentialsProvider(AwsCredentialsProviderBase):
             http_proxy_options)
         return cls(binding)
 
+    @classmethod
+    def new_x509(
+            cls,
+            *,
+            endpoint: str,
+            thing_name: str,
+            role_alias: str,
+            tls_ctx: awscrt.io.ClientTlsContext,
+            client_bootstrap: Optional[ClientBootstrap] = None,
+            http_proxy_options: Optional[HttpProxyOptions] = None):
+        """
+        Creates a provider that sources credentials from IoT's X509 credentials service.
+
+        Args:
+            endpoint (str): X509 service regional endpoint to source credentials from.
+                            This is a per-account value that can be determined via the CLI:
+                            `aws iot describe-endpoint --endpoint-type iot:CredentialProvider`
+
+            thing_name (str): The name of the IoT thing to use to fetch credentials.
+
+            role_alias (str): The name of the role alias to fetch credentials through.
+
+            tls_ctx (ClientTlsContext): The client TLS context to use when establishing the http connection to IoT's X509 credentials service.
+
+            client_bootstrap (Optional[ClientBootstrap]): Client bootstrap to use when initiating a socket connection.
+                If not set, uses the static default ClientBootstrap instead.
+
+            http_proxy_options (Optional[HttpProxyOptions]): Optional HTTP proxy options.
+                If None is provided then an HTTP proxy is not used.
+
+        Returns:
+            AwsCredentialsProvider:
+        """
+
+        assert isinstance(endpoint, str)
+        assert isinstance(thing_name, str)
+        assert isinstance(role_alias, str)
+        assert isinstance(tls_ctx, ClientTlsContext)
+        assert isinstance(http_proxy_options, HttpProxyOptions) or http_proxy_options is None
+        if client_bootstrap is None:
+            client_bootstrap = ClientBootstrap.get_or_create_static_default()
+        assert isinstance(client_bootstrap, ClientBootstrap)
+
+        binding = _awscrt.credentials_provider_new_x509(
+            endpoint,
+            thing_name,
+            role_alias,
+            tls_ctx,
+            client_bootstrap,
+            http_proxy_options)
+        return cls(binding)
+
     def get_credentials(self):
         """
         Asynchronously fetch AwsCredentials.
@@ -403,6 +455,9 @@ class AwsSigningAlgorithm(IntEnum):
     V4_ASYMMETRIC = 1
     """Signature Version 4 - Asymmetric"""
 
+    V4_S3EXPRESS = 2
+    """Signature Version 4 - S3 Express"""
+
 
 class AwsSignatureType(IntEnum):
     """Which sort of signature should be computed from the signable."""
@@ -452,7 +507,7 @@ class AwsSignedBodyHeaderType(IntEnum):
     """Do not add a header."""
 
     X_AMZ_CONTENT_SHA_256 = 1
-    """Add the "x-amz-content-sha-256" header with the canonical request's signed body value"""
+    """Add the "x-amz-content-sha256" header with the canonical request's signed body value"""
 
 
 class AwsSigningConfig(NativeResource):
@@ -543,11 +598,11 @@ class AwsSigningConfig(NativeResource):
     )
 
     def __init__(self,
-                 algorithm,
-                 signature_type,
-                 credentials_provider,
-                 region,
-                 service,
+                 algorithm=AwsSigningAlgorithm.V4,
+                 signature_type=AwsSignatureType.HTTP_REQUEST_HEADERS,
+                 credentials_provider=None,
+                 region="",
+                 service="",
                  date=None,
                  should_sign_header=None,
                  use_double_uri_encode=True,
@@ -560,7 +615,7 @@ class AwsSigningConfig(NativeResource):
 
         assert isinstance(algorithm, AwsSigningAlgorithm)
         assert isinstance(signature_type, AwsSignatureType)
-        assert isinstance(credentials_provider, AwsCredentialsProvider)
+        assert isinstance(credentials_provider, AwsCredentialsProvider) or credentials_provider is None
         assert isinstance(region, str)
         assert isinstance(service, str)
         assert callable(should_sign_header) or should_sign_header is None
