@@ -4,6 +4,7 @@
  */
 #include <aws/cal/private/symmetric_cipher_priv.h>
 
+#define OPENSSL_SUPPRESS_DEPRECATED
 #include <openssl/evp.h>
 
 struct openssl_aes_cipher {
@@ -13,8 +14,21 @@ struct openssl_aes_cipher {
     struct aws_byte_buf working_buffer;
 };
 
+static struct aws_byte_cursor s_empty_plain_text = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("");
+
 static int s_encrypt(struct aws_symmetric_cipher *cipher, struct aws_byte_cursor input, struct aws_byte_buf *out) {
 
+    /*
+     * Openssl 1.1.1 and its derivatives like aws-lc and boringssl do not handle
+     * the case of null input of 0 len gracefully in update (it succeeds, but
+     * finalize after it will fail). Openssl 3.0 fixed this. Other crypto implementations
+     * do not have similar issue.
+     * To workaround the issue, replace null cursor with empty cursor.
+     */
+    if (input.len == 0) {
+        input = s_empty_plain_text;
+    }
+
     size_t required_buffer_space = input.len + cipher->block_size;
 
     if (aws_symmetric_cipher_try_ensure_sufficient_buffer_space(out, required_buffer_space)) {
@@ -27,7 +41,7 @@ static int s_encrypt(struct aws_symmetric_cipher *cipher, struct aws_byte_cursor
     int len_written = (int)(available_write_space);
     if (!EVP_EncryptUpdate(
             openssl_cipher->encryptor_ctx, out->buffer + out->len, &len_written, input.ptr, (int)input.len)) {
-        cipher->good = false;
+        cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
         return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
 
@@ -46,7 +60,7 @@ static int s_finalize_encryption(struct aws_symmetric_cipher *cipher, struct aws
 
     int len_written = (int)(out->capacity - out->len);
     if (!EVP_EncryptFinal_ex(openssl_cipher->encryptor_ctx, out->buffer + out->len, &len_written)) {
-        cipher->good = false;
+        cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
         return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
 
@@ -68,8 +82,7 @@ static int s_decrypt(struct aws_symmetric_cipher *cipher, struct aws_byte_cursor
     int len_written = (int)available_write_space;
     if (!EVP_DecryptUpdate(
             openssl_cipher->decryptor_ctx, out->buffer + out->len, &len_written, input.ptr, (int)input.len)) {
-        cipher->good = false;
-
+        cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
         return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
 
@@ -88,7 +101,7 @@ static int s_finalize_decryption(struct aws_symmetric_cipher *cipher, struct aws
 
     int len_written = (int)out->capacity - out->len;
     if (!EVP_DecryptFinal_ex(openssl_cipher->decryptor_ctx, out->buffer + out->len, &len_written)) {
-        cipher->good = false;
+        cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
         return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
 
@@ -110,9 +123,7 @@ static void s_destroy(struct aws_symmetric_cipher *cipher) {
     aws_byte_buf_clean_up_secure(&cipher->key);
     aws_byte_buf_clean_up_secure(&cipher->iv);
 
-    if (cipher->tag.buffer) {
-        aws_byte_buf_clean_up_secure(&cipher->tag);
-    }
+    aws_byte_buf_clean_up_secure(&cipher->tag);
 
     if (cipher->aad.buffer) {
         aws_byte_buf_clean_up_secure(&cipher->aad);
@@ -129,7 +140,7 @@ static int s_clear_reusable_state(struct aws_symmetric_cipher *cipher) {
     EVP_CIPHER_CTX_cleanup(openssl_cipher->encryptor_ctx);
     EVP_CIPHER_CTX_cleanup(openssl_cipher->decryptor_ctx);
     aws_byte_buf_secure_zero(&openssl_cipher->working_buffer);
-    cipher->good = true;
+    cipher->state = AWS_SYMMETRIC_CIPHER_READY;
     return AWS_OP_SUCCESS;
 }
 
@@ -214,7 +225,7 @@ struct aws_symmetric_cipher *aws_aes_cbc_256_new_impl(
         goto error;
     }
 
-    cipher->cipher_base.good = true;
+    cipher->cipher_base.state = AWS_SYMMETRIC_CIPHER_READY;
     return &cipher->cipher_base;
 
 error:
@@ -305,7 +316,7 @@ struct aws_symmetric_cipher *aws_aes_ctr_256_new_impl(
         goto error;
     }
 
-    cipher->cipher_base.good = true;
+    cipher->cipher_base.state = AWS_SYMMETRIC_CIPHER_READY;
     return &cipher->cipher_base;
 
 error:
@@ -313,28 +324,51 @@ error:
     return NULL;
 }
 
+static int s_gcm_decrypt(struct aws_symmetric_cipher *cipher, struct aws_byte_cursor input, struct aws_byte_buf *out) {
+    if (cipher->tag.buffer == NULL) {
+        return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+    }
+
+    return s_decrypt(cipher, input, out);
+}
+
 static int s_finalize_gcm_encryption(struct aws_symmetric_cipher *cipher, struct aws_byte_buf *out) {
     struct openssl_aes_cipher *openssl_cipher = cipher->impl;
 
+    if (cipher->tag.buffer == NULL) {
+        aws_byte_buf_init(&cipher->tag, cipher->allocator, AWS_AES_256_CIPHER_BLOCK_SIZE);
+    }
+
     int ret_val = s_finalize_encryption(cipher, out);
 
     if (ret_val == AWS_OP_SUCCESS) {
-        if (!cipher->tag.len) {
-            if (!EVP_CIPHER_CTX_ctrl(
-                    openssl_cipher->encryptor_ctx,
-                    EVP_CTRL_GCM_GET_TAG,
-                    (int)cipher->tag.capacity,
-                    cipher->tag.buffer)) {
-                cipher->good = false;
-                return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
-            }
-            cipher->tag.len = AWS_AES_256_CIPHER_BLOCK_SIZE;
+        if (!EVP_CIPHER_CTX_ctrl(
+                openssl_cipher->encryptor_ctx, EVP_CTRL_GCM_GET_TAG, (int)cipher->tag.capacity, cipher->tag.buffer)) {
+            cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
+            return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
         }
+        cipher->tag.len = AWS_AES_256_CIPHER_BLOCK_SIZE;
     }
 
     return ret_val;
 }
 
+static int s_finalize_gcm_decryption(struct aws_symmetric_cipher *cipher, struct aws_byte_buf *out) {
+    struct openssl_aes_cipher *openssl_cipher = cipher->impl;
+
+    if (openssl_cipher->cipher_base.tag.len) {
+        if (!EVP_CIPHER_CTX_ctrl(
+                openssl_cipher->decryptor_ctx,
+                EVP_CTRL_GCM_SET_TAG,
+                (int)openssl_cipher->cipher_base.tag.len,
+                openssl_cipher->cipher_base.tag.buffer)) {
+            return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+        }
+    }
+
+    return s_finalize_decryption(cipher, out);
+}
+
 static int s_init_gcm_cipher_materials(struct aws_symmetric_cipher *cipher) {
     struct openssl_aes_cipher *openssl_cipher = cipher->impl;
 
@@ -375,15 +409,7 @@ static int s_init_gcm_cipher_materials(struct aws_symmetric_cipher *cipher) {
         }
     }
 
-    if (openssl_cipher->cipher_base.tag.len) {
-        if (!EVP_CIPHER_CTX_ctrl(
-                openssl_cipher->decryptor_ctx,
-                EVP_CTRL_GCM_SET_TAG,
-                (int)openssl_cipher->cipher_base.tag.len,
-                openssl_cipher->cipher_base.tag.buffer)) {
-            return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
-        }
-    }
+    aws_byte_buf_clean_up_secure(&openssl_cipher->cipher_base.tag);
 
     return AWS_OP_SUCCESS;
 }
@@ -403,9 +429,9 @@ static struct aws_symmetric_cipher_vtable s_gcm_vtable = {
     .provider = "OpenSSL Compatible LibCrypto",
     .destroy = s_destroy,
     .reset = s_reset_gcm_cipher_materials,
-    .decrypt = s_decrypt,
+    .decrypt = s_gcm_decrypt,
     .encrypt = s_encrypt,
-    .finalize_decryption = s_finalize_decryption,
+    .finalize_decryption = s_finalize_gcm_decryption,
     .finalize_encryption = s_finalize_gcm_encryption,
 };
 
@@ -413,8 +439,7 @@ struct aws_symmetric_cipher *aws_aes_gcm_256_new_impl(
     struct aws_allocator *allocator,
     const struct aws_byte_cursor *key,
     const struct aws_byte_cursor *iv,
-    const struct aws_byte_cursor *aad,
-    const struct aws_byte_cursor *decryption_tag) {
+    const struct aws_byte_cursor *aad) {
 
     struct openssl_aes_cipher *cipher = aws_mem_calloc(allocator, 1, sizeof(struct openssl_aes_cipher));
     cipher->cipher_base.allocator = allocator;
@@ -452,20 +477,12 @@ struct aws_symmetric_cipher *aws_aes_gcm_256_new_impl(
         aws_byte_buf_init_copy_from_cursor(&cipher->cipher_base.aad, allocator, *aad);
     }
 
-    /* Set tag for the decryptor to use.*/
-    if (decryption_tag) {
-        aws_byte_buf_init_copy_from_cursor(&cipher->cipher_base.tag, allocator, *decryption_tag);
-    } else {
-        /* we'll need this later when we grab the tag during encryption time. */
-        aws_byte_buf_init(&cipher->cipher_base.tag, allocator, AWS_AES_256_CIPHER_BLOCK_SIZE);
-    }
-
     /* Initialize the cipher contexts with the specified key and IV. */
     if (s_init_gcm_cipher_materials(&cipher->cipher_base)) {
         goto error;
     }
 
-    cipher->cipher_base.good = true;
+    cipher->cipher_base.state = AWS_SYMMETRIC_CIPHER_READY;
     return &cipher->cipher_base;
 
 error:
@@ -491,7 +508,7 @@ static int s_key_wrap_finalize_encryption(struct aws_symmetric_cipher *cipher, s
     struct openssl_aes_cipher *openssl_cipher = cipher->impl;
 
     if (openssl_cipher->working_buffer.len < MIN_CEK_LENGTH_BYTES) {
-        cipher->good = false;
+        cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
         return aws_raise_error(AWS_ERROR_INVALID_STATE);
     }
 
@@ -534,7 +551,7 @@ static int s_key_wrap_finalize_encryption(struct aws_symmetric_cipher *cipher, s
             /* encrypt the concatenated A and R[I] and store it in B */
             if (!EVP_EncryptUpdate(
                     openssl_cipher->encryptor_ctx, b.buffer, &b_out_len, temp_input.buffer, (int)temp_input.capacity)) {
-                cipher->good = false;
+                cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
                 return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
             }
 
@@ -559,7 +576,7 @@ static int s_key_wrap_finalize_decryption(struct aws_symmetric_cipher *cipher, s
     struct openssl_aes_cipher *openssl_cipher = cipher->impl;
 
     if (openssl_cipher->working_buffer.len < MIN_CEK_LENGTH_BYTES + KEYWRAP_BLOCK_SIZE) {
-        cipher->good = false;
+        cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
         return aws_raise_error(AWS_ERROR_INVALID_STATE);
     }
 
@@ -606,7 +623,7 @@ static int s_key_wrap_finalize_decryption(struct aws_symmetric_cipher *cipher, s
             /* Decrypt the concatenated buffer */
             if (!EVP_DecryptUpdate(
                     openssl_cipher->decryptor_ctx, b.buffer, &b_out_len, temp_input.buffer, (int)temp_input.capacity)) {
-                cipher->good = false;
+                cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
                 return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
             }
 
@@ -624,7 +641,7 @@ static int s_key_wrap_finalize_decryption(struct aws_symmetric_cipher *cipher, s
     /* here we perform the integrity check to make sure A == 0xA6A6A6A6A6A6A6A6 */
     for (size_t i = 0; i < KEYWRAP_BLOCK_SIZE; ++i) {
         if (a[i] != INTEGRITY_VALUE) {
-            cipher->good = false;
+            cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
             return aws_raise_error(AWS_ERROR_CAL_SIGNATURE_VALIDATION_FAILED);
         }
     }
@@ -640,7 +657,7 @@ static int s_init_keywrap_cipher_materials(struct aws_symmetric_cipher *cipher)
           EVP_CIPHER_CTX_set_padding(openssl_cipher->encryptor_ctx, 0)) ||
         !(EVP_DecryptInit_ex(openssl_cipher->decryptor_ctx, EVP_aes_256_ecb(), NULL, cipher->key.buffer, NULL) &&
           EVP_CIPHER_CTX_set_padding(openssl_cipher->decryptor_ctx, 0))) {
-        cipher->good = false;
+        cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
         return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
 
@@ -700,7 +717,7 @@ struct aws_symmetric_cipher *aws_aes_keywrap_256_new_impl(
         goto error;
     }
 
-    cipher->cipher_base.good = true;
+    cipher->cipher_base.state = AWS_SYMMETRIC_CIPHER_READY;
     return &cipher->cipher_base;
 
 error:
