@@ -161,10 +161,10 @@ typedef struct aws_string *(aws_http_proxy_negotiation_get_token_sync_fn)(void *
  * Synchronous (for now) callback function to fetch a token used in modifying CONNECT request.  Includes a (byte string)
  * context intended to be used as part of a challenge-response flow.
  */
-typedef struct aws_string *(aws_http_proxy_negotiation_get_challenge_token_sync_fn)(
-    void *user_data,
-    const struct aws_byte_cursor *challenge_context,
-    int *out_error_code);
+typedef struct aws_string *(
+    aws_http_proxy_negotiation_get_challenge_token_sync_fn)(void *user_data,
+                                                            const struct aws_byte_cursor *challenge_context,
+                                                            int *out_error_code);
 
 /**
  * Proxy negotiation logic must call this function to indicate an unsuccessful outcome
@@ -307,9 +307,9 @@ struct aws_http_proxy_negotiator {
 
 /*********************************************************************************************/
 
-typedef struct aws_http_proxy_negotiator *(aws_http_proxy_strategy_create_negotiator_fn)(
-    struct aws_http_proxy_strategy *proxy_strategy,
-    struct aws_allocator *allocator);
+typedef struct aws_http_proxy_negotiator *(
+    aws_http_proxy_strategy_create_negotiator_fn)(struct aws_http_proxy_strategy *proxy_strategy,
+                                                  struct aws_allocator *allocator);
 
 struct aws_http_proxy_strategy_vtable {
     aws_http_proxy_strategy_create_negotiator_fn *create_negotiator;
