@@ -144,7 +144,7 @@ struct aws_event_loop_vtable s_iocp_vtable = {
     .free_io_event_resources = s_free_io_event_resources,
 };
 
-struct aws_event_loop *aws_event_loop_new_default_with_options(
+struct aws_event_loop *aws_event_loop_new_with_iocp(
     struct aws_allocator *alloc,
     const struct aws_event_loop_options *options) {
     AWS_ASSERT(alloc);
