@@ -4,6 +4,7 @@
  */
 #include <aws/io/channel_bootstrap.h>
 #include <aws/io/event_loop.h>
+#include <aws/io/private/event_loop_impl.h>
 #include <aws/io/socket.h>
 #include <aws/io/socket_channel_handler.h>
 #include <aws/io/statistics.h>
@@ -17,6 +18,13 @@
 #include "statistics_handler_test.h"
 #include <read_write_test_handler.h>
 
+#ifdef _MSC_VER
+#    pragma warning(disable : 4996) /* allow strncpy() */
+#endif
+
+#define NANOS_PER_SEC ((uint64_t)AWS_TIMESTAMP_NANOS)
+#define TIMEOUT (10 * NANOS_PER_SEC)
+
 struct socket_test_args {
     struct aws_allocator *allocator;
     struct aws_mutex *mutex;
@@ -24,7 +32,7 @@ struct socket_test_args {
     struct aws_channel *channel;
     struct aws_channel_handler *rw_handler;
 
-    struct aws_atomic_var rw_slot; /* pointer-to struct aws_channel_slot */
+    struct aws_channel_slot *rw_slot;
     int error_code;
     bool shutdown_invoked;
     bool error_invoked;
@@ -37,6 +45,7 @@ struct socket_common_tester {
     struct aws_mutex mutex;
     struct aws_condition_variable condition_variable;
     struct aws_event_loop_group *el_group;
+    struct aws_host_resolver *resolver;
     struct aws_atomic_var current_time_ns;
     struct aws_atomic_var stats_handler;
 
@@ -51,7 +60,17 @@ static int s_socket_common_tester_init(struct aws_allocator *allocator, struct s
     AWS_ZERO_STRUCT(*tester);
     aws_io_library_init(allocator);
 
-    tester->el_group = aws_event_loop_group_new_default(allocator, 0, NULL);
+    struct aws_event_loop_group_options elg_options = {
+        .loop_count = 0,
+    };
+    tester->el_group = aws_event_loop_group_new(allocator, &elg_options);
+
+    struct aws_host_resolver_default_options resolver_options = {
+        .el_group = tester->el_group,
+        .max_entries = 8,
+    };
+    tester->resolver = aws_host_resolver_new_default(allocator, &resolver_options);
+
     struct aws_mutex mutex = AWS_MUTEX_INIT;
     struct aws_condition_variable condition_variable = AWS_CONDITION_VARIABLE_INIT;
     tester->mutex = mutex;
@@ -63,6 +82,7 @@ static int s_socket_common_tester_init(struct aws_allocator *allocator, struct s
 }
 
 static int s_socket_common_tester_clean_up(struct socket_common_tester *tester) {
+    aws_host_resolver_release(tester->resolver);
     aws_event_loop_group_release(tester->el_group);
 
     aws_mutex_clean_up(&tester->mutex);
@@ -87,7 +107,7 @@ static bool s_pinned_channel_setup_predicate(void *user_data) {
 
 static bool s_channel_setup_predicate(void *user_data) {
     struct socket_test_args *setup_test_args = (struct socket_test_args *)user_data;
-    return aws_atomic_load_ptr(&setup_test_args->rw_slot) != NULL;
+    return setup_test_args->rw_slot != NULL;
 }
 
 static bool s_channel_shutdown_predicate(void *user_data) {
@@ -120,7 +140,7 @@ static void s_socket_handler_test_client_setup_callback(
     aws_channel_slot_insert_end(channel, rw_slot);
 
     aws_channel_slot_set_handler(rw_slot, setup_test_args->rw_handler);
-    aws_atomic_store_ptr(&setup_test_args->rw_slot, rw_slot);
+    setup_test_args->rw_slot = rw_slot;
 
     aws_mutex_unlock(setup_test_args->mutex);
 
@@ -147,7 +167,7 @@ static void s_socket_handler_test_server_setup_callback(
         aws_channel_slot_insert_end(channel, rw_slot);
 
         aws_channel_slot_set_handler(rw_slot, setup_test_args->rw_handler);
-        aws_atomic_store_ptr(&setup_test_args->rw_slot, rw_slot);
+        setup_test_args->rw_slot = rw_slot;
     }
 
     aws_mutex_unlock(setup_test_args->mutex);
@@ -198,6 +218,7 @@ struct socket_test_rw_args {
     struct aws_byte_buf received_message;
     size_t amount_read;
     size_t expected_read;
+    size_t amount_written;
     bool invocation_happened;
     bool shutdown_finished;
 };
@@ -224,8 +245,7 @@ static struct aws_byte_buf s_socket_test_handle_read(
     struct socket_test_rw_args *rw_args = (struct socket_test_rw_args *)user_data;
 
     aws_mutex_lock(rw_args->mutex);
-    memcpy(rw_args->received_message.buffer + rw_args->received_message.len, data_read->buffer, data_read->len);
-    rw_args->received_message.len += data_read->len;
+    AWS_FATAL_ASSERT(aws_byte_buf_write_from_whole_buffer(&rw_args->received_message, *data_read) == true);
     rw_args->amount_read += data_read->len;
     rw_args->invocation_happened = true;
     aws_condition_variable_notify_one(rw_args->condition_variable);
@@ -234,6 +254,23 @@ static struct aws_byte_buf s_socket_test_handle_read(
     return rw_args->received_message;
 }
 
+void s_socket_test_handle_on_write_completed(
+    struct aws_channel *channel,
+    struct aws_io_message *message,
+    int error_code,
+    void *user_data) {
+
+    (void)channel;
+    AWS_FATAL_ASSERT(error_code == 0);
+    struct socket_test_rw_args *rw_args = (struct socket_test_rw_args *)user_data;
+
+    aws_mutex_lock(rw_args->mutex);
+    rw_args->amount_written += message->message_data.len;
+    rw_args->invocation_happened = true;
+    aws_condition_variable_notify_one(rw_args->condition_variable);
+    aws_mutex_unlock(rw_args->mutex);
+}
+
 static struct aws_byte_buf s_socket_test_handle_write(
     struct aws_channel_handler *handler,
     struct aws_channel_slot *slot,
@@ -291,13 +328,27 @@ static int s_local_server_tester_init(
     struct local_server_tester *tester,
     struct socket_test_args *args,
     struct socket_common_tester *s_c_tester,
+    enum aws_socket_domain socket_domain,
     bool enable_back_pressure) {
+
     AWS_ZERO_STRUCT(*tester);
     tester->socket_options.connect_timeout_ms = 3000;
     tester->socket_options.type = AWS_SOCKET_STREAM;
-    tester->socket_options.domain = AWS_SOCKET_LOCAL;
-
-    aws_socket_endpoint_init_local_address_for_test(&tester->endpoint);
+    tester->socket_options.domain = socket_domain;
+    switch (socket_domain) {
+        case AWS_SOCKET_LOCAL:
+            aws_socket_endpoint_init_local_address_for_test(&tester->endpoint);
+            break;
+        case AWS_SOCKET_IPV4:
+            strncpy(tester->endpoint.address, "127.0.0.1", sizeof(tester->endpoint.address));
+            break;
+        case AWS_SOCKET_IPV6:
+            strncpy(tester->endpoint.address, "::1", sizeof(tester->endpoint.address));
+            break;
+        default:
+            ASSERT_TRUE(false);
+            break;
+    }
 
     tester->server_bootstrap = aws_server_bootstrap_new(allocator, s_c_tester->el_group);
     ASSERT_NOT_NULL(tester->server_bootstrap);
@@ -316,6 +367,9 @@ static int s_local_server_tester_init(
     tester->listener = aws_server_bootstrap_new_socket_listener(&bootstrap_options);
     ASSERT_NOT_NULL(tester->listener);
 
+    /* find out which port the socket is bound to */
+    ASSERT_SUCCESS(aws_socket_get_bound_address(tester->listener, &tester->endpoint));
+
     return AWS_OP_SUCCESS;
 }
 
@@ -344,11 +398,12 @@ static int s_socket_pinned_event_loop_test(struct aws_allocator *allocator, void
     ASSERT_SUCCESS(s_socket_test_args_init(&client_args, &c_tester, client_rw_handler));
 
     struct local_server_tester local_server_tester;
-    ASSERT_SUCCESS(s_local_server_tester_init(allocator, &local_server_tester, &server_args, &c_tester, true));
+    ASSERT_SUCCESS(
+        s_local_server_tester_init(allocator, &local_server_tester, &server_args, &c_tester, AWS_SOCKET_LOCAL, true));
 
     struct aws_client_bootstrap_options client_bootstrap_options = {
         .event_loop_group = c_tester.el_group,
-        .host_resolver = NULL,
+        .host_resolver = c_tester.resolver,
     };
     struct aws_client_bootstrap *client_bootstrap = aws_client_bootstrap_new(allocator, &client_bootstrap_options);
     ASSERT_NOT_NULL(client_bootstrap);
@@ -359,7 +414,7 @@ static int s_socket_pinned_event_loop_test(struct aws_allocator *allocator, void
     AWS_ZERO_STRUCT(client_channel_options);
     client_channel_options.bootstrap = client_bootstrap;
     client_channel_options.host_name = local_server_tester.endpoint.address;
-    client_channel_options.port = 0;
+    client_channel_options.port = local_server_tester.endpoint.port;
     client_channel_options.socket_options = &local_server_tester.socket_options;
     client_channel_options.setup_callback = s_socket_handler_test_client_setup_callback;
     client_channel_options.shutdown_callback = s_socket_handler_test_client_shutdown_callback;
@@ -371,10 +426,10 @@ static int s_socket_pinned_event_loop_test(struct aws_allocator *allocator, void
 
     ASSERT_SUCCESS(aws_mutex_lock(&c_tester.mutex));
     /* wait for both ends to setup */
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_pinned_channel_setup_predicate, &server_args));
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_pinned_channel_setup_predicate, &client_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_pinned_channel_setup_predicate, &server_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_pinned_channel_setup_predicate, &client_args));
 
     /* Verify the client channel was placed on the requested event loop */
     ASSERT_PTR_EQUALS(pinned_event_loop, aws_channel_get_event_loop(client_args.channel));
@@ -382,13 +437,13 @@ static int s_socket_pinned_event_loop_test(struct aws_allocator *allocator, void
     ASSERT_SUCCESS(aws_channel_shutdown(server_args.channel, AWS_OP_SUCCESS));
     ASSERT_SUCCESS(aws_channel_shutdown(client_args.channel, AWS_OP_SUCCESS));
 
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_channel_shutdown_predicate, &server_args));
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_channel_shutdown_predicate, &client_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_channel_shutdown_predicate, &server_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_channel_shutdown_predicate, &client_args));
     aws_server_bootstrap_destroy_socket_listener(local_server_tester.server_bootstrap, local_server_tester.listener);
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_listener_destroy_predicate, &server_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_listener_destroy_predicate, &server_args));
 
     aws_mutex_unlock(&c_tester.mutex);
 
@@ -450,15 +505,9 @@ static int s_socket_pinned_event_loop_dns_failure_test(struct aws_allocator *all
 
     s_socket_common_tester_init(allocator, &c_tester);
 
-    struct aws_host_resolver_default_options resolver_options = {
-        .el_group = c_tester.el_group,
-        .max_entries = 8,
-    };
-    struct aws_host_resolver *resolver = aws_host_resolver_new_default(allocator, &resolver_options);
-
     struct aws_client_bootstrap_options client_bootstrap_options = {
         .event_loop_group = c_tester.el_group,
-        .host_resolver = resolver,
+        .host_resolver = c_tester.resolver,
     };
     struct aws_client_bootstrap *client_bootstrap = aws_client_bootstrap_new(allocator, &client_bootstrap_options);
     ASSERT_NOT_NULL(client_bootstrap);
@@ -487,8 +536,8 @@ static int s_socket_pinned_event_loop_dns_failure_test(struct aws_allocator *all
     ASSERT_SUCCESS(aws_client_bootstrap_new_socket_channel(&client_channel_options));
 
     ASSERT_SUCCESS(aws_mutex_lock(&c_tester.mutex));
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_dns_failure_channel_setup_predicate, &c_tester));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_dns_failure_channel_setup_predicate, &c_tester));
 
     /* Verify the setup callback failure was on the requested event loop */
     ASSERT_TRUE(c_tester.setup_error_code != 0);
@@ -496,7 +545,6 @@ static int s_socket_pinned_event_loop_dns_failure_test(struct aws_allocator *all
     aws_mutex_unlock(&c_tester.mutex);
 
     aws_client_bootstrap_release(client_bootstrap);
-    aws_host_resolver_release(resolver);
     ASSERT_SUCCESS(s_socket_common_tester_clean_up(&c_tester));
 
     return AWS_OP_SUCCESS;
@@ -556,11 +604,12 @@ static int s_socket_echo_and_backpressure_test(struct aws_allocator *allocator,
     ASSERT_SUCCESS(s_socket_test_args_init(&client_args, &c_tester, client_rw_handler));
 
     struct local_server_tester local_server_tester;
-    ASSERT_SUCCESS(s_local_server_tester_init(allocator, &local_server_tester, &server_args, &c_tester, true));
+    ASSERT_SUCCESS(
+        s_local_server_tester_init(allocator, &local_server_tester, &server_args, &c_tester, AWS_SOCKET_LOCAL, true));
 
     struct aws_client_bootstrap_options client_bootstrap_options = {
         .event_loop_group = c_tester.el_group,
-        .host_resolver = NULL,
+        .host_resolver = c_tester.resolver,
     };
     struct aws_client_bootstrap *client_bootstrap = aws_client_bootstrap_new(allocator, &client_bootstrap_options);
     ASSERT_NOT_NULL(client_bootstrap);
@@ -569,7 +618,7 @@ static int s_socket_echo_and_backpressure_test(struct aws_allocator *allocator,
     AWS_ZERO_STRUCT(client_channel_options);
     client_channel_options.bootstrap = client_bootstrap;
     client_channel_options.host_name = local_server_tester.endpoint.address;
-    client_channel_options.port = 0;
+    client_channel_options.port = local_server_tester.endpoint.port;
     client_channel_options.socket_options = &local_server_tester.socket_options;
     client_channel_options.setup_callback = s_socket_handler_test_client_setup_callback;
     client_channel_options.shutdown_callback = s_socket_handler_test_client_shutdown_callback;
@@ -581,20 +630,20 @@ static int s_socket_echo_and_backpressure_test(struct aws_allocator *allocator,
     ASSERT_SUCCESS(aws_mutex_lock(&c_tester.mutex));
 
     /* wait for both ends to setup */
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_channel_setup_predicate, &server_args));
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_channel_setup_predicate, &client_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_channel_setup_predicate, &server_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_channel_setup_predicate, &client_args));
 
     /* send msg from client to server, and wait for some bytes to be received */
-    rw_handler_write(client_args.rw_handler, aws_atomic_load_ptr(&client_args.rw_slot), &msg_from_client);
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_socket_test_read_predicate, &server_rw_args));
+    rw_handler_write(client_args.rw_handler, client_args.rw_slot, &msg_from_client);
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_socket_test_read_predicate, &server_rw_args));
 
     /* send msg from server to client, and wait for some bytes to be received */
-    rw_handler_write(server_args.rw_handler, aws_atomic_load_ptr(&server_args.rw_slot), &msg_from_server);
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_socket_test_read_predicate, &client_rw_args));
+    rw_handler_write(server_args.rw_handler, server_args.rw_slot, &msg_from_server);
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_socket_test_read_predicate, &client_rw_args));
 
     /* confirm that the initial read window was respected */
     server_rw_args.invocation_happened = false;
@@ -604,13 +653,13 @@ static int s_socket_echo_and_backpressure_test(struct aws_allocator *allocator,
     ASSERT_INT_EQUALS(s_server_initial_read_window, server_rw_args.amount_read);
 
     /* increment the read window on both sides and confirm they receive the remainder of their message */
-    rw_handler_trigger_increment_read_window(server_args.rw_handler, aws_atomic_load_ptr(&server_args.rw_slot), 100);
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_socket_test_full_read_predicate, &server_rw_args));
+    rw_handler_trigger_increment_read_window(server_args.rw_handler, server_args.rw_slot, 100);
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_socket_test_full_read_predicate, &server_rw_args));
 
-    rw_handler_trigger_increment_read_window(client_args.rw_handler, aws_atomic_load_ptr(&client_args.rw_slot), 100);
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_socket_test_full_read_predicate, &client_rw_args));
+    rw_handler_trigger_increment_read_window(client_args.rw_handler, client_args.rw_slot, 100);
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_socket_test_full_read_predicate, &client_rw_args));
 
     ASSERT_INT_EQUALS(msg_from_server.len, client_rw_args.amount_read);
     ASSERT_INT_EQUALS(msg_from_client.len, server_rw_args.amount_read);
@@ -630,13 +679,13 @@ static int s_socket_echo_and_backpressure_test(struct aws_allocator *allocator,
     ASSERT_SUCCESS(aws_channel_shutdown(server_args.channel, AWS_OP_SUCCESS));
     ASSERT_SUCCESS(aws_channel_shutdown(client_args.channel, AWS_OP_SUCCESS));
 
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_channel_shutdown_predicate, &server_args));
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_channel_shutdown_predicate, &client_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_channel_shutdown_predicate, &server_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_channel_shutdown_predicate, &client_args));
     aws_server_bootstrap_destroy_socket_listener(local_server_tester.server_bootstrap, local_server_tester.listener);
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_listener_destroy_predicate, &server_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_listener_destroy_predicate, &server_args));
 
     aws_mutex_unlock(&c_tester.mutex);
 
@@ -687,11 +736,12 @@ static int s_socket_close_test(struct aws_allocator *allocator, void *ctx) {
     ASSERT_SUCCESS(s_socket_test_args_init(&client_args, &c_tester, client_rw_handler));
 
     struct local_server_tester local_server_tester;
-    ASSERT_SUCCESS(s_local_server_tester_init(allocator, &local_server_tester, &server_args, &c_tester, false));
+    ASSERT_SUCCESS(
+        s_local_server_tester_init(allocator, &local_server_tester, &server_args, &c_tester, AWS_SOCKET_LOCAL, false));
 
     struct aws_client_bootstrap_options client_bootstrap_options = {
         .event_loop_group = c_tester.el_group,
-        .host_resolver = NULL,
+        .host_resolver = c_tester.resolver,
     };
     struct aws_client_bootstrap *client_bootstrap = aws_client_bootstrap_new(allocator, &client_bootstrap_options);
     ASSERT_NOT_NULL(client_bootstrap);
@@ -700,7 +750,7 @@ static int s_socket_close_test(struct aws_allocator *allocator, void *ctx) {
     AWS_ZERO_STRUCT(client_channel_options);
     client_channel_options.bootstrap = client_bootstrap;
     client_channel_options.host_name = local_server_tester.endpoint.address;
-    client_channel_options.port = 0;
+    client_channel_options.port = local_server_tester.endpoint.port;
     client_channel_options.socket_options = &local_server_tester.socket_options;
     client_channel_options.setup_callback = s_socket_handler_test_client_setup_callback;
     client_channel_options.shutdown_callback = s_socket_handler_test_client_shutdown_callback;
@@ -711,24 +761,24 @@ static int s_socket_close_test(struct aws_allocator *allocator, void *ctx) {
     ASSERT_SUCCESS(aws_mutex_lock(&c_tester.mutex));
 
     /* wait for both ends to setup */
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_channel_setup_predicate, &server_args));
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_channel_setup_predicate, &client_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_channel_setup_predicate, &server_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_channel_setup_predicate, &client_args));
 
     aws_channel_shutdown(server_args.channel, AWS_OP_SUCCESS);
 
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_channel_shutdown_predicate, &server_args));
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_channel_shutdown_predicate, &client_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_channel_shutdown_predicate, &server_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_channel_shutdown_predicate, &client_args));
 
     ASSERT_INT_EQUALS(AWS_OP_SUCCESS, server_args.error_code);
     ASSERT_TRUE(
         AWS_IO_SOCKET_CLOSED == client_args.error_code || AWS_IO_SOCKET_NOT_CONNECTED == client_args.error_code);
     aws_server_bootstrap_destroy_socket_listener(local_server_tester.server_bootstrap, local_server_tester.listener);
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_listener_destroy_predicate, &server_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_listener_destroy_predicate, &server_args));
 
     aws_mutex_unlock(&c_tester.mutex);
 
@@ -743,6 +793,182 @@ static int s_socket_close_test(struct aws_allocator *allocator, void *ctx) {
 
 AWS_TEST_CASE(socket_handler_close, s_socket_close_test)
 
+/* This is a regression test.
+ * Once upon a time, if the socket-handler received READABLE and HANGUP events simultaneously,
+ * it would read one last time from the socket before closing it. But one read may
+ * not be enough to get all remaining data. The correct thing is to do is
+ * repeatedly read until the read() call itself reports EOF or an error.
+ *
+ * Anyway, this test establishes a connection between server and client.
+ * The server sends a big chunk of data, and closes the socket immediately
+ * after the write completes. The client should still be able to read all the data. */
+static int s_socket_read_to_eof_after_peer_hangup_test_common(
+    struct aws_allocator *allocator,
+    void *ctx,
+    enum aws_socket_domain socket_domain) {
+
+    (void)ctx;
+    s_socket_common_tester_init(allocator, &c_tester);
+
+    const size_t total_bytes_to_send_from_server = g_aws_channel_max_fragment_size;
+
+    struct aws_byte_buf client_received_message;
+    ASSERT_SUCCESS(aws_byte_buf_init(&client_received_message, allocator, total_bytes_to_send_from_server));
+
+    struct aws_byte_buf msg_from_server;
+    ASSERT_SUCCESS(aws_byte_buf_init(&msg_from_server, allocator, total_bytes_to_send_from_server));
+
+    struct socket_test_rw_args server_rw_args;
+    ASSERT_SUCCESS(s_rw_args_init(&server_rw_args, &c_tester, aws_byte_buf_from_empty_array(NULL, 0), 0));
+
+    struct socket_test_rw_args client_rw_args;
+    ASSERT_SUCCESS(s_rw_args_init(&client_rw_args, &c_tester, client_received_message, 0));
+
+    /* NOTE: client starts with window=0, so we can VERY CAREFULLY control when it reads data from the socket */
+    struct aws_channel_handler *client_rw_handler = rw_handler_new(
+        allocator, s_socket_test_handle_read, s_socket_test_handle_write, true, 0 /*window*/, &client_rw_args);
+    ASSERT_NOT_NULL(client_rw_handler);
+
+    struct aws_channel_handler *server_rw_handler =
+        rw_handler_new(allocator, s_socket_test_handle_read, s_socket_test_handle_write, true, 10000, &server_rw_args);
+    ASSERT_NOT_NULL(server_rw_handler);
+
+    struct socket_test_args server_args;
+    ASSERT_SUCCESS(s_socket_test_args_init(&server_args, &c_tester, server_rw_handler));
+
+    struct socket_test_args client_args;
+    ASSERT_SUCCESS(s_socket_test_args_init(&client_args, &c_tester, client_rw_handler));
+
+    struct local_server_tester local_server_tester;
+    if (s_local_server_tester_init(allocator, &local_server_tester, &server_args, &c_tester, socket_domain, false)) {
+        /* Skip test if server can't bind to address (e.g. Codebuild's ubuntu runners don't allow IPv6) */
+        if (aws_last_error() == AWS_IO_SOCKET_INVALID_ADDRESS) {
+            return AWS_OP_SKIP;
+        } else {
+            ASSERT_TRUE(false, "s_local_server_tester_init() failed");
+        }
+    }
+
+    struct aws_client_bootstrap_options client_bootstrap_options = {
+        .event_loop_group = c_tester.el_group,
+        .host_resolver = c_tester.resolver,
+    };
+    struct aws_client_bootstrap *client_bootstrap = aws_client_bootstrap_new(allocator, &client_bootstrap_options);
+    ASSERT_NOT_NULL(client_bootstrap);
+
+    struct aws_socket_channel_bootstrap_options client_channel_options = {
+        .bootstrap = client_bootstrap,
+        .host_name = local_server_tester.endpoint.address,
+        .port = local_server_tester.endpoint.port,
+        .socket_options = &local_server_tester.socket_options,
+        .setup_callback = s_socket_handler_test_client_setup_callback,
+        .shutdown_callback = s_socket_handler_test_client_shutdown_callback,
+        .user_data = &client_args,
+        .enable_read_back_pressure = true,
+    };
+
+    ASSERT_SUCCESS(aws_client_bootstrap_new_socket_channel(&client_channel_options));
+
+    ASSERT_SUCCESS(aws_mutex_lock(&c_tester.mutex));
+
+    /* wait for both ends to setup */
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_channel_setup_predicate, &server_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_channel_setup_predicate, &client_args));
+
+    /* We want the server to send some data and hang up IMMEDIATELY after,
+     * before the client has fully read the data. This is tricky to do in a test.
+     *
+     * First, have the server send data... */
+    ASSERT_TRUE(aws_byte_buf_write_u8_n(&msg_from_server, 's', total_bytes_to_send_from_server));
+    rw_handler_write_with_callback(
+        server_rw_handler,
+        server_args.rw_slot,
+        &msg_from_server,
+        s_socket_test_handle_on_write_completed,
+        &server_rw_args);
+
+    /* ...now have the client open its read window and receive data in tiny chunks,
+     * stopping once the server has sent all data, but BEFORE the client has read all data.
+     * This is possible because the client's OS will buffer a certain amount of
+     * incoming data, before the client application calls read() on it. */
+    while (server_rw_args.amount_written < total_bytes_to_send_from_server) {
+        const size_t client_read_chunk_size = 128;
+        client_rw_args.expected_read += client_read_chunk_size;
+        rw_handler_trigger_increment_read_window(client_args.rw_handler, client_args.rw_slot, client_read_chunk_size);
+        ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+            &c_tester.condition_variable,
+            &c_tester.mutex,
+            TIMEOUT,
+            s_socket_test_full_read_predicate,
+            &client_rw_args));
+    }
+
+    /* Now close the server's socket.*/
+    ASSERT_SUCCESS(aws_channel_shutdown(server_args.channel, AWS_OP_SUCCESS));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_channel_shutdown_predicate, &server_args));
+
+    /* Now sleep a moment to 100% guarantee the OS propagates the socket-close event to the client-side. */
+    aws_mutex_unlock(&c_tester.mutex);
+    aws_thread_current_sleep(NANOS_PER_SEC / 4);
+    aws_mutex_lock(&c_tester.mutex);
+
+    /* Ensure the client hasn't shut down before reading all the data. */
+    ASSERT_FALSE(client_args.shutdown_invoked, "Client should read all data before shutting down.");
+
+    /* Ensure the client hasn't read all data yet */
+    ASSERT_TRUE(
+        client_rw_args.amount_read < total_bytes_to_send_from_server,
+        "If this fails, then we're not truly reproducing the regression test."
+        " The server needs to finish sending data, and close the socket,"
+        " BEFORE the client reads all the data.");
+
+    /* Have the client open its window more-than-enough to receive the rest of the data.
+     * If the client socket closes before all the data is received, then we still have the bug. */
+    rw_handler_trigger_increment_read_window(
+        client_args.rw_handler, client_args.rw_slot, total_bytes_to_send_from_server * 3 /*more-than-enough*/);
+    client_rw_args.expected_read = total_bytes_to_send_from_server;
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_socket_test_full_read_predicate, &client_rw_args));
+
+    /* Wait for client to shutdown, due to the server having closed the socket */
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_channel_shutdown_predicate, &client_args));
+
+    aws_server_bootstrap_destroy_socket_listener(local_server_tester.server_bootstrap, local_server_tester.listener);
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_listener_destroy_predicate, &server_args));
+
+    ASSERT_INT_EQUALS(AWS_IO_SOCKET_CLOSED, client_args.error_code);
+
+    aws_mutex_unlock(&c_tester.mutex);
+
+    /* clean up */
+    ASSERT_SUCCESS(s_local_server_tester_clean_up(&local_server_tester));
+    aws_byte_buf_clean_up(&client_received_message);
+    aws_byte_buf_clean_up(&msg_from_server);
+    aws_client_bootstrap_release(client_bootstrap);
+    ASSERT_SUCCESS(s_socket_common_tester_clean_up(&c_tester));
+
+    return AWS_OP_SUCCESS;
+}
+static int s_socket_read_to_eof_after_peer_hangup_test(struct aws_allocator *allocator, void *ctx) {
+    return s_socket_read_to_eof_after_peer_hangup_test_common(allocator, ctx, AWS_SOCKET_LOCAL);
+}
+AWS_TEST_CASE(socket_handler_read_to_eof_after_peer_hangup, s_socket_read_to_eof_after_peer_hangup_test)
+
+static int s_socket_ipv4_read_to_eof_after_peer_hangup_test(struct aws_allocator *allocator, void *ctx) {
+    return s_socket_read_to_eof_after_peer_hangup_test_common(allocator, ctx, AWS_SOCKET_IPV4);
+}
+AWS_TEST_CASE(socket_handler_ipv4_read_to_eof_after_peer_hangup, s_socket_ipv4_read_to_eof_after_peer_hangup_test)
+
+static int s_socket_ipv6_read_to_eof_after_peer_hangup_test(struct aws_allocator *allocator, void *ctx) {
+    return s_socket_read_to_eof_after_peer_hangup_test_common(allocator, ctx, AWS_SOCKET_IPV6);
+}
+AWS_TEST_CASE(socket_handler_ipv6_read_to_eof_after_peer_hangup, s_socket_ipv6_read_to_eof_after_peer_hangup_test)
+
 static void s_creation_callback_test_channel_creation_callback(
     struct aws_client_bootstrap *bootstrap,
     int error_code,
@@ -768,7 +994,7 @@ static struct aws_event_loop *s_default_new_event_loop(
     void *user_data) {
 
     (void)user_data;
-    return aws_event_loop_new_default_with_options(allocator, options);
+    return aws_event_loop_new(allocator, options);
 }
 
 static int s_statistic_test_clock_fn(uint64_t *timestamp) {
@@ -784,8 +1010,13 @@ static int s_socket_common_tester_statistics_init(
     aws_io_library_init(allocator);
 
     AWS_ZERO_STRUCT(*tester);
-    tester->el_group =
-        aws_event_loop_group_new(allocator, s_statistic_test_clock_fn, 1, s_default_new_event_loop, NULL, NULL);
+
+    struct aws_event_loop_group_options elg_options = {
+        .loop_count = 1,
+        .clock_override = s_statistic_test_clock_fn,
+    };
+    tester->el_group = aws_event_loop_group_new_internal(allocator, &elg_options, s_default_new_event_loop, NULL);
+
     struct aws_mutex mutex = AWS_MUTEX_INIT;
     struct aws_condition_variable condition_variable = AWS_CONDITION_VARIABLE_INIT;
     tester->mutex = mutex;
@@ -843,12 +1074,13 @@ static int s_open_channel_statistics_test(struct aws_allocator *allocator, void
     ASSERT_SUCCESS(s_socket_test_args_init(&client_args, &c_tester, client_rw_handler));
 
     struct local_server_tester local_server_tester;
-    ASSERT_SUCCESS(s_local_server_tester_init(allocator, &local_server_tester, &server_args, &c_tester, false));
+    ASSERT_SUCCESS(
+        s_local_server_tester_init(allocator, &local_server_tester, &server_args, &c_tester, AWS_SOCKET_LOCAL, false));
 
     struct aws_client_bootstrap_options client_bootstrap_options;
     AWS_ZERO_STRUCT(client_bootstrap_options);
     client_bootstrap_options.event_loop_group = c_tester.el_group;
-    client_bootstrap_options.host_resolver = NULL;
+    client_bootstrap_options.host_resolver = c_tester.resolver;
 
     struct aws_client_bootstrap *client_bootstrap = aws_client_bootstrap_new(allocator, &client_bootstrap_options);
     ASSERT_NOT_NULL(client_bootstrap);
@@ -857,7 +1089,7 @@ static int s_open_channel_statistics_test(struct aws_allocator *allocator, void
     AWS_ZERO_STRUCT(client_channel_options);
     client_channel_options.bootstrap = client_bootstrap;
     client_channel_options.host_name = local_server_tester.endpoint.address;
-    client_channel_options.port = 0;
+    client_channel_options.port = local_server_tester.endpoint.port;
     client_channel_options.socket_options = &local_server_tester.socket_options;
     client_channel_options.creation_callback = s_creation_callback_test_channel_creation_callback;
     client_channel_options.setup_callback = s_socket_handler_test_client_setup_callback;
@@ -869,22 +1101,20 @@ static int s_open_channel_statistics_test(struct aws_allocator *allocator, void
     ASSERT_SUCCESS(aws_mutex_lock(&c_tester.mutex));
 
     /* wait for both ends to setup */
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_channel_setup_predicate, &server_args));
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_channel_setup_predicate, &client_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_channel_setup_predicate, &server_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_channel_setup_predicate, &client_args));
 
     ASSERT_TRUE(client_args.creation_callback_invoked);
 
-    struct aws_channel_slot *client_rw_slot = aws_atomic_load_ptr(&client_args.rw_slot);
-    rw_handler_write(client_args.rw_handler, client_rw_slot, &msg_from_client);
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_socket_test_read_predicate, &server_rw_args));
+    rw_handler_write(client_args.rw_handler, client_args.rw_slot, &msg_from_client);
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_socket_test_read_predicate, &server_rw_args));
 
-    struct aws_channel_slot *server_rw_slot = aws_atomic_load_ptr(&server_args.rw_slot);
-    rw_handler_write(server_args.rw_handler, server_rw_slot, &msg_from_server);
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_socket_test_read_predicate, &client_rw_args));
+    rw_handler_write(server_args.rw_handler, server_args.rw_slot, &msg_from_server);
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_socket_test_read_predicate, &client_rw_args));
 
     uint64_t ms_to_ns = aws_timestamp_convert(1, AWS_TIMESTAMP_MILLIS, AWS_TIMESTAMP_NANOS, NULL);
 
@@ -894,8 +1124,8 @@ static int s_open_channel_statistics_test(struct aws_allocator *allocator, void
     struct aws_statistics_handler_test_impl *stats_impl = stats_handler->impl;
 
     aws_mutex_lock(&stats_impl->lock);
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &stats_impl->signal, &stats_impl->lock, s_stats_processed_predicate, stats_handler));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &stats_impl->signal, &stats_impl->lock, TIMEOUT, s_stats_processed_predicate, stats_handler));
 
     ASSERT_TRUE(stats_impl->total_bytes_read == msg_from_server.len);
     ASSERT_TRUE(stats_impl->total_bytes_written == msg_from_client.len);
@@ -904,14 +1134,14 @@ static int s_open_channel_statistics_test(struct aws_allocator *allocator, void
 
     aws_channel_shutdown(server_args.channel, AWS_OP_SUCCESS);
 
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_channel_shutdown_predicate, &server_args));
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_channel_shutdown_predicate, &client_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_channel_shutdown_predicate, &server_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_channel_shutdown_predicate, &client_args));
 
     aws_server_bootstrap_destroy_socket_listener(local_server_tester.server_bootstrap, local_server_tester.listener);
-    ASSERT_SUCCESS(aws_condition_variable_wait_pred(
-        &c_tester.condition_variable, &c_tester.mutex, s_listener_destroy_predicate, &server_args));
+    ASSERT_SUCCESS(aws_condition_variable_wait_for_pred(
+        &c_tester.condition_variable, &c_tester.mutex, TIMEOUT, s_listener_destroy_predicate, &server_args));
 
     aws_mutex_unlock(&c_tester.mutex);
 
