@@ -94,6 +94,10 @@ struct aws_s3_endpoint_options {
      * If the transfer speed falls below the specified minimum_throughput_bytes_per_second, the operation is aborted.
      */
     struct aws_http_connection_monitoring_options *monitoring_options;
+
+    /* An array of `struct aws_byte_cursor` of network interface names. */
+    const struct aws_byte_cursor *network_interface_names_array;
+    size_t num_network_interface_names;
 };
 
 /* global vtable, only used when mocking for tests */
@@ -165,6 +169,10 @@ struct aws_s3_client_vtable {
 
     struct aws_parallel_input_stream *(
         *parallel_input_stream_new_from_file)(struct aws_allocator *allocator, struct aws_byte_cursor file_name);
+
+    struct aws_http_stream *(*http_connection_make_request)(
+        struct aws_http_connection *client_connection,
+        const struct aws_http_make_request_options *options);
 };
 
 struct aws_s3_upload_part_timeout_stats {
@@ -242,8 +250,8 @@ struct aws_s3_client {
     /* Throughput target in Gbps that we are trying to reach. */
     const double throughput_target_gbps;
 
-    /* The calculated ideal number of VIP's based on throughput target and throughput per vip. */
-    const uint32_t ideal_vip_count;
+    /* The calculated ideal number of HTTP connections, based on throughput target and throughput per connection. */
+    const uint32_t ideal_connection_count;
 
     /**
      * For multi-part upload, content-md5 will be calculated if the AWS_MR_CONTENT_MD5_ENABLED is specified
@@ -313,6 +321,19 @@ struct aws_s3_client {
      */
     struct aws_atomic_var upload_timeout_ms;
 
+    /*
+     * An aws_array_list<struct aws_string *> of network interface names.
+     */
+    struct aws_array_list network_interface_names;
+    /*
+     * An array of `struct aws_byte_cursor` of network interface names. The cursors are over the strings in
+     * `network_interface_names` so that we can easily pass it to the connection manager without any processing. We need
+     * to create both `struct aws_array_list<struct aws_string *>` and the array of `struct aws_byte_cursor` since byte
+     * cursors are non-owning and we need to ensure that the underlying memory is valid as long as the client lives.
+     */
+    struct aws_byte_cursor *network_interface_names_cursor_array;
+    size_t num_network_interface_names;
+
     struct {
         /* Number of overall requests currently being processed by the client. */
         struct aws_atomic_var num_requests_in_flight;
@@ -350,6 +371,9 @@ struct aws_s3_client {
         /* Task for trimming buffer bool. */
         struct aws_task trim_buffer_pool_task;
 
+        /* Task to cleanup endpoints */
+        struct aws_task endpoints_cleanup_task;
+
         /* Number of endpoints currently allocated. Used during clean up to know how many endpoints are still in
          * memory.*/
         uint32_t num_endpoints_allocated;
@@ -376,6 +400,9 @@ struct aws_s3_client {
         /* True if client has been flagged to finish destroying itself. Used to catch double-destroy bugs.*/
         uint32_t finish_destroy : 1;
 
+        /* Whether or not endpoints cleanup task is currently scheduled. */
+        uint32_t endpoints_cleanup_task_scheduled : 1;
+
         struct aws_s3_upload_part_timeout_stats upload_part_stats;
     } synced_data;
 
@@ -483,11 +510,14 @@ struct aws_s3_endpoint *aws_s3_endpoint_acquire(struct aws_s3_endpoint *endpoint
  * from the client's hashtable) */
 void aws_s3_endpoint_release(struct aws_s3_endpoint *endpoint);
 
-AWS_S3_API
-extern const uint32_t g_max_num_connections_per_vip;
+/*
+ * Destroys the endpoint. Before calling this function, the endpoint must be removed from the Client's hash table, and
+ * its ref count must be zero. You MUST NOT call this while the client's lock is held.
+ */
+void aws_s3_endpoint_destroy(struct aws_s3_endpoint *endpoint);
 
 AWS_S3_API
-extern const uint32_t g_num_conns_per_vip_meta_request_look_up[];
+extern const uint32_t g_min_num_connections;
 
 AWS_S3_API
 extern const size_t g_expect_timeout_offset_ms;
