@@ -143,9 +143,11 @@ struct aws_s3_meta_request {
     /* The meta request's outgoing body comes from one of these:
      * 1) request_body_async_stream: if set, then async stream 1 part at a time
      * 2) request_body_parallel_stream: if set, then stream multiple parts in parallel
-     * 3) initial_request_message's body_stream: else synchronously stream parts */
+     * 3) request_body_using_async_writes: if set, then synchronously copy async_write data from 1 part at a time
+     * 4) initial_request_message's body_stream: else synchronously stream parts */
     struct aws_async_input_stream *request_body_async_stream;
     struct aws_parallel_input_stream *request_body_parallel_stream;
+    bool request_body_using_async_writes;
 
     /* Part size to use for uploads and downloads.  Passed down by the creating client. */
     const size_t part_size;
@@ -174,11 +176,6 @@ struct aws_s3_meta_request {
     aws_s3_meta_request_telemetry_fn *telemetry_callback;
     aws_s3_meta_request_upload_review_fn *upload_review_callback;
 
-    /* Customer specified callbacks to be called by our specialized callback to calculate the response checksum. */
-    aws_s3_meta_request_headers_callback_fn *headers_user_callback_after_checksum;
-    aws_s3_meta_request_receive_body_callback_fn *body_user_callback_after_checksum;
-    aws_s3_meta_request_finish_fn *finish_user_callback_after_checksum;
-
     enum aws_s3_meta_request_type type;
     struct aws_string *s3express_session_host;
 
@@ -229,6 +226,27 @@ struct aws_s3_meta_request {
         /* To track aws_s3_requests with cancellable HTTP streams */
         struct aws_linked_list cancellable_http_streams_list;
 
+        /* Data for async-writes. */
+        struct {
+            /* Whether a part request can be sent (we have 1 part's worth of data, or EOF) */
+            bool ready_to_send;
+
+            /* True once user passes `eof` to their final write() call */
+            bool eof;
+
+            /* Holds buffered data we can't immediately send.
+             * The length will always be less than part-size */
+            struct aws_byte_buf buffered_data;
+            struct aws_s3_buffer_pool_ticket *buffered_data_ticket;
+
+            /* Waker callback.
+             * Stored if a poll_write() call returns result.is_pending
+             * because we already had 1 part's worth of data.
+             * Invoked when we're ready to accept another poll_write() call. */
+            aws_simple_completion_callback *waker;
+            void *waker_user_data;
+        } async_write;
+
     } synced_data;
 
     /* Anything in this structure should only ever be accessed by the client on its process work event loop task. */
@@ -254,13 +272,18 @@ struct aws_s3_meta_request {
     const bool should_compute_content_md5;
 
     /* deep copy of the checksum config. */
-    struct checksum_config checksum_config;
+    struct checksum_config_storage checksum_config;
 
     /* checksum found in either a default get request, or in the initial head request of a multipart get */
     struct aws_byte_buf meta_request_level_response_header_checksum;
 
     /* running checksum of all the parts of a default get, or ranged get meta request*/
     struct aws_s3_checksum *meta_request_level_running_response_sum;
+
+    /* The receiving file handler */
+    FILE *recv_file;
+    struct aws_string *recv_filepath;
+    bool recv_file_delete_on_failure;
 };
 
 /* Info for each part, that we need to remember until we send CompleteMultipartUpload */
@@ -327,6 +350,14 @@ void aws_s3_meta_request_init_signing_date_time_default(
     struct aws_s3_meta_request *meta_request,
     struct aws_date_time *date_time);
 
+AWS_S3_API
+void aws_s3_meta_request_sign_request_default_impl(
+    struct aws_s3_meta_request *meta_request,
+    struct aws_s3_request *request,
+    aws_signing_complete_fn *on_signing_complete,
+    void *user_data,
+    bool disable_s3_express_signing);
+
 AWS_S3_API
 void aws_s3_meta_request_sign_request_default(
     struct aws_s3_meta_request *meta_request,
@@ -385,8 +416,6 @@ struct aws_future_bool *aws_s3_meta_request_read_body(
     uint64_t offset,
     struct aws_byte_buf *buffer);
 
-bool aws_s3_meta_request_body_has_no_more_data(const struct aws_s3_meta_request *meta_request);
-
 /* Set the meta request finish result as failed. This is meant to be called sometime before aws_s3_meta_request_finish.
  * Subsequent calls to this function or to aws_s3_meta_request_set_success_synced will not overwrite the end result of
  * the meta request. */
@@ -436,6 +465,13 @@ bool aws_s3_meta_request_checksum_config_has_algorithm(
     struct aws_s3_meta_request *meta_request,
     enum aws_s3_checksum_algorithm algorithm);
 
+void aws_s3_meta_request_schedule_prepare_request_default_impl(
+    struct aws_s3_meta_request *meta_request,
+    struct aws_s3_request *request,
+    bool parallel,
+    aws_s3_meta_request_prepare_request_callback_fn *callback,
+    void *user_data);
+
 AWS_EXTERN_C_END
 
 #endif /* AWS_S3_META_REQUEST_IMPL_H */
