@@ -82,6 +82,11 @@ struct aws_s3_paginator_params {
  * Parameters for initiating paginated operation. All values are copied out or re-seated and reference counted.
  */
 struct aws_s3_paginated_operation_params {
+    /**
+     * The S3 operation name (e.g. "ListParts"). Must not be empty.
+     */
+    struct aws_byte_cursor operation_name;
+
     /**
      * Name of the top level result node. Must not be empty.
      */
