@@ -11,6 +11,11 @@
 #include <aws/common/string.h>
 #include <inttypes.h>
 
+/* Dont use buffer pool when we know response size, and its below this number,
+ * i.e. when user provides explicit range that is small, ex. range = 1-100.
+ * Instead of going through the pool in that case, we just use a dynamic buffer
+ * for response (pre-mempool behavior). */
+const uint64_t s_min_size_response_for_pooling = 1 * 1024 * 1024;
 const uint32_t s_conservative_max_requests_in_flight = 8;
 const struct aws_byte_cursor g_application_xml_value = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("application/xml");
 
@@ -287,10 +292,20 @@ static bool s_s3_auto_ranged_get_update(
                             AWS_LS_S3_META_REQUEST,
                             "id=%p: Doing a ranged get to discover the size of the object and get the first part",
                             (void *)meta_request);
-                        ticket = aws_s3_buffer_pool_reserve(meta_request->client->buffer_pool, (size_t)first_part_size);
 
-                        if (ticket == NULL) {
-                            goto has_work_remaining;
+                        if (first_part_size >= s_min_size_response_for_pooling) {
+                            /* Note: explicitly reserving the whole part size
+                             * even if expect to receive less data. Pool will
+                             * reserve the whole part size for it anyways, so no
+                             * reason getting a smaller chunk. */
+                            ticket = aws_s3_buffer_pool_reserve(
+                                meta_request->client->buffer_pool, (size_t)meta_request->part_size);
+
+                            if (ticket == NULL) {
+                                goto has_work_remaining;
+                            }
+                        } else {
+                            ticket = NULL;
                         }
 
                         request = aws_s3_request_new(
@@ -794,11 +809,21 @@ static void s_s3_auto_ranged_get_request_finished(
         error_code = AWS_ERROR_SUCCESS;
         found_object_size = true;
 
+        /* Check for checksums if requested to */
+        if (meta_request->checksum_config.validate_response_checksum) {
+            if (aws_s3_check_headers_for_checksum(
+                    meta_request,
+                    request->send_data.response_headers,
+                    &meta_request->meta_request_level_running_response_sum,
+                    &meta_request->meta_request_level_response_header_checksum,
+                    true) != AWS_OP_SUCCESS) {
+                error_code = aws_last_error_or_unknown();
+                goto update_synced_data;
+            }
+        }
+
         if (!empty_file_error && meta_request->headers_callback != NULL) {
-            struct aws_http_headers *response_headers = aws_http_headers_new(meta_request->allocator);
-
-            copy_http_headers(request->send_data.response_headers, response_headers);
-
+            /* Modify the header received to fake the header for the whole meta request. */
             if (request->request_tag == AWS_S3_AUTO_RANGE_GET_REQUEST_TYPE_GET_OBJECT_WITH_RANGE ||
                 request->request_tag == AWS_S3_AUTO_RANGE_GET_REQUEST_TYPE_GET_OBJECT_WITH_PART_NUMBER_1) {
 
@@ -813,12 +838,12 @@ static void s_s3_auto_ranged_get_request_finished(
                         object_range_end,
                         object_size);
                     aws_http_headers_set(
-                        response_headers,
+                        request->send_data.response_headers,
                         g_content_range_header_name,
                         aws_byte_cursor_from_c_str(content_range_buffer));
                 } else {
                     /* content range isn't applicable. */
-                    aws_http_headers_erase(response_headers, g_content_range_header_name);
+                    aws_http_headers_erase(request->send_data.response_headers, g_content_range_header_name);
                 }
             }
 
@@ -826,19 +851,19 @@ static void s_s3_auto_ranged_get_request_finished(
             char content_length_buffer[64] = "";
             snprintf(content_length_buffer, sizeof(content_length_buffer), "%" PRIu64, content_length);
             aws_http_headers_set(
-                response_headers, g_content_length_header_name, aws_byte_cursor_from_c_str(content_length_buffer));
+                request->send_data.response_headers,
+                g_content_length_header_name,
+                aws_byte_cursor_from_c_str(content_length_buffer));
 
             if (meta_request->headers_callback(
                     meta_request,
-                    response_headers,
+                    request->send_data.response_headers,
                     s_s3_auto_ranged_get_success_status(meta_request),
                     meta_request->user_data)) {
 
                 error_code = aws_last_error_or_unknown();
             }
             meta_request->headers_callback = NULL;
-
-            aws_http_headers_release(response_headers);
         }
     }
 
