@@ -176,6 +176,10 @@ static const struct aws_http_header s_put_object_test_headers[] = {
         .name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-expected-bucket-owner"),
         .value = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("ExpectedBucketOwnerValue"),
     },
+    {
+        .name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-checksum-crc32"),
+        .value = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("AAAAAA=="),
+    },
 };
 
 static int s_fill_byte_buf(struct aws_byte_buf *buffer, struct aws_allocator *allocator, size_t buffer_size) {
@@ -675,19 +679,23 @@ static int s_test_s3_create_multipart_upload_message_new(struct aws_allocator *a
     ASSERT_TRUE(original_message != NULL);
 
     struct aws_http_message *create_multipart_upload_message =
-        aws_s3_create_multipart_upload_message_new(allocator, original_message, AWS_SCA_NONE);
+        aws_s3_create_multipart_upload_message_new(allocator, original_message, NULL);
     ASSERT_TRUE(create_multipart_upload_message != NULL);
 
     ASSERT_SUCCESS(s_test_http_message_request_method(create_multipart_upload_message, "POST"));
     ASSERT_SUCCESS(s_test_http_message_request_path(create_multipart_upload_message, &expected_create_path));
+
+    const struct aws_byte_cursor header_exclude_exceptions[] = {
+        AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Content-Length"),
+    };
     ASSERT_SUCCESS(s_test_http_headers_match(
         allocator,
         original_message,
         create_multipart_upload_message,
         g_s3_create_multipart_upload_excluded_headers,
         g_s3_create_multipart_upload_excluded_headers_count,
-        NULL,
-        0));
+        header_exclude_exceptions,
+        AWS_ARRAY_SIZE(header_exclude_exceptions)));
 
     aws_http_message_release(create_multipart_upload_message);
     aws_http_message_release(original_message);
@@ -858,6 +866,7 @@ static int s_test_s3_complete_multipart_message_new(struct aws_allocator *alloca
 
     const struct aws_byte_cursor header_exclude_exceptions[] = {
         AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Content-Length"),
+        AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-mp-object-size"),
     };
 
     struct aws_byte_cursor path = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(TEST_PATH);
@@ -872,8 +881,8 @@ static int s_test_s3_complete_multipart_message_new(struct aws_allocator *alloca
     struct aws_byte_buf body_buffer;
     aws_byte_buf_init(&body_buffer, allocator, 64);
 
-    struct aws_http_message *complete_multipart_message = aws_s3_complete_multipart_message_new(
-        allocator, original_message, &body_buffer, upload_id, &parts, AWS_SCA_NONE);
+    struct aws_http_message *complete_multipart_message =
+        aws_s3_complete_multipart_message_new(allocator, original_message, &body_buffer, upload_id, &parts, NULL);
 
     ASSERT_SUCCESS(s_test_http_message_request_method(complete_multipart_message, "POST"));
     ASSERT_SUCCESS(s_test_http_message_request_path(complete_multipart_message, &expected_create_path));
