@@ -11,6 +11,8 @@
 #include <aws/sdkutils/private/endpoints_types_impl.h>
 #include <aws/sdkutils/private/endpoints_util.h>
 
+void s_endpoints_value_clean_up_cb(void *value);
+
 uint64_t aws_endpoints_fn_name_hash[AWS_ENDPOINTS_FN_LAST];
 
 void aws_endpoints_rule_engine_init(void) {
@@ -85,6 +87,10 @@ void aws_endpoints_parameter_destroy(struct aws_endpoints_parameter *parameter)
         return;
     }
 
+    if (parameter->has_default_value && parameter->type == AWS_ENDPOINTS_PARAMETER_STRING_ARRAY) {
+        aws_array_list_deep_clean_up(&parameter->default_value.v.array, s_endpoints_value_clean_up_cb);
+    }
+
     aws_mem_release(parameter->allocator, parameter);
 }
 
@@ -189,6 +195,7 @@ void aws_endpoints_scope_value_destroy(struct aws_endpoints_scope_value *scope_v
         return;
     }
     aws_string_destroy(scope_value->name.string);
+
     aws_endpoints_value_clean_up(&scope_value->value);
     aws_mem_release(scope_value->allocator, scope_value);
 }
@@ -198,6 +205,10 @@ void aws_endpoints_value_clean_up_cb(void *value);
 void aws_endpoints_value_clean_up(struct aws_endpoints_value *aws_endpoints_value) {
     AWS_PRECONDITION(aws_endpoints_value);
 
+    if (aws_endpoints_value->is_ref) {
+        goto on_done;
+    }
+
     if (aws_endpoints_value->type == AWS_ENDPOINTS_VALUE_STRING) {
         aws_string_destroy(aws_endpoints_value->v.owning_cursor_string.string);
     }
@@ -207,13 +218,14 @@ void aws_endpoints_value_clean_up(struct aws_endpoints_value *aws_endpoints_valu
     }
 
     if (aws_endpoints_value->type == AWS_ENDPOINTS_VALUE_ARRAY) {
-        aws_array_list_deep_clean_up(&aws_endpoints_value->v.array, aws_endpoints_value_clean_up_cb);
+        aws_array_list_deep_clean_up(&aws_endpoints_value->v.array, s_endpoints_value_clean_up_cb);
     }
 
+on_done:
     AWS_ZERO_STRUCT(*aws_endpoints_value);
 }
 
-void aws_endpoints_value_clean_up_cb(void *value) {
+void s_endpoints_value_clean_up_cb(void *value) {
     struct aws_endpoints_value *aws_endpoints_value = value;
     aws_endpoints_value_clean_up(aws_endpoints_value);
 }
@@ -224,15 +236,35 @@ int aws_endpoints_deep_copy_parameter_value(
     struct aws_endpoints_value *to) {
 
     to->type = from->type;
+    to->is_ref = false;
 
     if (to->type == AWS_ENDPOINTS_VALUE_STRING) {
-        to->v.owning_cursor_string = aws_endpoints_owning_cursor_create(allocator, from->v.owning_cursor_string.string);
+        to->v.owning_cursor_string =
+            aws_endpoints_owning_cursor_from_cursor(allocator, from->v.owning_cursor_string.cur);
     } else if (to->type == AWS_ENDPOINTS_VALUE_BOOLEAN) {
         to->v.boolean = from->v.boolean;
+    } else if (to->type == AWS_ENDPOINTS_VALUE_ARRAY) {
+        size_t len = aws_array_list_length(&from->v.array);
+        aws_array_list_init_dynamic(&to->v.array, allocator, len, sizeof(struct aws_endpoints_value));
+        for (size_t i = 0; i < len; ++i) {
+            struct aws_endpoints_value val;
+            aws_array_list_get_at(&from->v.array, &val, i);
+
+            struct aws_endpoints_value to_val;
+            if (aws_endpoints_deep_copy_parameter_value(allocator, &val, &to_val)) {
+                AWS_LOGF_ERROR(AWS_LS_SDKUTILS_ENDPOINTS_RESOLVE, "Unexpected array element type.");
+                goto on_error;
+            }
+
+            aws_array_list_set_at(&to->v.array, &to_val, i);
+        }
     } else {
         AWS_LOGF_ERROR(AWS_LS_SDKUTILS_ENDPOINTS_RESOLVE, "Unexpected value type.");
         return aws_raise_error(AWS_ERROR_INVALID_STATE);
     }
-
     return AWS_OP_SUCCESS;
+
+on_error:
+    aws_endpoints_value_clean_up(to);
+    return AWS_OP_ERR;
 }
