@@ -7,10 +7,11 @@
 use crate::renegotiate::RenegotiateState;
 use crate::{
     callbacks::*,
-    cert_chain::CertificateChain,
+    cert_chain::{CertificateChain, CertificateChainHandle},
     config::Config,
     enums::*,
     error::{Error, Fallible, Pollable},
+    psk::Psk,
     security,
 };
 
@@ -54,6 +55,7 @@ pub struct KeyUpdateCount {
     pub recv_key_updates: u8,
 }
 
+/// Corresponds to [s2n_connection].
 pub struct Connection {
     connection: NonNull<s2n_connection>,
 }
@@ -107,6 +109,8 @@ impl Connection {
     /// and [`Connection::set_config`] to set the policy on the Config instead of on the Connection.
     /// See the s2n-tls usage guide:
     /// <https://aws.github.io/s2n-tls/usage-guide/ch06-security-policies.html>
+    ///
+    /// Corresponds to [s2n_connection_new].
     pub fn new(mode: Mode) -> Self {
         crate::init::init();
 
@@ -167,6 +171,8 @@ impl Connection {
     /// can be used to configure s2n to either use built-in blinding (set blinding
     /// to Blinding::BuiltIn) or self-service blinding (set blinding to
     /// Blinding::SelfService).
+    ///
+    /// Corresponds to [s2n_connection_set_blinding].
     pub fn set_blinding(&mut self, blinding: Blinding) -> Result<&mut Self, Error> {
         unsafe {
             s2n_connection_set_blinding(self.connection.as_ptr(), blinding.into()).into_result()
@@ -180,6 +186,8 @@ impl Connection {
     /// [underlying C call](`s2n_connection_get_delay`) encounters errors.
     /// Failure indicates that calls to [`Self::poll_shutdown`] will also fail and
     /// that a graceful two-way shutdown of the connection will not be possible.
+    ///
+    /// Corresponds to [s2n_connection_get_delay].
     pub fn remaining_blinding_delay(&self) -> Result<Duration, Error> {
         let nanos = unsafe { s2n_connection_get_delay(self.connection.as_ptr()).into_result() }?;
         Ok(Duration::from_nanos(nanos))
@@ -190,6 +198,8 @@ impl Connection {
     /// If this is set to ClientAuthType::Optional the server will request a client certificate
     /// but allow the client to not provide one. Rejecting a client certificate when using
     /// ClientAuthType::Optional will terminate the handshake.
+    ///
+    /// Corresponds to [s2n_connection_set_client_auth_type].
     pub fn set_client_auth_type(
         &mut self,
         client_auth_type: ClientAuthType,
@@ -226,6 +236,8 @@ impl Connection {
     }
 
     /// Associates a configuration object with a connection.
+    ///
+    /// Corresponds to [s2n_connection_set_config].
     pub fn set_config(&mut self, mut config: Config) -> Result<&mut Self, Error> {
         unsafe {
             // attempt to drop the currently set config
@@ -262,6 +274,7 @@ impl Connection {
         Some(config)
     }
 
+    /// Corresponds to [s2n_connection_set_cipher_preferences].
     pub fn set_security_policy(&mut self, policy: &security::Policy) -> Result<&mut Self, Error> {
         unsafe {
             s2n_connection_set_cipher_preferences(
@@ -278,6 +291,8 @@ impl Connection {
     /// s2n_send uses small TLS records that fit into a single TCP segment for the resize_threshold
     /// bytes (cap to 8M) of data and reset record size back to a single segment after timeout_threshold
     /// seconds of inactivity.
+    ///
+    /// Corresponds to [s2n_connection_set_dynamic_record_threshold].
     pub fn set_dynamic_record_threshold(
         &mut self,
         resize_threshold: u32,
@@ -305,6 +320,8 @@ impl Connection {
     /// update their sending key. Note that s2n-tls currently only supports
     /// `peer_request` being set to `KeyUpdateNotRequested` and will return an error
     /// if any other value is used.
+    ///
+    /// Corresponds to [s2n_connection_request_key_update].
     pub fn request_key_update(&mut self, peer_request: PeerKeyUpdate) -> Result<&mut Self, Error> {
         unsafe {
             s2n_connection_request_key_update(self.connection.as_ptr(), peer_request.into())
@@ -316,6 +333,8 @@ impl Connection {
     /// Reports the number of times sending and receiving keys have been updated.
     ///
     /// This only applies to TLS1.3. Earlier versions do not support key updates.
+    ///
+    /// Corresponds to [s2n_connection_get_key_update_counts].
     #[cfg(feature = "unstable-ktls")]
     pub fn key_update_counts(&self) -> Result<KeyUpdateCount, Error> {
         let mut send_key_updates = 0;
@@ -341,6 +360,8 @@ impl Connection {
     /// Client Hello message as the ALPN extension. As a server, the list is used to negotiate
     /// a mutual application protocol with the client. After the negotiation for the connection has
     /// completed, the agreed upon protocol can be retrieved with s2n_get_application_protocol
+    ///
+    /// Corresponds to [s2n_connection_set_protocol_preferences].
     pub fn set_application_protocol_preference<P: IntoIterator<Item = I>, I: AsRef<[u8]>>(
         &mut self,
         protocols: P,
@@ -358,6 +379,7 @@ impl Connection {
         Ok(self)
     }
 
+    /// Corresponds to [s2n_connection_append_protocol_preference].
     pub fn append_application_protocol_preference(
         &mut self,
         protocol: &[u8],
@@ -377,6 +399,8 @@ impl Connection {
     }
 
     /// may be used to receive data with callbacks defined by the user.
+    ///
+    /// Corresponds to [s2n_connection_set_recv_cb].
     pub fn set_receive_callback(&mut self, callback: s2n_recv_fn) -> Result<&mut Self, Error> {
         unsafe { s2n_connection_set_recv_cb(self.connection.as_ptr(), callback).into_result() }?;
         Ok(self)
@@ -385,12 +409,16 @@ impl Connection {
     /// # Safety
     ///
     /// The `context` pointer must live at least as long as the connection
+    ///
+    /// Corresponds to [s2n_connection_set_recv_ctx].
     pub unsafe fn set_receive_context(&mut self, context: *mut c_void) -> Result<&mut Self, Error> {
         s2n_connection_set_recv_ctx(self.connection.as_ptr(), context).into_result()?;
         Ok(self)
     }
 
     /// may be used to receive data with callbacks defined by the user.
+    ///
+    /// Corresponds to [s2n_connection_set_send_cb].
     pub fn set_send_callback(&mut self, callback: s2n_send_fn) -> Result<&mut Self, Error> {
         unsafe { s2n_connection_set_send_cb(self.connection.as_ptr(), callback).into_result() }?;
         Ok(self)
@@ -399,6 +427,8 @@ impl Connection {
     /// # Safety
     ///
     /// The `context` pointer must live at least as long as the connection
+    ///
+    /// Corresponds to [s2n_connection_set_send_ctx].
     pub unsafe fn set_send_context(&mut self, context: *mut c_void) -> Result<&mut Self, Error> {
         s2n_connection_set_send_ctx(self.connection.as_ptr(), context).into_result()?;
         Ok(self)
@@ -410,8 +440,7 @@ impl Connection {
     /// The callback may be called more than once during certificate validation as each SAN on
     /// the certificate will be checked.
     ///
-    /// Corresponds to the underlying C API
-    /// [s2n_connection_set_verify_host_callback](https://aws.github.io/s2n-tls/doxygen/s2n_8h.html).
+    /// Corresponds to [s2n_connection_set_verify_host_callback].
     pub fn set_verify_host_callback<T: 'static + VerifyHostNameCallback>(
         &mut self,
         handler: T,
@@ -438,14 +467,18 @@ impl Connection {
         Ok(self)
     }
 
-    /// Connections prefering low latency will be encrypted using small record sizes that
+    /// Connections preferring low latency will be encrypted using small record sizes that
     /// can be decrypted sooner by the recipient.
+    ///
+    /// Corresponds to [s2n_connection_prefer_low_latency].
     pub fn prefer_low_latency(&mut self) -> Result<&mut Self, Error> {
         unsafe { s2n_connection_prefer_low_latency(self.connection.as_ptr()).into_result() }?;
         Ok(self)
     }
 
-    /// Connections prefering throughput will use large record sizes that minimize overhead.
+    /// Connections preferring throughput will use large record sizes that minimize overhead.
+    ///
+    /// Corresponds to [s2n_connection_prefer_throughput].
     pub fn prefer_throughput(&mut self) -> Result<&mut Self, Error> {
         unsafe { s2n_connection_prefer_throughput(self.connection.as_ptr()).into_result() }?;
         Ok(self)
@@ -453,9 +486,7 @@ impl Connection {
 
     /// Configure the connection to reduce potentially expensive calls to recv.
     ///
-    /// Refer to the corresponding C API
-    /// [s2n_connection_set_recv_buffering](https://aws.github.io/s2n-tls/doxygen/s2n_8h.html)
-    /// for more information.
+    /// Corresponds to [s2n_connection_set_recv_buffering].
     pub fn set_receive_buffering(&mut self, enabled: bool) -> Result<&mut Self, Error> {
         unsafe {
             s2n_connection_set_recv_buffering(self.connection.as_ptr(), enabled).into_result()
@@ -467,11 +498,14 @@ impl Connection {
     ///
     /// This function may be called when a connection is in keep-alive or idle state to
     /// reduce memory overhead of long lived connections.
+    ///
+    /// Corresponds to [s2n_connection_release_buffers].
     pub fn release_buffers(&mut self) -> Result<&mut Self, Error> {
         unsafe { s2n_connection_release_buffers(self.connection.as_ptr()).into_result() }?;
         Ok(self)
     }
 
+    /// Corresponds to [s2n_connection_use_corked_io].
     pub fn use_corked_io(&mut self) -> Result<&mut Self, Error> {
         unsafe { s2n_connection_use_corked_io(self.connection.as_ptr()).into_result() }?;
         Ok(self)
@@ -502,6 +536,8 @@ impl Connection {
     /// This function should be called after all I/O is completed and s2n_shutdown has been
     /// called. Reusing the same connection handle(s) is more performant than repeatedly
     /// calling s2n_connection_new and s2n_connection_free
+    ///
+    /// Corresponds to [s2n_connection_wipe].
     pub fn wipe(&mut self) -> Result<&mut Self, Error> {
         self.wipe_method(|conn| unsafe { s2n_connection_wipe(conn.as_ptr()).into_result() })?;
         Ok(self)
@@ -582,6 +618,8 @@ impl Connection {
     ///
     /// The handshake does not continue execution (and therefore can't call
     /// any other callbacks) until the blocking async task reports completion.
+    ///
+    /// Corresponds to [s2n_negotiate].
     pub fn poll_negotiate(&mut self) -> Poll<Result<&mut Self, Error>> {
         let mut blocked = s2n_blocked_status::NOT_BLOCKED;
         self.poll_negotiate_method(|conn| unsafe {
@@ -594,6 +632,8 @@ impl Connection {
     /// [negotiate](`Self::poll_negotiate`) has succeeded.
     ///
     /// Returns the number of bytes written, and may indicate a partial write.
+    ///
+    /// Corresponds to [s2n_send].
     #[cfg(not(feature = "unstable-renegotiate"))]
     pub fn poll_send(&mut self, buf: &[u8]) -> Poll<Result<usize, Error>> {
         let mut blocked = s2n_blocked_status::NOT_BLOCKED;
@@ -617,6 +657,8 @@ impl Connection {
     ///
     /// Returns the number of bytes read, and may indicate a partial read.
     /// 0 bytes returned indicates EOF due to connection closure.
+    ///
+    /// Corresponds to [s2n_recv].
     pub fn poll_recv(&mut self, buf: &mut [u8]) -> Poll<Result<usize, Error>> {
         let buf_len: isize = buf.len().try_into().map_err(|_| Error::INVALID_INPUT)?;
         let buf_ptr = buf.as_ptr() as *mut ::libc::c_void;
@@ -631,9 +673,11 @@ impl Connection {
     /// 0 bytes returned indicates EOF due to connection closure.
     ///
     /// Safety: this function is always safe to call, and additionally:
-    /// 1. It will never deinitialize any bytes in `buf`.
+    /// 1. It will never uninitialize any bytes in `buf`.
     /// 2. If it returns `Ok(n)`, then the first `n` bytes of `buf`
     ///    will have been initialized by this function.
+    ///
+    /// Corresponds to [s2n_recv].
     pub fn poll_recv_uninitialized(
         &mut self,
         buf: &mut [MaybeUninit<u8>],
@@ -655,6 +699,8 @@ impl Connection {
     /// buffered for sending. poll_send may need to be called again to fully send
     /// all data. See the [Usage Guide](https://github.com/aws/s2n-tls/blob/main/docs/usage-guide/topics/ch07-io.md)
     /// for more details.
+    ///
+    /// Corresponds to [s2n_flush].
     pub fn poll_flush(&mut self) -> Poll<Result<&mut Self, Error>> {
         let mut blocked = s2n_blocked_status::NOT_BLOCKED;
         unsafe {
@@ -665,6 +711,8 @@ impl Connection {
     }
 
     /// Gets the number of bytes that are currently available in the buffer to be read.
+    ///
+    /// Corresponds to [s2n_peek].
     pub fn peek_len(&self) -> usize {
         unsafe { s2n_peek(self.connection.as_ptr()) as usize }
     }
@@ -674,6 +722,8 @@ impl Connection {
     /// The shutdown is not complete until the necessary shutdown messages
     /// have been successfully sent and received. If the peer does not respond
     /// correctly, the graceful shutdown may fail.
+    ///
+    /// Corresponds to [s2n_shutdown].
     pub fn poll_shutdown(&mut self) -> Poll<Result<&mut Self, Error>> {
         if !self.remaining_blinding_delay()?.is_zero() {
             return Poll::Pending;
@@ -688,8 +738,10 @@ impl Connection {
 
     /// Attempts a graceful shutdown of the write side of a TLS connection.
     ///
-    /// Unlike Self::poll_shutdown, no reponse from the peer is necessary.
+    /// Unlike Self::poll_shutdown, no response from the peer is necessary.
     /// If using TLS1.3, the connection can continue to be used for reading afterwards.
+    ///
+    /// Corresponds to [s2n_shutdown_send].
     pub fn poll_shutdown_send(&mut self) -> Poll<Result<&mut Self, Error>> {
         if !self.remaining_blinding_delay()?.is_zero() {
             return Poll::Pending;
@@ -703,6 +755,8 @@ impl Connection {
     }
 
     /// Returns the TLS alert code, if any
+    ///
+    /// Corresponds to [s2n_connection_get_alert].
     pub fn alert(&self) -> Option<u8> {
         let alert =
             unsafe { s2n_connection_get_alert(self.connection.as_ptr()).into_result() }.ok()?;
@@ -710,6 +764,8 @@ impl Connection {
     }
 
     /// Sets the server name value for the connection
+    ///
+    /// Corresponds to [s2n_set_server_name].
     pub fn set_server_name(&mut self, server_name: &str) -> Result<&mut Self, Error> {
         let server_name = std::ffi::CString::new(server_name).map_err(|_| Error::INVALID_INPUT)?;
         unsafe {
@@ -719,6 +775,8 @@ impl Connection {
     }
 
     /// Get the server name associated with the connection client hello.
+    ///
+    /// Corresponds to [s2n_get_server_name].
     pub fn server_name(&self) -> Option<&str> {
         unsafe {
             let server_name = s2n_get_server_name(self.connection.as_ptr());
@@ -730,6 +788,8 @@ impl Connection {
     }
 
     /// Adds a session ticket from a previous TLS connection to create a resumed session
+    ///
+    /// Corresponds to [s2n_connection_set_session].
     pub fn set_session_ticket(&mut self, session: &[u8]) -> Result<&mut Self, Error> {
         unsafe {
             s2n_connection_set_session(self.connection.as_ptr(), session.as_ptr(), session.len())
@@ -739,6 +799,8 @@ impl Connection {
     }
 
     /// Retrieves the size of the session ticket.
+    ///
+    /// Corresponds to [s2n_connection_get_session_length].
     pub fn session_ticket_length(&self) -> Result<usize, Error> {
         let len =
             unsafe { s2n_connection_get_session_length(self.connection.as_ptr()).into_result()? };
@@ -754,6 +816,8 @@ impl Connection {
     /// Note: This function is not recommended for > TLS1.2 because in TLS1.3
     /// servers can send multiple session tickets and this will return only
     /// the most recently received ticket.
+    ///
+    /// Corresponds to [s2n_connection_get_session].
     pub fn session_ticket(&self, output: &mut [u8]) -> Result<usize, Error> {
         if output.len() < self.session_ticket_length()? {
             return Err(Error::INVALID_INPUT);
@@ -771,7 +835,7 @@ impl Connection {
 
         if let Some(waker) = waker {
             if let Some(prev_waker) = ctx.waker.as_mut() {
-                // only replace the Waker if they dont reference the same task
+                // only replace the Waker if they don't reference the same task
                 if !prev_waker.will_wake(waker) {
                     prev_waker.clone_from(waker);
                 }
@@ -845,6 +909,8 @@ impl Connection {
     }
 
     /// Mark that the server_name extension was used to configure the connection.
+    ///
+    /// Corresponds to [s2n_connection_server_name_extension_used].
     pub fn server_name_extension_used(&mut self) {
         // TODO: requiring the application to call this method is a pretty sharp edge.
         // Figure out if its possible to automatically call this from the Rust bindings.
@@ -858,11 +924,15 @@ impl Connection {
     /// Check if client auth was used for a connection.
     ///
     /// This is only relevant if [`ClientAuthType::Optional] was used.
+    ///
+    /// Corresponds to [s2n_connection_client_cert_used].
     pub fn client_cert_used(&self) -> bool {
         unsafe { s2n_connection_client_cert_used(self.connection.as_ptr()) == 1 }
     }
 
     /// Retrieves the raw bytes of the client cert chain received from the peer, if present.
+    ///
+    /// Corresponds to [s2n_connection_get_client_cert_chain].
     pub fn client_cert_chain_bytes(&self) -> Result<Option<&[u8]>, Error> {
         if !self.client_cert_used() {
             return Ok(None);
@@ -911,6 +981,8 @@ impl Connection {
     /// client_hello.raw_message();
     /// drop(conn);
     /// ```
+    ///
+    /// Corresponds to [s2n_connection_get_client_hello].
     pub fn client_hello(&self) -> Result<&crate::client_hello::ClientHello, Error> {
         let mut handle =
             unsafe { s2n_connection_get_client_hello(self.connection.as_ptr()).into_result()? };
@@ -919,6 +991,7 @@ impl Connection {
         }))
     }
 
+    /// Corresponds to [s2n_client_hello_cb_done].
     pub(crate) fn mark_client_hello_cb_done(&mut self) -> Result<(), Error> {
         unsafe {
             s2n_client_hello_cb_done(self.connection.as_ptr()).into_result()?;
@@ -927,6 +1000,8 @@ impl Connection {
     }
 
     /// Access the protocol version selected for the connection.
+    ///
+    /// Corresponds to [s2n_connection_get_actual_protocol_version].
     pub fn actual_protocol_version(&self) -> Result<Version, Error> {
         let version = unsafe {
             s2n_connection_get_actual_protocol_version(self.connection.as_ptr()).into_result()?
@@ -940,6 +1015,9 @@ impl Connection {
     /// advertising a higher protocol version like SSLv3 or TLS1.0.
     /// [Connection::actual_protocol_version()] can be used to retrieve the
     /// protocol version that is actually used on the connection.
+    ///
+    /// Corresponds to [s2n_connection_get_client_hello_version], but only checks
+    /// for SSLv2.
     pub fn client_hello_is_sslv2(&self) -> Result<bool, Error> {
         let version = unsafe {
             s2n_connection_get_client_hello_version(self.connection.as_ptr()).into_result()?
@@ -948,6 +1026,7 @@ impl Connection {
         Ok(version == Version::SSLV2)
     }
 
+    /// Corresponds to [s2n_connection_get_handshake_type_name].
     pub fn handshake_type(&self) -> Result<&str, Error> {
         let handshake = unsafe {
             s2n_connection_get_handshake_type_name(self.connection.as_ptr()).into_result()?
@@ -961,6 +1040,7 @@ impl Connection {
         }
     }
 
+    /// Corresponds to [s2n_connection_get_cipher].
     pub fn cipher_suite(&self) -> Result<&str, Error> {
         let cipher = unsafe { s2n_connection_get_cipher(self.connection.as_ptr()).into_result()? };
         unsafe {
@@ -972,6 +1052,7 @@ impl Connection {
         }
     }
 
+    /// Corresponds to [s2n_connection_get_kem_name].
     #[deprecated = "PQ TLS 1.2 KEM Names are no longer supported. Use kem_group_name() to retrieve PQ TLS 1.3 Group name."]
     pub fn kem_name(&self) -> Option<&str> {
         let name_bytes = {
@@ -1001,6 +1082,7 @@ impl Connection {
         }
     }
 
+    /// Corresponds to [s2n_connection_get_kem_group_name].
     pub fn kem_group_name(&self) -> Option<&str> {
         let name_bytes = {
             let name = unsafe { s2n_connection_get_kem_group_name(self.connection.as_ptr()) };
@@ -1029,6 +1111,7 @@ impl Connection {
         }
     }
 
+    /// Corresponds to [s2n_connection_get_curve].
     pub fn selected_curve(&self) -> Result<&str, Error> {
         let curve = unsafe { s2n_connection_get_curve(self.connection.as_ptr()).into_result()? };
         unsafe {
@@ -1040,6 +1123,7 @@ impl Connection {
         }
     }
 
+    /// Corresponds to [s2n_connection_get_selected_signature_algorithm].
     pub fn selected_signature_algorithm(&self) -> Result<SignatureAlgorithm, Error> {
         let mut sig_alg = s2n_tls_signature_algorithm::ANONYMOUS;
         unsafe {
@@ -1049,6 +1133,7 @@ impl Connection {
         sig_alg.try_into()
     }
 
+    /// Corresponds to [s2n_connection_get_selected_digest_algorithm].
     pub fn selected_hash_algorithm(&self) -> Result<HashAlgorithm, Error> {
         let mut hash_alg = s2n_tls_hash_algorithm::NONE;
         unsafe {
@@ -1058,6 +1143,7 @@ impl Connection {
         hash_alg.try_into()
     }
 
+    /// Corresponds to [s2n_connection_get_selected_client_cert_signature_algorithm].
     pub fn selected_client_signature_algorithm(&self) -> Result<Option<SignatureAlgorithm>, Error> {
         let mut sig_alg = s2n_tls_signature_algorithm::ANONYMOUS;
         unsafe {
@@ -1073,6 +1159,7 @@ impl Connection {
         })
     }
 
+    /// Corresponds to [s2n_connection_get_selected_client_cert_digest_algorithm].
     pub fn selected_client_hash_algorithm(&self) -> Result<Option<HashAlgorithm>, Error> {
         let mut hash_alg = s2n_tls_hash_algorithm::NONE;
         unsafe {
@@ -1088,6 +1175,7 @@ impl Connection {
         })
     }
 
+    /// Corresponds to [s2n_get_application_protocol].
     pub fn application_protocol(&self) -> Option<&[u8]> {
         let protocol = unsafe { s2n_get_application_protocol(self.connection.as_ptr()) };
         if protocol.is_null() {
@@ -1101,6 +1189,8 @@ impl Connection {
     /// See https://datatracker.ietf.org/doc/html/rfc5705 and https://www.rfc-editor.org/rfc/rfc8446.
     ///
     /// This is currently only available with TLS 1.3 connections which have finished a handshake.
+    ///
+    /// Corresponds to [s2n_connection_tls_exporter].
     pub fn tls_exporter(
         &self,
         label: &[u8],
@@ -1125,13 +1215,18 @@ impl Connection {
     /// Returns the validated peer certificate chain.
     // 'static lifetime is because this copies the certificate chain from the connection into a new
     // chain, so the lifetime is independent of the connection.
+    ///
+    /// Corresponds to [s2n_connection_get_peer_cert_chain].
     pub fn peer_cert_chain(&self) -> Result<CertificateChain<'static>, Error> {
         unsafe {
-            let mut chain = CertificateChain::allocate_owned()?;
-            s2n_connection_get_peer_cert_chain(self.connection.as_ptr(), chain.as_mut_ptr())
-                .into_result()
-                .map(|_| ())?;
-            Ok(chain)
+            let chain_handle = CertificateChainHandle::allocate()?;
+            s2n_connection_get_peer_cert_chain(
+                self.connection.as_ptr(),
+                chain_handle.cert.as_ptr(),
+            )
+            .into_result()
+            .map(|_| ())?;
+            Ok(CertificateChain::from_allocated(chain_handle))
         }
     }
 
@@ -1142,6 +1237,8 @@ impl Connection {
     /// - If `self` is a client connection, the certificate sent in response to a CertificateRequest
     ///   message is returned. Currently s2n-tls supports loading only one certificate in client mode. Note that
     ///   not all TLS endpoints will request a certificate.
+    ///
+    /// Corresponds to [s2n_connection_get_selected_cert].
     pub fn selected_cert(&self) -> Option<CertificateChain<'_>> {
         unsafe {
             // The API only returns null, no error is actually set.
@@ -1157,6 +1254,7 @@ impl Connection {
         }
     }
 
+    /// Corresponds to [s2n_connection_get_master_secret].
     pub fn master_secret(&self) -> Result<Vec<u8>, Error> {
         // TLS1.2 master secrets are always 48 bytes
         let mut secret = vec![0; 48];
@@ -1172,6 +1270,8 @@ impl Connection {
     }
 
     /// Retrieves the size of the serialized connection
+    ///
+    /// Corresponds to [s2n_connection_serialization_length].
     pub fn serialization_length(&self) -> Result<usize, Error> {
         unsafe {
             let mut length = 0;
@@ -1182,6 +1282,8 @@ impl Connection {
     }
 
     /// Serializes the TLS connection into the provided buffer
+    ///
+    /// Corresponds to [s2n_connection_serialize].
     pub fn serialize(&self, output: &mut [u8]) -> Result<(), Error> {
         unsafe {
             s2n_connection_serialize(
@@ -1196,6 +1298,8 @@ impl Connection {
 
     /// Deserializes the input buffer into a new TLS connection that can send/recv
     /// data from the original peer.
+    ///
+    /// Corresponds to [s2n_connection_deserialize].
     pub fn deserialize(&mut self, input: &[u8]) -> Result<(), Error> {
         let size = input.len();
         /* This is not ideal, we know that s2n_connection_deserialize will not mutate the
@@ -1213,14 +1317,59 @@ impl Connection {
     }
 
     /// Determines whether the connection was resumed from an earlier handshake.
+    ///
+    /// Corresponds to [s2n_connection_is_session_resumed].
     pub fn resumed(&self) -> bool {
         unsafe { s2n_connection_is_session_resumed(self.connection.as_ptr()) == 1 }
     }
 
+    /// Append an external psk to a connection.
+    ///
+    /// This may be called repeatedly to support multiple PSKs.
+    ///
+    /// Corresponds to [s2n_connection_append_psk].
+    pub fn append_psk(&mut self, psk: &Psk) -> Result<(), Error> {
+        unsafe {
+            // SAFETY: retrieving a *mut s2n_psk from &Psk: s2n-tls does not treat
+            // the pointer as mutable, and only holds the reference to copy the
+            // PSK onto the connection.
+            s2n_connection_append_psk(self.as_ptr(), psk.ptr.as_ptr()).into_result()?
+        };
+        Ok(())
+    }
+
+    /// Corresponds to [s2n_connection_get_negotiated_psk_identity_length].
+    pub fn negotiated_psk_identity_length(&self) -> Result<usize, Error> {
+        let mut length = 0;
+        unsafe {
+            s2n_connection_get_negotiated_psk_identity_length(self.connection.as_ptr(), &mut length)
+                .into_result()?
+        };
+        Ok(length as usize)
+    }
+
+    /// Retrieve the negotiated psk identity. Use [Connection::negotiated_psk_identity_length]
+    /// to retrieve the length of the psk identity.
+    ///
+    /// Corresponds to [s2n_connection_get_negotiated_psk_identity].
+    pub fn negotiated_psk_identity(&self, destination: &mut [u8]) -> Result<(), Error> {
+        unsafe {
+            s2n_connection_get_negotiated_psk_identity(
+                self.connection.as_ptr(),
+                destination.as_mut_ptr(),
+                destination.len().min(u16::MAX as usize) as u16,
+            )
+            .into_result()?;
+        }
+        Ok(())
+    }
+
     /// Associates an arbitrary application context with the Connection to be later retrieved via
     /// the [`Self::application_context()`] and [`Self::application_context_mut()`] APIs.
     ///
     /// This API will override an existing application context set on the Connection.
+    ///
+    /// Corresponds to [s2n_connection_set_ctx].
     pub fn set_application_context<T: Send + Sync + 'static>(&mut self, app_context: T) {
         self.context_mut().app_context = Some(Box::new(app_context));
     }
@@ -1232,6 +1381,8 @@ impl Connection {
     ///
     /// To set a context on the connection, use [`Self::set_application_context()`]. To retrieve a
     /// mutable reference to the context, use [`Self::application_context_mut()`].
+    ///
+    /// Corresponds to [s2n_connection_get_ctx].
     pub fn application_context<T: Send + Sync + 'static>(&self) -> Option<&T> {
         match self.context().app_context.as_ref() {
             None => None,
@@ -1249,6 +1400,8 @@ impl Connection {
     ///
     /// To set a context on the connection, use [`Self::set_application_context()`]. To retrieve an
     /// immutable reference to the context, use [`Self::application_context()`].
+    ///
+    /// Corresponds to [s2n_connection_get_ctx].
     pub fn application_context_mut<T: Send + Sync + 'static>(&mut self) -> Option<&mut T> {
         match self.context_mut().app_context.as_mut() {
             None => None,
@@ -1295,11 +1448,13 @@ impl Context {
 
 #[cfg(feature = "quic")]
 impl Connection {
+    /// Corresponds to [s2n_connection_enable_quic].
     pub fn enable_quic(&mut self) -> Result<&mut Self, Error> {
         unsafe { s2n_connection_enable_quic(self.connection.as_ptr()).into_result() }?;
         Ok(self)
     }
 
+    /// Corresponds to [s2n_connection_set_quic_transport_parameters].
     pub fn set_quic_transport_parameters(&mut self, buffer: &[u8]) -> Result<&mut Self, Error> {
         unsafe {
             s2n_connection_set_quic_transport_parameters(
@@ -1312,6 +1467,7 @@ impl Connection {
         Ok(self)
     }
 
+    /// Corresponds to [s2n_connection_get_quic_transport_parameters].
     pub fn quic_transport_parameters(&mut self) -> Result<&[u8], Error> {
         let mut ptr = core::ptr::null();
         let mut len = 0;
@@ -1330,6 +1486,8 @@ impl Connection {
     /// # Safety
     ///
     /// The `context` pointer must live at least as long as the connection
+    ///
+    /// Corresponds to [s2n_connection_set_secret_callback].
     pub unsafe fn set_secret_callback(
         &mut self,
         callback: s2n_secret_cb,
@@ -1340,6 +1498,7 @@ impl Connection {
         Ok(self)
     }
 
+    /// Corresponds to [s2n_recv_quic_post_handshake_message].
     pub fn quic_process_post_handshake_message(&mut self) -> Result<&mut Self, Error> {
         let mut blocked = s2n_blocked_status::NOT_BLOCKED;
         unsafe {
@@ -1350,6 +1509,8 @@ impl Connection {
     }
 
     /// Allows the quic library to check if session tickets are expected
+    ///
+    /// Corresponds to [s2n_connection_are_session_tickets_enabled].
     pub fn are_session_tickets_enabled(&self) -> bool {
         unsafe { s2n_connection_are_session_tickets_enabled(self.connection.as_ptr()) }
     }
@@ -1368,6 +1529,7 @@ impl AsMut<Connection> for Connection {
 }
 
 impl Drop for Connection {
+    /// Corresponds to [s2n_connection_free].
     fn drop(&mut self) {
         // ignore failures since there's not much we can do about it
         unsafe {
