@@ -17,10 +17,15 @@
 
 #include "crypto/s2n_fips.h"
 #include "crypto/s2n_hmac.h"
-#include "crypto/s2n_openssl.h"
 #include "error/s2n_errno.h"
 #include "utils/s2n_safety.h"
 
+#if S2N_LIBCRYPTO_SUPPORTS_PROVIDERS
+static EVP_MD *s2n_evp_mds[S2N_HASH_ALGS_COUNT] = { 0 };
+#else
+static const EVP_MD *s2n_evp_mds[S2N_HASH_ALGS_COUNT] = { 0 };
+#endif
+
 static bool s2n_use_custom_md5_sha1()
 {
 #if defined(S2N_LIBCRYPTO_SUPPORTS_EVP_MD5_SHA1_HASH)
@@ -30,38 +35,75 @@ static bool s2n_use_custom_md5_sha1()
 #endif
 }
 
-static bool s2n_use_evp_impl()
-{
-    return s2n_is_in_fips_mode();
-}
-
+/* This is currently only used by s2n_evp_signing
+ * to determine whether or not to use EVP signing.
+ * Historically we only used EVP signing for FIPS.
+ * To avoid a premature behavior change, consider FIPS a requirement.
+ */
 bool s2n_hash_evp_fully_supported()
 {
-    return s2n_use_evp_impl() && !s2n_use_custom_md5_sha1();
+    return s2n_is_in_fips_mode() && !s2n_use_custom_md5_sha1();
+}
+
+S2N_RESULT s2n_hash_algorithms_init()
+{
+#if S2N_LIBCRYPTO_SUPPORTS_PROVIDERS
+    /* openssl-3.0 introduced the concept of providers.
+     * After openssl-3.0, the old EVP_sha256()-style methods will still work,
+     * but may be inefficient. See
+     * https://docs.openssl.org/3.4/man7/ossl-guide-libcrypto-introduction/#performance
+     *
+     * Additionally, the old style methods do not support property query strings
+     * to guide which provider to fetch from. This is important for FIPS, where
+     * the default query string of "fips=yes" will need to be overridden for
+     * legacy algorithms.
+     */
+    s2n_evp_mds[S2N_HASH_MD5] = EVP_MD_fetch(NULL, "MD5", "-fips");
+    s2n_evp_mds[S2N_HASH_MD5_SHA1] = EVP_MD_fetch(NULL, "MD5-SHA1", "-fips");
+    s2n_evp_mds[S2N_HASH_SHA1] = EVP_MD_fetch(NULL, "SHA1", NULL);
+    s2n_evp_mds[S2N_HASH_SHA224] = EVP_MD_fetch(NULL, "SHA224", NULL);
+    s2n_evp_mds[S2N_HASH_SHA256] = EVP_MD_fetch(NULL, "SHA256", NULL);
+    s2n_evp_mds[S2N_HASH_SHA384] = EVP_MD_fetch(NULL, "SHA384", NULL);
+    s2n_evp_mds[S2N_HASH_SHA512] = EVP_MD_fetch(NULL, "SHA512", NULL);
+#else
+    s2n_evp_mds[S2N_HASH_MD5] = EVP_md5();
+    s2n_evp_mds[S2N_HASH_SHA1] = EVP_sha1();
+    s2n_evp_mds[S2N_HASH_SHA224] = EVP_sha224();
+    s2n_evp_mds[S2N_HASH_SHA256] = EVP_sha256();
+    s2n_evp_mds[S2N_HASH_SHA384] = EVP_sha384();
+    s2n_evp_mds[S2N_HASH_SHA512] = EVP_sha512();
+    /* Very old libcryptos like openssl-1.0.2 do not support EVP_MD_md5_sha1().
+     * We work around that by manually combining MD5 and SHA1, rather than
+     * using the composite algorithm.
+     */
+    #if defined(S2N_LIBCRYPTO_SUPPORTS_EVP_MD5_SHA1_HASH)
+    s2n_evp_mds[S2N_HASH_MD5_SHA1] = EVP_md5_sha1();
+    #endif
+#endif
+    return S2N_RESULT_OK;
+}
+
+S2N_RESULT s2n_hash_algorithms_cleanup()
+{
+#if S2N_LIBCRYPTO_SUPPORTS_PROVIDERS
+    for (size_t i = 0; i < S2N_HASH_ALGS_COUNT; i++) {
+        /* https://docs.openssl.org/3.4/man3/EVP_DigestInit/
+         * > Decrements the reference count for the fetched EVP_MD structure.
+         * > If the reference count drops to 0 then the structure is freed.
+         * > If the argument is NULL, nothing is done.
+         */
+        EVP_MD_free(s2n_evp_mds[i]);
+        s2n_evp_mds[i] = NULL;
+    }
+#endif
+    return S2N_RESULT_OK;
 }
 
 const EVP_MD *s2n_hash_alg_to_evp_md(s2n_hash_algorithm alg)
 {
-    switch (alg) {
-        case S2N_HASH_MD5:
-            return EVP_md5();
-        case S2N_HASH_SHA1:
-            return EVP_sha1();
-        case S2N_HASH_SHA224:
-            return EVP_sha224();
-        case S2N_HASH_SHA256:
-            return EVP_sha256();
-        case S2N_HASH_SHA384:
-            return EVP_sha384();
-        case S2N_HASH_SHA512:
-            return EVP_sha512();
-#if defined(S2N_LIBCRYPTO_SUPPORTS_EVP_MD5_SHA1_HASH)
-        case S2N_HASH_MD5_SHA1:
-            return EVP_md5_sha1();
-#endif
-        default:
-            return NULL;
-    }
+    PTR_ENSURE_GTE(alg, 0);
+    PTR_ENSURE_LT(alg, S2N_HASH_ALGS_COUNT);
+    return s2n_evp_mds[alg];
 }
 
 int s2n_hash_digest_size(s2n_hash_algorithm alg, uint8_t *out)
@@ -113,8 +155,6 @@ bool s2n_hash_is_available(s2n_hash_algorithm alg)
     switch (alg) {
         case S2N_HASH_MD5:
         case S2N_HASH_MD5_SHA1:
-            /* return false if in FIPS mode, as MD5 algs are not available in FIPS mode. */
-            return !s2n_is_in_fips_mode();
         case S2N_HASH_NONE:
         case S2N_HASH_SHA1:
         case S2N_HASH_SHA224:
@@ -122,7 +162,7 @@ bool s2n_hash_is_available(s2n_hash_algorithm alg)
         case S2N_HASH_SHA384:
         case S2N_HASH_SHA512:
             return true;
-        case S2N_HASH_SENTINEL:
+        case S2N_HASH_ALGS_COUNT:
             return false;
     }
     return false;
@@ -134,160 +174,6 @@ int s2n_hash_is_ready_for_input(struct s2n_hash_state *state)
     return state->is_ready_for_input;
 }
 
-static int s2n_low_level_hash_new(struct s2n_hash_state *state)
-{
-    /* s2n_hash_new will always call the corresponding implementation of the s2n_hash
-     * being used. For the s2n_low_level_hash implementation, new is a no-op.
-     */
-
-    *state = (struct s2n_hash_state){ 0 };
-    return S2N_SUCCESS;
-}
-
-static int s2n_low_level_hash_init(struct s2n_hash_state *state, s2n_hash_algorithm alg)
-{
-    switch (alg) {
-        case S2N_HASH_NONE:
-            break;
-        case S2N_HASH_MD5:
-            POSIX_GUARD_OSSL(MD5_Init(&state->digest.low_level.md5), S2N_ERR_HASH_INIT_FAILED);
-            break;
-        case S2N_HASH_SHA1:
-            POSIX_GUARD_OSSL(SHA1_Init(&state->digest.low_level.sha1), S2N_ERR_HASH_INIT_FAILED);
-            break;
-        case S2N_HASH_SHA224:
-            POSIX_GUARD_OSSL(SHA224_Init(&state->digest.low_level.sha224), S2N_ERR_HASH_INIT_FAILED);
-            break;
-        case S2N_HASH_SHA256:
-            POSIX_GUARD_OSSL(SHA256_Init(&state->digest.low_level.sha256), S2N_ERR_HASH_INIT_FAILED);
-            break;
-        case S2N_HASH_SHA384:
-            POSIX_GUARD_OSSL(SHA384_Init(&state->digest.low_level.sha384), S2N_ERR_HASH_INIT_FAILED);
-            break;
-        case S2N_HASH_SHA512:
-            POSIX_GUARD_OSSL(SHA512_Init(&state->digest.low_level.sha512), S2N_ERR_HASH_INIT_FAILED);
-            break;
-        case S2N_HASH_MD5_SHA1:
-            POSIX_GUARD_OSSL(SHA1_Init(&state->digest.low_level.md5_sha1.sha1), S2N_ERR_HASH_INIT_FAILED);
-            POSIX_GUARD_OSSL(MD5_Init(&state->digest.low_level.md5_sha1.md5), S2N_ERR_HASH_INIT_FAILED);
-            break;
-
-        default:
-            POSIX_BAIL(S2N_ERR_HASH_INVALID_ALGORITHM);
-    }
-
-    state->alg = alg;
-    state->is_ready_for_input = 1;
-    state->currently_in_hash = 0;
-
-    return 0;
-}
-
-static int s2n_low_level_hash_update(struct s2n_hash_state *state, const void *data, uint32_t size)
-{
-    POSIX_ENSURE(state->is_ready_for_input, S2N_ERR_HASH_NOT_READY);
-
-    switch (state->alg) {
-        case S2N_HASH_NONE:
-            break;
-        case S2N_HASH_MD5:
-            POSIX_GUARD_OSSL(MD5_Update(&state->digest.low_level.md5, data, size), S2N_ERR_HASH_UPDATE_FAILED);
-            break;
-        case S2N_HASH_SHA1:
-            POSIX_GUARD_OSSL(SHA1_Update(&state->digest.low_level.sha1, data, size), S2N_ERR_HASH_UPDATE_FAILED);
-            break;
-        case S2N_HASH_SHA224:
-            POSIX_GUARD_OSSL(SHA224_Update(&state->digest.low_level.sha224, data, size), S2N_ERR_HASH_UPDATE_FAILED);
-            break;
-        case S2N_HASH_SHA256:
-            POSIX_GUARD_OSSL(SHA256_Update(&state->digest.low_level.sha256, data, size), S2N_ERR_HASH_UPDATE_FAILED);
-            break;
-        case S2N_HASH_SHA384:
-            POSIX_GUARD_OSSL(SHA384_Update(&state->digest.low_level.sha384, data, size), S2N_ERR_HASH_UPDATE_FAILED);
-            break;
-        case S2N_HASH_SHA512:
-            POSIX_GUARD_OSSL(SHA512_Update(&state->digest.low_level.sha512, data, size), S2N_ERR_HASH_UPDATE_FAILED);
-            break;
-        case S2N_HASH_MD5_SHA1:
-            POSIX_GUARD_OSSL(SHA1_Update(&state->digest.low_level.md5_sha1.sha1, data, size), S2N_ERR_HASH_UPDATE_FAILED);
-            POSIX_GUARD_OSSL(MD5_Update(&state->digest.low_level.md5_sha1.md5, data, size), S2N_ERR_HASH_UPDATE_FAILED);
-            break;
-        default:
-            POSIX_BAIL(S2N_ERR_HASH_INVALID_ALGORITHM);
-    }
-
-    POSIX_ENSURE(size <= (UINT64_MAX - state->currently_in_hash), S2N_ERR_INTEGER_OVERFLOW);
-    state->currently_in_hash += size;
-
-    return S2N_SUCCESS;
-}
-
-static int s2n_low_level_hash_digest(struct s2n_hash_state *state, void *out, uint32_t size)
-{
-    POSIX_ENSURE(state->is_ready_for_input, S2N_ERR_HASH_NOT_READY);
-
-    switch (state->alg) {
-        case S2N_HASH_NONE:
-            break;
-        case S2N_HASH_MD5:
-            POSIX_ENSURE_EQ(size, MD5_DIGEST_LENGTH);
-            POSIX_GUARD_OSSL(MD5_Final(out, &state->digest.low_level.md5), S2N_ERR_HASH_DIGEST_FAILED);
-            break;
-        case S2N_HASH_SHA1:
-            POSIX_ENSURE_EQ(size, SHA_DIGEST_LENGTH);
-            POSIX_GUARD_OSSL(SHA1_Final(out, &state->digest.low_level.sha1), S2N_ERR_HASH_DIGEST_FAILED);
-            break;
-        case S2N_HASH_SHA224:
-            POSIX_ENSURE_EQ(size, SHA224_DIGEST_LENGTH);
-            POSIX_GUARD_OSSL(SHA224_Final(out, &state->digest.low_level.sha224), S2N_ERR_HASH_DIGEST_FAILED);
-            break;
-        case S2N_HASH_SHA256:
-            POSIX_ENSURE_EQ(size, SHA256_DIGEST_LENGTH);
-            POSIX_GUARD_OSSL(SHA256_Final(out, &state->digest.low_level.sha256), S2N_ERR_HASH_DIGEST_FAILED);
-            break;
-        case S2N_HASH_SHA384:
-            POSIX_ENSURE_EQ(size, SHA384_DIGEST_LENGTH);
-            POSIX_GUARD_OSSL(SHA384_Final(out, &state->digest.low_level.sha384), S2N_ERR_HASH_DIGEST_FAILED);
-            break;
-        case S2N_HASH_SHA512:
-            POSIX_ENSURE_EQ(size, SHA512_DIGEST_LENGTH);
-            POSIX_GUARD_OSSL(SHA512_Final(out, &state->digest.low_level.sha512), S2N_ERR_HASH_DIGEST_FAILED);
-            break;
-        case S2N_HASH_MD5_SHA1:
-            POSIX_ENSURE_EQ(size, MD5_DIGEST_LENGTH + SHA_DIGEST_LENGTH);
-            POSIX_GUARD_OSSL(SHA1_Final(((uint8_t *) out) + MD5_DIGEST_LENGTH, &state->digest.low_level.md5_sha1.sha1), S2N_ERR_HASH_DIGEST_FAILED);
-            POSIX_GUARD_OSSL(MD5_Final(out, &state->digest.low_level.md5_sha1.md5), S2N_ERR_HASH_DIGEST_FAILED);
-            break;
-        default:
-            POSIX_BAIL(S2N_ERR_HASH_INVALID_ALGORITHM);
-    }
-
-    state->currently_in_hash = 0;
-    state->is_ready_for_input = 0;
-    return 0;
-}
-
-static int s2n_low_level_hash_copy(struct s2n_hash_state *to, struct s2n_hash_state *from)
-{
-    POSIX_CHECKED_MEMCPY(to, from, sizeof(struct s2n_hash_state));
-    return 0;
-}
-
-static int s2n_low_level_hash_reset(struct s2n_hash_state *state)
-{
-    /* hash_init resets the ready_for_input and currently_in_hash fields. */
-    return s2n_low_level_hash_init(state, state->alg);
-}
-
-static int s2n_low_level_hash_free(struct s2n_hash_state *state)
-{
-    /* s2n_hash_free will always call the corresponding implementation of the s2n_hash
-     * being used. For the s2n_low_level_hash implementation, free is a no-op.
-     */
-    state->is_ready_for_input = 0;
-    return S2N_SUCCESS;
-}
-
 static int s2n_evp_hash_new(struct s2n_hash_state *state)
 {
     POSIX_ENSURE_REF(state->digest.high_level.evp.ctx = S2N_EVP_MD_CTX_NEW());
@@ -301,20 +187,6 @@ static int s2n_evp_hash_new(struct s2n_hash_state *state)
     return S2N_SUCCESS;
 }
 
-static int s2n_evp_hash_allow_md5_for_fips(struct s2n_hash_state *state)
-{
-    /* This is only to be used for s2n_hash_states that will require MD5 to be used
-     * to comply with the TLS 1.0 and 1.1 RFC's for the PRF. MD5 cannot be used
-     * outside of the TLS 1.0 and 1.1 PRF when in FIPS mode. When needed, this must
-     * be called prior to s2n_hash_init().
-     */
-    POSIX_GUARD(s2n_digest_allow_md5_for_fips(&state->digest.high_level.evp));
-    if (s2n_use_custom_md5_sha1()) {
-        POSIX_GUARD(s2n_digest_allow_md5_for_fips(&state->digest.high_level.evp_md5_secondary));
-    }
-    return S2N_SUCCESS;
-}
-
 static int s2n_evp_hash_init(struct s2n_hash_state *state, s2n_hash_algorithm alg)
 {
     POSIX_ENSURE_REF(state->digest.high_level.evp.ctx);
@@ -329,13 +201,20 @@ static int s2n_evp_hash_init(struct s2n_hash_state *state, s2n_hash_algorithm al
 
     if (alg == S2N_HASH_MD5_SHA1 && s2n_use_custom_md5_sha1()) {
         POSIX_ENSURE_REF(state->digest.high_level.evp_md5_secondary.ctx);
-        POSIX_GUARD_OSSL(EVP_DigestInit_ex(state->digest.high_level.evp.ctx, EVP_sha1(), NULL), S2N_ERR_HASH_INIT_FAILED);
-        POSIX_GUARD_OSSL(EVP_DigestInit_ex(state->digest.high_level.evp_md5_secondary.ctx, EVP_md5(), NULL), S2N_ERR_HASH_INIT_FAILED);
+        POSIX_GUARD_OSSL(EVP_DigestInit_ex(state->digest.high_level.evp.ctx,
+                                 s2n_hash_alg_to_evp_md(S2N_HASH_SHA1), NULL),
+                S2N_ERR_HASH_INIT_FAILED);
+        POSIX_GUARD_OSSL(EVP_DigestInit_ex(state->digest.high_level.evp_md5_secondary.ctx,
+                                 s2n_hash_alg_to_evp_md(S2N_HASH_MD5), NULL),
+                S2N_ERR_HASH_INIT_FAILED);
         return S2N_SUCCESS;
     }
 
-    POSIX_ENSURE_REF(s2n_hash_alg_to_evp_md(alg));
-    POSIX_GUARD_OSSL(EVP_DigestInit_ex(state->digest.high_level.evp.ctx, s2n_hash_alg_to_evp_md(alg), NULL), S2N_ERR_HASH_INIT_FAILED);
+    const EVP_MD *md = s2n_hash_alg_to_evp_md(alg);
+    POSIX_ENSURE(md, S2N_ERR_HASH_INVALID_ALGORITHM);
+    POSIX_GUARD_OSSL(EVP_DigestInit_ex(state->digest.high_level.evp.ctx, md, NULL),
+            S2N_ERR_HASH_INIT_FAILED);
+
     return S2N_SUCCESS;
 }
 
@@ -419,32 +298,16 @@ static int s2n_evp_hash_copy(struct s2n_hash_state *to, struct s2n_hash_state *f
         POSIX_GUARD_OSSL(EVP_MD_CTX_copy_ex(to->digest.high_level.evp_md5_secondary.ctx, from->digest.high_level.evp_md5_secondary.ctx), S2N_ERR_HASH_COPY_FAILED);
     }
 
-    bool is_md5_allowed_for_fips = false;
-    POSIX_GUARD_RESULT(s2n_digest_is_md5_allowed_for_fips(&from->digest.high_level.evp, &is_md5_allowed_for_fips));
-    if (is_md5_allowed_for_fips && (from->alg == S2N_HASH_MD5 || from->alg == S2N_HASH_MD5_SHA1)) {
-        POSIX_GUARD(s2n_hash_allow_md5_for_fips(to));
-    }
     return S2N_SUCCESS;
 }
 
 static int s2n_evp_hash_reset(struct s2n_hash_state *state)
 {
-    int reset_md5_for_fips = 0;
-    bool is_md5_allowed_for_fips = false;
-    POSIX_GUARD_RESULT(s2n_digest_is_md5_allowed_for_fips(&state->digest.high_level.evp, &is_md5_allowed_for_fips));
-    if ((state->alg == S2N_HASH_MD5 || state->alg == S2N_HASH_MD5_SHA1) && is_md5_allowed_for_fips) {
-        reset_md5_for_fips = 1;
-    }
-
     POSIX_GUARD_OSSL(S2N_EVP_MD_CTX_RESET(state->digest.high_level.evp.ctx), S2N_ERR_HASH_WIPE_FAILED);
     if (state->alg == S2N_HASH_MD5_SHA1 && s2n_use_custom_md5_sha1()) {
         POSIX_GUARD_OSSL(S2N_EVP_MD_CTX_RESET(state->digest.high_level.evp_md5_secondary.ctx), S2N_ERR_HASH_WIPE_FAILED);
     }
 
-    if (reset_md5_for_fips) {
-        POSIX_GUARD(s2n_hash_allow_md5_for_fips(state));
-    }
-
     /* hash_init resets the ready_for_input and currently_in_hash fields. */
     return s2n_evp_hash_init(state, state->alg);
 }
@@ -463,20 +326,8 @@ static int s2n_evp_hash_free(struct s2n_hash_state *state)
     return S2N_SUCCESS;
 }
 
-static const struct s2n_hash s2n_low_level_hash = {
-    .alloc = &s2n_low_level_hash_new,
-    .allow_md5_for_fips = NULL,
-    .init = &s2n_low_level_hash_init,
-    .update = &s2n_low_level_hash_update,
-    .digest = &s2n_low_level_hash_digest,
-    .copy = &s2n_low_level_hash_copy,
-    .reset = &s2n_low_level_hash_reset,
-    .free = &s2n_low_level_hash_free,
-};
-
 static const struct s2n_hash s2n_evp_hash = {
     .alloc = &s2n_evp_hash_new,
-    .allow_md5_for_fips = &s2n_evp_hash_allow_md5_for_fips,
     .init = &s2n_evp_hash_init,
     .update = &s2n_evp_hash_update,
     .digest = &s2n_evp_hash_digest,
@@ -487,10 +338,7 @@ static const struct s2n_hash s2n_evp_hash = {
 
 static int s2n_hash_set_impl(struct s2n_hash_state *state)
 {
-    state->hash_impl = &s2n_low_level_hash;
-    if (s2n_use_evp_impl()) {
-        state->hash_impl = &s2n_evp_hash;
-    }
+    state->hash_impl = &s2n_evp_hash;
     return S2N_SUCCESS;
 }
 
@@ -514,19 +362,6 @@ S2N_RESULT s2n_hash_state_validate(struct s2n_hash_state *state)
     return S2N_RESULT_OK;
 }
 
-int s2n_hash_allow_md5_for_fips(struct s2n_hash_state *state)
-{
-    POSIX_ENSURE_REF(state);
-    /* Ensure that hash_impl is set, as it may have been reset for s2n_hash_state on s2n_connection_wipe.
-     * When in FIPS mode, the EVP API's must be used for hashes.
-     */
-    POSIX_GUARD(s2n_hash_set_impl(state));
-
-    POSIX_ENSURE_REF(state->hash_impl->allow_md5_for_fips);
-
-    return state->hash_impl->allow_md5_for_fips(state);
-}
-
 int s2n_hash_init(struct s2n_hash_state *state, s2n_hash_algorithm alg)
 {
     POSIX_ENSURE_REF(state);
@@ -535,15 +370,8 @@ int s2n_hash_init(struct s2n_hash_state *state, s2n_hash_algorithm alg)
      */
     POSIX_GUARD(s2n_hash_set_impl(state));
 
-    bool is_md5_allowed_for_fips = false;
-    POSIX_GUARD_RESULT(s2n_digest_is_md5_allowed_for_fips(&state->digest.high_level.evp, &is_md5_allowed_for_fips));
-
-    if (s2n_hash_is_available(alg) || ((alg == S2N_HASH_MD5 || alg == S2N_HASH_MD5_SHA1) && is_md5_allowed_for_fips)) {
-        /* s2n will continue to initialize an "unavailable" hash when s2n is in FIPS mode and
-         * FIPS is forcing the hash to be made available.
-         */
+    if (s2n_hash_is_available(alg)) {
         POSIX_ENSURE_REF(state->hash_impl->init);
-
         return state->hash_impl->init(state, alg);
     } else {
         POSIX_BAIL(S2N_ERR_HASH_INVALID_ALGORITHM);
