@@ -240,6 +240,7 @@ typedef enum {
     S2N_ERR_BAD_HEX,
     S2N_ERR_TEST_ASSERTION,
     S2N_ERR_CONFIG_NULL_BEFORE_CH_CALLBACK,
+    S2N_ERR_ZERO_LIFETIME_TICKET,
     S2N_ERR_T_INTERNAL_END,
 
     /* S2N_ERR_T_USAGE */
@@ -295,6 +296,7 @@ typedef enum {
     S2N_ERR_INVALID_SIGNATURE_ALGORITHMS_PREFERENCES,
     S2N_ERR_RSA_PSS_NOT_SUPPORTED,
     S2N_ERR_INVALID_ECC_PREFERENCES,
+    S2N_ERR_DEPRECATED_SECURITY_POLICY,
     S2N_ERR_INVALID_SECURITY_POLICY,
     S2N_ERR_INVALID_KEM_PREFERENCES,
     S2N_ERR_ASYNC_ALREADY_PERFORMED,
