@@ -1,12 +1,17 @@
+# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# SPDX-License-Identifier: Apache-2.0
 import copy
 import pytest
 
 from configuration import available_ports, TLS13_CIPHERS
-from common import ProviderOptions, Protocols, data_bytes
+from common import ProviderOptions, Protocols, random_str
 from fixtures import managed_process  # lgtm [py/unused-import]
 from providers import Provider, S2N, OpenSSL
 from utils import invalid_test_parameters, get_parameter_name
 
+SERVER_DATA = f"Some random data from the server:" + random_str(10)
+CLIENT_DATA = f"Some random data from the client:" + random_str(10)
+
 
 def test_nothing():
     """
@@ -28,8 +33,6 @@ def test_s2n_server_key_update(managed_process, cipher, provider, other_provider
     port = next(available_ports)
 
     update_requested = b"K"
-    server_data = data_bytes(10)
-    client_data = data_bytes(10)
     starting_marker = "Verify return code"
     key_update_marker = "KEYUPDATE"
 
@@ -40,7 +43,7 @@ def test_s2n_server_key_update(managed_process, cipher, provider, other_provider
         host=host,
         port=port,
         cipher=cipher,
-        data_to_send=[update_requested, client_data],
+        data_to_send=[update_requested, CLIENT_DATA.encode()],
         insecure=True,
         protocol=protocol,
     )
@@ -50,27 +53,27 @@ def test_s2n_server_key_update(managed_process, cipher, provider, other_provider
     server_options.mode = Provider.ServerMode
     server_options.key = "../pems/ecdsa_p384_pkcs1_key.pem"
     server_options.cert = "../pems/ecdsa_p384_pkcs1_cert.pem"
-    server_options.data_to_send = [server_data]
+    server_options.data_to_send = [SERVER_DATA.encode()]
 
     server = managed_process(
-        S2N, server_options, send_marker=[str(client_data)], timeout=30
+        S2N, server_options, send_marker=CLIENT_DATA, timeout=30
     )
     client = managed_process(
         provider,
         client_options,
         send_marker=send_marker_list,
-        close_marker=str(server_data),
+        close_marker=SERVER_DATA,
         timeout=30,
     )
 
     for results in client.get_results():
         results.assert_success()
         assert key_update_marker in str(results.stderr)
-        assert server_data in results.stdout
+        assert SERVER_DATA.encode() in results.stdout
 
     for results in server.get_results():
         results.assert_success()
-        assert client_data in results.stdout
+        assert CLIENT_DATA.encode() in results.stdout
 
 
 @pytest.mark.flaky(reruns=5)
@@ -84,8 +87,6 @@ def test_s2n_client_key_update(managed_process, cipher, provider, other_provider
     port = next(available_ports)
 
     update_requested = b"K\n"
-    server_data = data_bytes(10)
-    client_data = data_bytes(10)
     # Last statement printed out by Openssl after handshake
     starting_marker = "Secure Renegotiation IS supported"
     key_update_marker = "TLSv1.3 write server key update"
@@ -98,7 +99,7 @@ def test_s2n_client_key_update(managed_process, cipher, provider, other_provider
         host=host,
         port=port,
         cipher=cipher,
-        data_to_send=[client_data],
+        data_to_send=[CLIENT_DATA.encode()],
         insecure=True,
         protocol=protocol,
     )
@@ -108,28 +109,28 @@ def test_s2n_client_key_update(managed_process, cipher, provider, other_provider
     server_options.mode = Provider.ServerMode
     server_options.key = "../pems/ecdsa_p384_pkcs1_key.pem"
     server_options.cert = "../pems/ecdsa_p384_pkcs1_cert.pem"
-    server_options.data_to_send = [update_requested, server_data]
+    server_options.data_to_send = [update_requested, SERVER_DATA.encode()]
 
     server = managed_process(
         provider,
         server_options,
         send_marker=send_marker_list,
-        close_marker=str(client_data),
+        close_marker=CLIENT_DATA,
         timeout=30,
     )
     client = managed_process(
         S2N,
         client_options,
-        send_marker=[str(server_data)],
-        close_marker=str(server_data),
+        send_marker=SERVER_DATA,
+        close_marker=SERVER_DATA,
         timeout=30,
     )
 
     for results in client.get_results():
         results.assert_success()
-        assert server_data in results.stdout
+        assert SERVER_DATA.encode() in results.stdout
 
     for results in server.get_results():
         results.assert_success()
         assert read_key_update_marker in results.stderr
-        assert client_data in results.stdout
+        assert CLIENT_DATA.encode() in results.stdout
