@@ -33,8 +33,8 @@
 #define RSA_PKCS1_SIG_SCHEME         &s2n_rsa_pkcs1_md5_sha1
 #define RSA_PSS_PSS_SIG_SCHEME       &s2n_rsa_pss_pss_sha256
 #define RSA_PSS_RSAE_SIG_SCHEME      &s2n_rsa_pss_rsae_sha256
-#define ECDSA_SIG_SCHEME             &s2n_ecdsa_secp384r1_sha384
-#define ECDSA_SIG_SCHEME_OTHER_CURVE &s2n_ecdsa_secp256r1_sha256
+#define ECDSA_SIG_SCHEME             &s2n_ecdsa_sha384
+#define ECDSA_SIG_SCHEME_OTHER_CURVE &s2n_ecdsa_sha256
 
 #define EXPECT_SUCCESS_IF_RSA_PSS_CERTS_SUPPORTED(x) \
     if (s2n_is_rsa_pss_certs_supported()) {          \
@@ -47,7 +47,7 @@ static int s2n_test_auth_combo(struct s2n_connection *conn,
         struct s2n_cipher_suite *cipher_suite, const struct s2n_signature_scheme *sig_scheme,
         struct s2n_cert_chain_and_key *expected_cert_chain)
 {
-    struct s2n_cert_chain_and_key *actual_cert_chain;
+    struct s2n_cert_chain_and_key *actual_cert_chain = NULL;
 
     POSIX_GUARD(s2n_is_cipher_suite_valid_for_auth(conn, cipher_suite));
     conn->secure->cipher_suite = cipher_suite;
@@ -65,11 +65,11 @@ int main(int argc, char **argv)
     BEGIN_TEST();
     EXPECT_SUCCESS(s2n_disable_tls13_in_test());
 
-    struct s2n_cert_chain_and_key *rsa_cert_chain;
+    struct s2n_cert_chain_and_key *rsa_cert_chain = NULL;
     EXPECT_SUCCESS(s2n_test_cert_chain_and_key_new(&rsa_cert_chain,
             S2N_RSA_2048_PKCS1_CERT_CHAIN, S2N_RSA_2048_PKCS1_KEY));
 
-    struct s2n_cert_chain_and_key *ecdsa_cert_chain;
+    struct s2n_cert_chain_and_key *ecdsa_cert_chain = NULL;
     EXPECT_SUCCESS(s2n_test_cert_chain_and_key_new(&ecdsa_cert_chain,
             S2N_ECDSA_P384_PKCS1_CERT_CHAIN, S2N_ECDSA_P384_PKCS1_KEY));
 
@@ -102,31 +102,31 @@ int main(int argc, char **argv)
         /* Test: not valid if certs not available */
         {
             /* No certs exist */
-            s2n_connection_set_config(conn, no_certs_config);
+            EXPECT_SUCCESS(s2n_connection_set_config(conn, no_certs_config));
             EXPECT_FAILURE(s2n_is_cipher_suite_valid_for_auth(conn, RSA_AUTH_CIPHER_SUITE));
             EXPECT_FAILURE(s2n_is_cipher_suite_valid_for_auth(conn, ECDSA_AUTH_CIPHER_SUITE));
             EXPECT_SUCCESS(s2n_is_cipher_suite_valid_for_auth(conn, NO_AUTH_CIPHER_SUITE));
 
             /* RSA certs exist */
-            s2n_connection_set_config(conn, rsa_cert_config);
+            EXPECT_SUCCESS(s2n_connection_set_config(conn, rsa_cert_config));
             EXPECT_SUCCESS(s2n_is_cipher_suite_valid_for_auth(conn, RSA_AUTH_CIPHER_SUITE));
             EXPECT_FAILURE(s2n_is_cipher_suite_valid_for_auth(conn, ECDSA_AUTH_CIPHER_SUITE));
             EXPECT_SUCCESS(s2n_is_cipher_suite_valid_for_auth(conn, NO_AUTH_CIPHER_SUITE));
 
             /* RSA-PSS certs exist */
-            s2n_connection_set_config(conn, rsa_pss_cert_config);
+            EXPECT_SUCCESS(s2n_connection_set_config(conn, rsa_pss_cert_config));
             EXPECT_SUCCESS_IF_RSA_PSS_CERTS_SUPPORTED(s2n_is_cipher_suite_valid_for_auth(conn, RSA_AUTH_CIPHER_SUITE));
             EXPECT_FAILURE(s2n_is_cipher_suite_valid_for_auth(conn, ECDSA_AUTH_CIPHER_SUITE));
             EXPECT_SUCCESS(s2n_is_cipher_suite_valid_for_auth(conn, NO_AUTH_CIPHER_SUITE));
 
             /* ECDSA certs exist */
-            s2n_connection_set_config(conn, ecdsa_cert_config);
+            EXPECT_SUCCESS(s2n_connection_set_config(conn, ecdsa_cert_config));
             EXPECT_FAILURE(s2n_is_cipher_suite_valid_for_auth(conn, RSA_AUTH_CIPHER_SUITE));
             EXPECT_SUCCESS(s2n_is_cipher_suite_valid_for_auth(conn, ECDSA_AUTH_CIPHER_SUITE));
             EXPECT_SUCCESS(s2n_is_cipher_suite_valid_for_auth(conn, NO_AUTH_CIPHER_SUITE));
 
             /* All certs exist */
-            s2n_connection_set_config(conn, all_certs_config);
+            EXPECT_SUCCESS(s2n_connection_set_config(conn, all_certs_config));
             EXPECT_SUCCESS(s2n_is_cipher_suite_valid_for_auth(conn, RSA_AUTH_CIPHER_SUITE));
             EXPECT_SUCCESS(s2n_is_cipher_suite_valid_for_auth(conn, ECDSA_AUTH_CIPHER_SUITE));
             EXPECT_SUCCESS(s2n_is_cipher_suite_valid_for_auth(conn, NO_AUTH_CIPHER_SUITE));
@@ -144,44 +144,48 @@ int main(int argc, char **argv)
             conn->secure->cipher_suite = NO_AUTH_CIPHER_SUITE;
 
             /* No certs exist */
-            s2n_connection_set_config(conn, no_certs_config);
+            EXPECT_SUCCESS(s2n_connection_set_config(conn, no_certs_config));
             EXPECT_FAILURE(s2n_is_sig_scheme_valid_for_auth(conn, RSA_PKCS1_SIG_SCHEME));
             EXPECT_FAILURE(s2n_is_sig_scheme_valid_for_auth(conn, RSA_PSS_PSS_SIG_SCHEME));
             EXPECT_FAILURE(s2n_is_sig_scheme_valid_for_auth(conn, RSA_PSS_RSAE_SIG_SCHEME));
             EXPECT_FAILURE(s2n_is_sig_scheme_valid_for_auth(conn, ECDSA_SIG_SCHEME));
 
             /* RSA certs exist */
-            s2n_connection_set_config(conn, rsa_cert_config);
+            EXPECT_SUCCESS(s2n_connection_set_config(conn, rsa_cert_config));
             EXPECT_SUCCESS(s2n_is_sig_scheme_valid_for_auth(conn, RSA_PKCS1_SIG_SCHEME));
             EXPECT_FAILURE(s2n_is_sig_scheme_valid_for_auth(conn, RSA_PSS_PSS_SIG_SCHEME));
             EXPECT_SUCCESS(s2n_is_sig_scheme_valid_for_auth(conn, RSA_PSS_RSAE_SIG_SCHEME));
             EXPECT_FAILURE(s2n_is_sig_scheme_valid_for_auth(conn, ECDSA_SIG_SCHEME));
 
             /* RSA-PSS certs exist */
-            s2n_connection_set_config(conn, rsa_pss_cert_config);
+            EXPECT_SUCCESS(s2n_connection_set_config(conn, rsa_pss_cert_config));
             EXPECT_FAILURE(s2n_is_sig_scheme_valid_for_auth(conn, RSA_PKCS1_SIG_SCHEME));
             EXPECT_SUCCESS_IF_RSA_PSS_CERTS_SUPPORTED(s2n_is_sig_scheme_valid_for_auth(conn, RSA_PSS_PSS_SIG_SCHEME));
             EXPECT_FAILURE(s2n_is_sig_scheme_valid_for_auth(conn, RSA_PSS_RSAE_SIG_SCHEME));
             EXPECT_FAILURE(s2n_is_sig_scheme_valid_for_auth(conn, ECDSA_SIG_SCHEME));
 
             /* ECDSA certs exist */
-            s2n_connection_set_config(conn, ecdsa_cert_config);
+            EXPECT_SUCCESS(s2n_connection_set_config(conn, ecdsa_cert_config));
             EXPECT_FAILURE(s2n_is_sig_scheme_valid_for_auth(conn, RSA_PKCS1_SIG_SCHEME));
             EXPECT_FAILURE(s2n_is_sig_scheme_valid_for_auth(conn, RSA_PSS_PSS_SIG_SCHEME));
             EXPECT_FAILURE(s2n_is_sig_scheme_valid_for_auth(conn, RSA_PSS_RSAE_SIG_SCHEME));
             EXPECT_SUCCESS(s2n_is_sig_scheme_valid_for_auth(conn, ECDSA_SIG_SCHEME));
 
             /* All certs exist */
-            s2n_connection_set_config(conn, all_certs_config);
+            EXPECT_SUCCESS(s2n_connection_set_config(conn, all_certs_config));
             EXPECT_SUCCESS(s2n_is_sig_scheme_valid_for_auth(conn, RSA_PKCS1_SIG_SCHEME));
             EXPECT_SUCCESS_IF_RSA_PSS_CERTS_SUPPORTED(s2n_is_sig_scheme_valid_for_auth(conn, RSA_PSS_PSS_SIG_SCHEME));
             EXPECT_SUCCESS(s2n_is_sig_scheme_valid_for_auth(conn, RSA_PSS_RSAE_SIG_SCHEME));
             EXPECT_SUCCESS(s2n_is_sig_scheme_valid_for_auth(conn, ECDSA_SIG_SCHEME));
         }
 
-        /* Test: If signature algorithm specifies curve, must match cert curve */
+        /* Test: If signature algorithm is TLS1.3 ECDSA, must match cert curve */
         {
-            struct s2n_cert_chain_and_key *ecdsa_cert_chain_for_other_curve;
+            DEFER_CLEANUP(struct s2n_connection *test_conn = s2n_connection_new(S2N_CLIENT),
+                    s2n_connection_ptr_free);
+            test_conn->actual_protocol_version = S2N_TLS13;
+
+            struct s2n_cert_chain_and_key *ecdsa_cert_chain_for_other_curve = NULL;
             EXPECT_SUCCESS(s2n_test_cert_chain_and_key_new(&ecdsa_cert_chain_for_other_curve,
                     S2N_ECDSA_P256_PKCS1_CERT_CHAIN, S2N_ECDSA_P256_PKCS1_KEY));
 
@@ -189,15 +193,15 @@ int main(int argc, char **argv)
             EXPECT_SUCCESS(s2n_config_add_cert_chain_and_key_to_store(
                     ecdsa_cert_config_for_other_curve, ecdsa_cert_chain_for_other_curve));
 
-            conn->secure->cipher_suite = NO_AUTH_CIPHER_SUITE;
+            test_conn->secure->cipher_suite = NO_AUTH_CIPHER_SUITE;
 
-            s2n_connection_set_config(conn, ecdsa_cert_config);
-            EXPECT_SUCCESS(s2n_is_sig_scheme_valid_for_auth(conn, ECDSA_SIG_SCHEME));
-            EXPECT_FAILURE(s2n_is_sig_scheme_valid_for_auth(conn, ECDSA_SIG_SCHEME_OTHER_CURVE));
+            EXPECT_SUCCESS(s2n_connection_set_config(test_conn, ecdsa_cert_config));
+            EXPECT_SUCCESS(s2n_is_sig_scheme_valid_for_auth(test_conn, ECDSA_SIG_SCHEME));
+            EXPECT_FAILURE(s2n_is_sig_scheme_valid_for_auth(test_conn, ECDSA_SIG_SCHEME_OTHER_CURVE));
 
-            s2n_connection_set_config(conn, ecdsa_cert_config_for_other_curve);
-            EXPECT_FAILURE(s2n_is_sig_scheme_valid_for_auth(conn, ECDSA_SIG_SCHEME));
-            EXPECT_SUCCESS(s2n_is_sig_scheme_valid_for_auth(conn, ECDSA_SIG_SCHEME_OTHER_CURVE));
+            EXPECT_SUCCESS(s2n_connection_set_config(test_conn, ecdsa_cert_config_for_other_curve));
+            EXPECT_FAILURE(s2n_is_sig_scheme_valid_for_auth(test_conn, ECDSA_SIG_SCHEME));
+            EXPECT_SUCCESS(s2n_is_sig_scheme_valid_for_auth(test_conn, ECDSA_SIG_SCHEME_OTHER_CURVE));
 
             EXPECT_SUCCESS(s2n_config_free(ecdsa_cert_config_for_other_curve));
             EXPECT_SUCCESS(s2n_cert_chain_and_key_free(ecdsa_cert_chain_for_other_curve));
@@ -205,7 +209,7 @@ int main(int argc, char **argv)
 
         /* Test: If cipher suite specifies auth type, auth type must be valid for sig alg on server */
         {
-            s2n_connection_set_config(conn, all_certs_config);
+            EXPECT_SUCCESS(s2n_connection_set_config(conn, all_certs_config));
 
             /* RSA auth type */
             conn->secure->cipher_suite = RSA_AUTH_CIPHER_SUITE;
@@ -243,7 +247,7 @@ int main(int argc, char **argv)
 
         /* Test: RSA-PSS requires a non-ephemeral kex */
         {
-            s2n_connection_set_config(conn, all_certs_config);
+            EXPECT_SUCCESS(s2n_connection_set_config(conn, all_certs_config));
 
             /* ephemeral key */
             conn->secure->cipher_suite = &s2n_dhe_rsa_with_3des_ede_cbc_sha; /* kex = (dhe) */
@@ -298,10 +302,10 @@ int main(int argc, char **argv)
     /* s2n_select_certs_for_server_auth */
     {
         struct s2n_connection *conn = s2n_connection_new(S2N_SERVER);
-        struct s2n_cert_chain_and_key *chosen_certs;
+        struct s2n_cert_chain_and_key *chosen_certs = NULL;
 
         /* Requested cert chain exists */
-        s2n_connection_set_config(conn, all_certs_config);
+        EXPECT_SUCCESS(s2n_connection_set_config(conn, all_certs_config));
 
         struct s2n_signature_scheme test_sig_scheme = { 0 };
         conn->handshake_params.server_cert_sig_scheme = &test_sig_scheme;
@@ -326,7 +330,7 @@ int main(int argc, char **argv)
         EXPECT_EQUAL(chosen_certs, ecdsa_cert_chain);
 
         /* Requested cert chain does NOT exist */
-        s2n_connection_set_config(conn, no_certs_config);
+        EXPECT_SUCCESS(s2n_connection_set_config(conn, no_certs_config));
 
         /* cppcheck-suppress redundantAssignment */
         test_sig_scheme.sig_alg = S2N_SIGNATURE_RSA;
@@ -354,10 +358,10 @@ int main(int argc, char **argv)
     /* Test all possible combos */
     {
         struct s2n_connection *conn = s2n_connection_new(S2N_SERVER);
-        s2n_connection_set_config(conn, all_certs_config);
+        EXPECT_SUCCESS(s2n_connection_set_config(conn, all_certs_config));
 
         /* No certs exist */
-        s2n_connection_set_config(conn, no_certs_config);
+        EXPECT_SUCCESS(s2n_connection_set_config(conn, no_certs_config));
 
         EXPECT_FAILURE(s2n_test_auth_combo(conn, RSA_AUTH_CIPHER_SUITE, RSA_PKCS1_SIG_SCHEME, NULL));
         EXPECT_FAILURE(s2n_test_auth_combo(conn, RSA_AUTH_CIPHER_SUITE, RSA_PSS_PSS_SIG_SCHEME, NULL));
@@ -375,7 +379,7 @@ int main(int argc, char **argv)
         EXPECT_FAILURE(s2n_test_auth_combo(conn, NO_AUTH_CIPHER_SUITE, ECDSA_SIG_SCHEME, NULL));
 
         /* RSA certs exist */
-        s2n_connection_set_config(conn, rsa_cert_config);
+        EXPECT_SUCCESS(s2n_connection_set_config(conn, rsa_cert_config));
 
         EXPECT_SUCCESS(s2n_test_auth_combo(conn, RSA_AUTH_CIPHER_SUITE, RSA_PKCS1_SIG_SCHEME, rsa_cert_chain));
         EXPECT_FAILURE(s2n_test_auth_combo(conn, RSA_AUTH_CIPHER_SUITE, RSA_PSS_PSS_SIG_SCHEME, NULL));
@@ -393,7 +397,7 @@ int main(int argc, char **argv)
         EXPECT_FAILURE(s2n_test_auth_combo(conn, NO_AUTH_CIPHER_SUITE, ECDSA_SIG_SCHEME, NULL));
 
         /* RSA_PSS certs exist */
-        s2n_connection_set_config(conn, rsa_pss_cert_config);
+        EXPECT_SUCCESS(s2n_connection_set_config(conn, rsa_pss_cert_config));
 
         EXPECT_FAILURE(s2n_test_auth_combo(conn, RSA_AUTH_CIPHER_SUITE, RSA_PKCS1_SIG_SCHEME, NULL));
         EXPECT_SUCCESS_IF_RSA_PSS_CERTS_SUPPORTED(s2n_test_auth_combo(conn, RSA_AUTH_CIPHER_SUITE, RSA_PSS_PSS_SIG_SCHEME, rsa_pss_cert_chain));
@@ -411,7 +415,7 @@ int main(int argc, char **argv)
         EXPECT_FAILURE(s2n_test_auth_combo(conn, NO_AUTH_CIPHER_SUITE, ECDSA_SIG_SCHEME, NULL));
 
         /* ECDSA certs exist */
-        s2n_connection_set_config(conn, ecdsa_cert_config);
+        EXPECT_SUCCESS(s2n_connection_set_config(conn, ecdsa_cert_config));
 
         EXPECT_FAILURE(s2n_test_auth_combo(conn, RSA_AUTH_CIPHER_SUITE, RSA_PKCS1_SIG_SCHEME, NULL));
         EXPECT_FAILURE(s2n_test_auth_combo(conn, RSA_AUTH_CIPHER_SUITE, RSA_PSS_PSS_SIG_SCHEME, NULL));
@@ -429,7 +433,7 @@ int main(int argc, char **argv)
         EXPECT_SUCCESS(s2n_test_auth_combo(conn, NO_AUTH_CIPHER_SUITE, ECDSA_SIG_SCHEME, ecdsa_cert_chain));
 
         /* All certs exist */
-        s2n_connection_set_config(conn, all_certs_config);
+        EXPECT_SUCCESS(s2n_connection_set_config(conn, all_certs_config));
 
         EXPECT_SUCCESS(s2n_test_auth_combo(conn, RSA_AUTH_CIPHER_SUITE, RSA_PKCS1_SIG_SCHEME, rsa_cert_chain));
         EXPECT_SUCCESS_IF_RSA_PSS_CERTS_SUPPORTED(s2n_test_auth_combo(conn, RSA_AUTH_CIPHER_SUITE, RSA_PSS_PSS_SIG_SCHEME, rsa_pss_cert_chain));
