@@ -26,7 +26,7 @@ int main(int argc, char **argv)
 
     /* Test should_send */
     {
-        struct s2n_connection *conn;
+        struct s2n_connection *conn = NULL;
         EXPECT_NOT_NULL(conn = s2n_connection_new(S2N_CLIENT));
 
         EXPECT_SUCCESS(s2n_connection_set_protocol_preferences(conn, NULL, 0));
@@ -40,7 +40,7 @@ int main(int argc, char **argv)
 
     /* Test send */
     {
-        struct s2n_connection *conn;
+        struct s2n_connection *conn = NULL;
         EXPECT_NOT_NULL(conn = s2n_connection_new(S2N_CLIENT));
         EXPECT_SUCCESS(s2n_connection_set_protocol_preferences(conn, protocols, protocols_count));
 
@@ -50,7 +50,7 @@ int main(int argc, char **argv)
         EXPECT_SUCCESS(s2n_client_alpn_extension.send(conn, &stuffer));
 
         /* Should have correct size */
-        uint16_t actual_size;
+        uint16_t actual_size = 0;
         EXPECT_SUCCESS(s2n_stuffer_read_uint16(&stuffer, &actual_size));
         EXPECT_EQUAL(actual_size, s2n_stuffer_data_available(&stuffer));
         EXPECT_EQUAL(actual_size, conn->application_protocols_overridden.size);
@@ -66,7 +66,7 @@ int main(int argc, char **argv)
 
     /* Test receive can accept the output of send */
     {
-        struct s2n_connection *conn;
+        struct s2n_connection *conn = NULL;
         EXPECT_NOT_NULL(conn = s2n_connection_new(S2N_CLIENT));
         EXPECT_SUCCESS(s2n_connection_set_protocol_preferences(conn, protocols, protocols_count));
 
@@ -86,7 +86,7 @@ int main(int argc, char **argv)
 
     /* Test receive does nothing if no protocol preferences configured */
     {
-        struct s2n_connection *conn;
+        struct s2n_connection *conn = NULL;
         EXPECT_NOT_NULL(conn = s2n_connection_new(S2N_CLIENT));
 
         struct s2n_stuffer stuffer = { 0 };
@@ -104,7 +104,7 @@ int main(int argc, char **argv)
 
     /* Test receive does nothing if extension malformed */
     {
-        struct s2n_connection *conn;
+        struct s2n_connection *conn = NULL;
         EXPECT_NOT_NULL(conn = s2n_connection_new(S2N_CLIENT));
 
         struct s2n_stuffer stuffer = { 0 };
