@@ -32,7 +32,7 @@
 #define ZERO_TO_THIRTY_ONE 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, \
                            0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F
 
-static uint8_t server_ocsp_status[] = {
+static uint8_t server_ocsp_status_bytes[] = {
     /* clang-format off */
         0x30, 0x82, 0x06, 0x45, 0x0a, 0x01, 0x00, 0xa0, 0x82, 0x06, 0x3e, 0x30, 0x82, 0x06, 0x3a, 0x06, 0x09, 0x2b, 0x06, 0x01, 0x05, 0x05, 0x07, 0x30, 0x01, 0x01, 0x04, 0x82, 0x06, 0x2b, 0x30, 0x82, 0x06, 0x27, 0x30, 0x81, 0xeb, 0xa1, 0x70, 0x30, 0x6e, 0x31, 0x0b, 0x30, 0x09, 0x06, 0x03, 0x55, 0x04, 0x06, 0x13, 0x02, 0x49, 0x4c, 0x31, 0x31, 0x30, 0x2f, 0x06, 0x03, 0x55, 0x04, 0x0a, 0x13, 0x28, 0x53, 0x74, 0x61, 0x72, 0x74, 0x43, 0x6f, 0x6d, 0x20, 0x4c, 0x74, 0x64, 0x2e, 0x20, 0x28, 0x53, 0x74, 0x61, 0x72, 0x74, 0x20, 0x43, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x63, 0x69, 0x61, 0x6c, 0x20, 0x4c, 0x69, 0x6d, 0x69, 0x74, 0x65, 0x64, 0x29, 0x31, 0x2c, 0x30, 0x2a, 0x06, 0x03, 0x55, 0x04, 0x03, 0x13, 0x23, 0x53, 0x74, 0x61, 0x72,
         0x74, 0x43, 0x6f, 0x6d, 0x20, 0x43, 0x6c, 0x61, 0x73, 0x73, 0x20, 0x31, 0x20, 0x53, 0x65, 0x72, 0x76, 0x65, 0x72, 0x20, 0x4f, 0x43, 0x53, 0x50, 0x20, 0x53, 0x69, 0x67, 0x6e, 0x65, 0x72, 0x18, 0x0f, 0x32, 0x30, 0x31, 0x35, 0x30, 0x32, 0x32, 0x37, 0x30, 0x36, 0x34, 0x36, 0x34, 0x35, 0x5a, 0x30, 0x66, 0x30, 0x64, 0x30, 0x3c, 0x30, 0x09, 0x06, 0x05, 0x2b, 0x0e, 0x03, 0x02, 0x1a, 0x05, 0x00, 0x04, 0x14, 0x65, 0x68, 0x87, 0x4f, 0x40, 0x75, 0x0f, 0x01, 0x6a, 0x34, 0x75, 0x62, 0x5e, 0x1f, 0x5c, 0x93, 0xe5, 0xa2, 0x6d, 0x58, 0x04, 0x14, 0xeb, 0x42, 0x34, 0xd0, 0x98, 0xb0, 0xab, 0x9f, 0xf4, 0x1b, 0x6b, 0x08, 0xf7, 0xcc, 0x64, 0x2e, 0xef, 0x0e, 0x2c, 0x45, 0x02, 0x03, 0x0f, 0x87, 0x2c, 0x80, 0x00, 0x18, 0x0f, 0x32, 0x30,
@@ -59,8 +59,6 @@ static uint8_t sct_list[] = {
     0xff, 0xff, 0xff, 0xff, 0xff
 };
 
-message_type_t s2n_conn_get_current_message_type(struct s2n_connection *conn);
-
 /* Helper function to allow us to easily repeat the PQ extension test for many scenarios.
  * If the KEM negotiation is expected to fail (because of e.g. a client/server extension
  * mismatch), pass in expected_kem_id = -1. The tests should always EXPECT_SUCCESS when
@@ -69,17 +67,17 @@ static int negotiate_kem(const uint8_t client_extensions[], const size_t client_
         const uint8_t client_hello_message[], const size_t client_hello_len,
         const char cipher_pref_version[], const int expected_kem_id, struct s2n_test_io_pair *io_pair)
 {
-    char *cert_chain;
-    char *private_key;
+    char *cert_chain = NULL;
+    char *private_key = NULL;
 
     POSIX_GUARD_PTR(cert_chain = malloc(S2N_MAX_TEST_PEM_SIZE));
     POSIX_GUARD_PTR(private_key = malloc(S2N_MAX_TEST_PEM_SIZE));
     POSIX_GUARD(setenv("S2N_DONT_MLOCK", "1", 0));
 
-    struct s2n_connection *server_conn;
-    struct s2n_config *server_config;
+    struct s2n_connection *server_conn = NULL;
+    struct s2n_config *server_config = NULL;
     s2n_blocked_status server_blocked;
-    struct s2n_cert_chain_and_key *chain_and_key;
+    struct s2n_cert_chain_and_key *chain_and_key = NULL;
 
     size_t body_len = client_hello_len + client_extensions_len;
     uint8_t message_header[] = {
@@ -130,7 +128,7 @@ static int negotiate_kem(const uint8_t client_extensions[], const size_t client_
         return S2N_FAILURE;
     }
 
-    int negotiated_kem_id;
+    int negotiated_kem_id = 0;
 
     if (server_conn->kex_params.kem_params.kem != NULL) {
         negotiated_kem_id = server_conn->kex_params.kem_params.kem->kem_extension_id;
@@ -168,12 +166,12 @@ int main(int argc, char **argv)
 
     /* Client doesn't use the server name extension. */
     {
-        struct s2n_connection *client_conn;
-        struct s2n_connection *server_conn;
-        struct s2n_config *server_config;
-        struct s2n_cert_chain_and_key *chain_and_key;
+        struct s2n_connection *client_conn = NULL;
+        struct s2n_connection *server_conn = NULL;
+        struct s2n_config *server_config = NULL;
+        struct s2n_cert_chain_and_key *chain_and_key = NULL;
 
-        struct s2n_config *client_config;
+        struct s2n_config *client_config = NULL;
         EXPECT_NOT_NULL(client_config = s2n_config_new());
         EXPECT_SUCCESS(s2n_config_set_check_stapled_ocsp_response(client_config, 0));
         EXPECT_SUCCESS(s2n_config_disable_x509_verification(client_config));
@@ -215,15 +213,15 @@ int main(int argc, char **argv)
 
     /* Client uses the server name extension. */
     {
-        struct s2n_connection *client_conn;
-        struct s2n_connection *server_conn;
-        struct s2n_config *server_config;
-        struct s2n_cert_chain_and_key *chain_and_key;
+        struct s2n_connection *client_conn = NULL;
+        struct s2n_connection *server_conn = NULL;
+        struct s2n_config *server_config = NULL;
+        struct s2n_cert_chain_and_key *chain_and_key = NULL;
 
         const char *sent_server_name = "www.alligator.com";
-        const char *received_server_name;
+        const char *received_server_name = NULL;
 
-        struct s2n_config *client_config;
+        struct s2n_config *client_config = NULL;
         EXPECT_NOT_NULL(client_config = s2n_config_new());
         EXPECT_SUCCESS(s2n_config_set_check_stapled_ocsp_response(client_config, 0));
         EXPECT_SUCCESS(s2n_config_disable_x509_verification(client_config));
@@ -271,12 +269,12 @@ int main(int argc, char **argv)
 
     /* Client sends multiple server names. */
     {
-        struct s2n_connection *server_conn;
-        struct s2n_config *server_config;
+        struct s2n_connection *server_conn = NULL;
+        struct s2n_config *server_config = NULL;
         s2n_blocked_status server_blocked;
         const char *sent_server_name = "svr";
-        const char *received_server_name;
-        struct s2n_cert_chain_and_key *chain_and_key;
+        const char *received_server_name = NULL;
+        struct s2n_cert_chain_and_key *chain_and_key = NULL;
         uint32_t cert_chain_len = 0;
         uint32_t private_key_len = 0;
 
@@ -397,10 +395,10 @@ int main(int argc, char **argv)
 
     /* Client sends duplicate server name extension */
     {
-        struct s2n_connection *server_conn;
-        struct s2n_config *server_config;
+        struct s2n_connection *server_conn = NULL;
+        struct s2n_config *server_config = NULL;
         s2n_blocked_status server_blocked;
-        struct s2n_cert_chain_and_key *chain_and_key;
+        struct s2n_cert_chain_and_key *chain_and_key = NULL;
 
         uint8_t client_extensions[] = {
             /* Extension type TLS_EXTENSION_SERVER_NAME */
@@ -532,10 +530,10 @@ int main(int argc, char **argv)
 
     /* Client sends a valid initial renegotiation_info */
     {
-        struct s2n_connection *server_conn;
-        struct s2n_config *server_config;
+        struct s2n_connection *server_conn = NULL;
+        struct s2n_config *server_config = NULL;
         s2n_blocked_status server_blocked;
-        struct s2n_cert_chain_and_key *chain_and_key;
+        struct s2n_cert_chain_and_key *chain_and_key = NULL;
 
         uint8_t client_extensions[] = {
             /* Extension type TLS_EXTENSION_RENEGOTIATION_INFO */
@@ -632,10 +630,10 @@ int main(int argc, char **argv)
 
     /* Client sends a non-empty initial renegotiation_info */
     {
-        struct s2n_connection *server_conn;
-        struct s2n_config *server_config;
+        struct s2n_connection *server_conn = NULL;
+        struct s2n_config *server_config = NULL;
         s2n_blocked_status server_blocked;
-        struct s2n_cert_chain_and_key *chain_and_key;
+        struct s2n_cert_chain_and_key *chain_and_key = NULL;
         uint8_t buf[5120];
 
         uint8_t client_extensions[] = {
@@ -728,12 +726,12 @@ int main(int argc, char **argv)
 
     /* Client doesn't use the OCSP extension. */
     {
-        struct s2n_connection *client_conn;
-        struct s2n_connection *server_conn;
-        struct s2n_config *server_config;
-        uint32_t length;
+        struct s2n_connection *client_conn = NULL;
+        struct s2n_connection *server_conn = NULL;
+        struct s2n_config *server_config = NULL;
+        uint32_t length = 0;
 
-        struct s2n_config *client_config;
+        struct s2n_config *client_config = NULL;
         EXPECT_NOT_NULL(client_config = s2n_config_new());
         EXPECT_SUCCESS(s2n_config_set_check_stapled_ocsp_response(client_config, 0));
         EXPECT_SUCCESS(s2n_config_disable_x509_verification(client_config));
@@ -755,7 +753,7 @@ int main(int argc, char **argv)
         EXPECT_SUCCESS(s2n_read_test_pem(S2N_DEFAULT_TEST_PRIVATE_KEY, private_key, S2N_MAX_TEST_PEM_SIZE));
         EXPECT_SUCCESS(s2n_config_add_cert_chain_and_key(server_config, cert_chain, private_key));
         EXPECT_SUCCESS(s2n_config_set_extension_data(server_config, S2N_EXTENSION_OCSP_STAPLING,
-                server_ocsp_status, sizeof(server_ocsp_status)));
+                server_ocsp_status_bytes, sizeof(server_ocsp_status_bytes)));
         EXPECT_SUCCESS(s2n_connection_set_config(server_conn, server_config));
 
         EXPECT_SUCCESS(s2n_negotiate_test_server_and_client(server_conn, client_conn));
@@ -779,7 +777,7 @@ int main(int argc, char **argv)
 
     /* Cannot enable OCSP stapling if there's no support for it */
     if (!s2n_x509_ocsp_stapling_supported()) {
-        struct s2n_config *client_config;
+        struct s2n_config *client_config = NULL;
         EXPECT_NOT_NULL(client_config = s2n_config_new());
         EXPECT_FAILURE(s2n_config_set_check_stapled_ocsp_response(client_config, 1));
         EXPECT_SUCCESS(s2n_config_free(client_config));
@@ -787,12 +785,12 @@ int main(int argc, char **argv)
 
     /* Server doesn't support the OCSP extension. We can't run this test if ocsp isn't supported by the client. */
     if (s2n_x509_ocsp_stapling_supported()) {
-        struct s2n_connection *client_conn;
-        struct s2n_connection *server_conn;
-        struct s2n_config *server_config;
-        struct s2n_config *client_config;
-        uint32_t length;
-        struct s2n_cert_chain_and_key *chain_and_key;
+        struct s2n_connection *client_conn = NULL;
+        struct s2n_connection *server_conn = NULL;
+        struct s2n_config *server_config = NULL;
+        struct s2n_config *client_config = NULL;
+        uint32_t length = 0;
+        struct s2n_cert_chain_and_key *chain_and_key = NULL;
 
         EXPECT_NOT_NULL(client_config = s2n_config_new());
         EXPECT_SUCCESS(s2n_config_set_check_stapled_ocsp_response(client_config, 0));
@@ -844,12 +842,12 @@ int main(int argc, char **argv)
 
     /* Test with s2n_config_set_extension_data(). Can be removed once API is deprecated */
     if (s2n_x509_ocsp_stapling_supported()) {
-        struct s2n_connection *client_conn;
-        struct s2n_connection *server_conn;
-        struct s2n_config *server_config;
-        struct s2n_config *client_config;
-        const uint8_t *server_ocsp_reply;
-        uint32_t length;
+        struct s2n_connection *client_conn = NULL;
+        struct s2n_connection *server_conn = NULL;
+        struct s2n_config *server_config = NULL;
+        struct s2n_config *client_config = NULL;
+        const uint8_t *server_ocsp_reply = NULL;
+        uint32_t length = 0;
 
         EXPECT_NOT_NULL(client_config = s2n_config_new());
         EXPECT_SUCCESS(s2n_config_set_check_stapled_ocsp_response(client_config, 0));
@@ -864,7 +862,7 @@ int main(int argc, char **argv)
         EXPECT_SUCCESS(s2n_read_test_pem(S2N_DEFAULT_TEST_PRIVATE_KEY, private_key, S2N_MAX_TEST_PEM_SIZE));
         EXPECT_SUCCESS(s2n_config_add_cert_chain_and_key(server_config, cert_chain, private_key));
         EXPECT_SUCCESS(s2n_config_set_extension_data(server_config, S2N_EXTENSION_OCSP_STAPLING,
-                server_ocsp_status, sizeof(server_ocsp_status)));
+                server_ocsp_status_bytes, sizeof(server_ocsp_status_bytes)));
 
         EXPECT_NOT_NULL(server_conn = s2n_connection_new(S2N_SERVER));
         EXPECT_SUCCESS(s2n_connection_set_config(server_conn, server_config));
@@ -879,10 +877,10 @@ int main(int argc, char **argv)
         /* Verify that the client received an OCSP response. */
         EXPECT_EQUAL(s2n_connection_is_ocsp_stapled(client_conn), 1);
         EXPECT_NOT_NULL(server_ocsp_reply = s2n_connection_get_ocsp_response(client_conn, &length));
-        EXPECT_EQUAL(length, sizeof(server_ocsp_status));
+        EXPECT_EQUAL(length, sizeof(server_ocsp_status_bytes));
 
-        for (size_t i = 0; i < sizeof(server_ocsp_status); i++) {
-            EXPECT_EQUAL(server_ocsp_reply[i], server_ocsp_status[i]);
+        for (size_t i = 0; i < sizeof(server_ocsp_status_bytes); i++) {
+            EXPECT_EQUAL(server_ocsp_reply[i], server_ocsp_status_bytes[i]);
         }
 
         EXPECT_SUCCESS(s2n_shutdown_test_server_and_client(server_conn, client_conn));
@@ -895,12 +893,12 @@ int main(int argc, char **argv)
 
     /* Server and client support the OCSP extension. Test only runs if ocsp stapled responses are supported by the client */
     if (s2n_x509_ocsp_stapling_supported()) {
-        struct s2n_connection *client_conn;
-        struct s2n_connection *server_conn;
-        struct s2n_config *server_config;
-        struct s2n_config *client_config;
-        const uint8_t *server_ocsp_reply;
-        uint32_t length;
+        struct s2n_connection *client_conn = NULL;
+        struct s2n_connection *server_conn = NULL;
+        struct s2n_config *server_config = NULL;
+        struct s2n_config *client_config = NULL;
+        const uint8_t *server_ocsp_reply = NULL;
+        uint32_t length = 0;
 
         EXPECT_NOT_NULL(client_config = s2n_config_new());
         EXPECT_SUCCESS(s2n_config_set_check_stapled_ocsp_response(client_config, 0));
@@ -925,7 +923,7 @@ int main(int argc, char **argv)
         EXPECT_SUCCESS(s2n_read_test_pem(S2N_DEFAULT_TEST_PRIVATE_KEY, private_key, S2N_MAX_TEST_PEM_SIZE));
         EXPECT_SUCCESS(s2n_config_add_cert_chain_and_key(server_config, cert_chain, private_key));
         EXPECT_SUCCESS(s2n_config_set_extension_data(server_config, S2N_EXTENSION_OCSP_STAPLING,
-                server_ocsp_status, sizeof(server_ocsp_status)));
+                server_ocsp_status_bytes, sizeof(server_ocsp_status_bytes)));
         EXPECT_SUCCESS(s2n_connection_set_config(server_conn, server_config));
 
         EXPECT_SUCCESS(s2n_negotiate_test_server_and_client(server_conn, client_conn));
@@ -936,10 +934,10 @@ int main(int argc, char **argv)
         /* Verify that the client received an OCSP response. */
         EXPECT_EQUAL(s2n_connection_is_ocsp_stapled(client_conn), 1);
         EXPECT_NOT_NULL(server_ocsp_reply = s2n_connection_get_ocsp_response(client_conn, &length));
-        EXPECT_EQUAL(length, sizeof(server_ocsp_status));
+        EXPECT_EQUAL(length, sizeof(server_ocsp_status_bytes));
 
-        for (size_t i = 0; i < sizeof(server_ocsp_status); i++) {
-            EXPECT_EQUAL(server_ocsp_reply[i], server_ocsp_status[i]);
+        for (size_t i = 0; i < sizeof(server_ocsp_status_bytes); i++) {
+            EXPECT_EQUAL(server_ocsp_reply[i], server_ocsp_status_bytes[i]);
         }
 
         EXPECT_SUCCESS(s2n_shutdown_test_server_and_client(server_conn, client_conn));
@@ -953,12 +951,12 @@ int main(int argc, char **argv)
 
     /* Server and client support the OCSP extension. Test Behavior for TLS 1.3 */
     if (s2n_x509_ocsp_stapling_supported() && s2n_is_tls13_fully_supported()) {
-        struct s2n_connection *client_conn;
-        struct s2n_connection *server_conn;
-        struct s2n_config *server_config;
-        struct s2n_config *client_config;
-        const uint8_t *server_ocsp_reply;
-        uint32_t length;
+        struct s2n_connection *client_conn = NULL;
+        struct s2n_connection *server_conn = NULL;
+        struct s2n_config *server_config = NULL;
+        struct s2n_config *client_config = NULL;
+        const uint8_t *server_ocsp_reply = NULL;
+        uint32_t length = 0;
 
         EXPECT_SUCCESS(s2n_enable_tls13_in_test());
 
@@ -987,7 +985,7 @@ int main(int argc, char **argv)
         EXPECT_SUCCESS(s2n_read_test_pem(S2N_DEFAULT_ECDSA_TEST_PRIVATE_KEY, private_key, S2N_MAX_TEST_PEM_SIZE));
         EXPECT_SUCCESS(s2n_config_add_cert_chain_and_key(server_config, cert_chain, private_key));
         EXPECT_SUCCESS(s2n_config_set_extension_data(server_config, S2N_EXTENSION_OCSP_STAPLING,
-                server_ocsp_status, sizeof(server_ocsp_status)));
+                server_ocsp_status_bytes, sizeof(server_ocsp_status_bytes)));
         EXPECT_SUCCESS(s2n_connection_set_config(server_conn, server_config));
 
         EXPECT_SUCCESS(s2n_negotiate_test_server_and_client(server_conn, client_conn));
@@ -999,10 +997,10 @@ int main(int argc, char **argv)
         EXPECT_EQUAL(s2n_connection_is_ocsp_stapled(client_conn), 1);
 
         EXPECT_NOT_NULL(server_ocsp_reply = s2n_connection_get_ocsp_response(client_conn, &length));
-        EXPECT_EQUAL(length, sizeof(server_ocsp_status));
+        EXPECT_EQUAL(length, sizeof(server_ocsp_status_bytes));
 
-        for (size_t i = 0; i < sizeof(server_ocsp_status); i++) {
-            EXPECT_EQUAL(server_ocsp_reply[i], server_ocsp_status[i]);
+        for (size_t i = 0; i < sizeof(server_ocsp_status_bytes); i++) {
+            EXPECT_EQUAL(server_ocsp_reply[i], server_ocsp_status_bytes[i]);
         }
 
         EXPECT_SUCCESS(s2n_shutdown_test_server_and_client(server_conn, client_conn));
@@ -1018,13 +1016,13 @@ int main(int argc, char **argv)
 
     /* Client does not request SCT, but server is configured to serve them. */
     {
-        struct s2n_connection *client_conn;
-        struct s2n_connection *server_conn;
-        struct s2n_config *server_config;
+        struct s2n_connection *client_conn = NULL;
+        struct s2n_connection *server_conn = NULL;
+        struct s2n_config *server_config = NULL;
 
-        uint32_t length;
+        uint32_t length = 0;
 
-        struct s2n_config *client_config;
+        struct s2n_config *client_config = NULL;
         EXPECT_NOT_NULL(client_config = s2n_config_new());
         EXPECT_SUCCESS(s2n_config_set_check_stapled_ocsp_response(client_config, 0));
         EXPECT_SUCCESS(s2n_config_disable_x509_verification(client_config));
@@ -1066,12 +1064,12 @@ int main(int argc, char **argv)
 
     /* Client requests SCT and server does have it. */
     {
-        struct s2n_connection *client_conn;
-        struct s2n_connection *server_conn;
-        struct s2n_config *client_config;
-        struct s2n_config *server_config;
+        struct s2n_connection *client_conn = NULL;
+        struct s2n_connection *server_conn = NULL;
+        struct s2n_config *client_config = NULL;
+        struct s2n_config *server_config = NULL;
 
-        uint32_t length;
+        uint32_t length = 0;
 
         EXPECT_NOT_NULL(client_config = s2n_config_new());
         EXPECT_SUCCESS(s2n_config_set_check_stapled_ocsp_response(client_config, 0));
@@ -1118,13 +1116,13 @@ int main(int argc, char **argv)
 
     /* Client requests SCT and server does *not* have it. */
     {
-        struct s2n_connection *client_conn;
-        struct s2n_connection *server_conn;
-        struct s2n_config *client_config;
-        struct s2n_config *server_config;
-        struct s2n_cert_chain_and_key *chain_and_key;
+        struct s2n_connection *client_conn = NULL;
+        struct s2n_connection *server_conn = NULL;
+        struct s2n_config *client_config = NULL;
+        struct s2n_config *server_config = NULL;
+        struct s2n_cert_chain_and_key *chain_and_key = NULL;
 
-        uint32_t length;
+        uint32_t length = 0;
 
         EXPECT_NOT_NULL(client_config = s2n_config_new());
         EXPECT_SUCCESS(s2n_config_set_check_stapled_ocsp_response(client_config, 0));
@@ -1173,11 +1171,11 @@ int main(int argc, char **argv)
 
     /* Client requests 512, 1024, 2048, and 4096 maximum fragment lengths */
     for (uint8_t mfl_code = S2N_TLS_MAX_FRAG_LEN_512; mfl_code <= S2N_TLS_MAX_FRAG_LEN_4096; mfl_code++) {
-        struct s2n_connection *client_conn;
-        struct s2n_connection *server_conn;
-        struct s2n_config *server_config;
-        struct s2n_config *client_config;
-        struct s2n_cert_chain_and_key *chain_and_key;
+        struct s2n_connection *client_conn = NULL;
+        struct s2n_connection *server_conn = NULL;
+        struct s2n_config *server_config = NULL;
+        struct s2n_config *client_config = NULL;
+        struct s2n_cert_chain_and_key *chain_and_key = NULL;
 
         EXPECT_NOT_NULL(client_conn = s2n_connection_new(S2N_CLIENT));
         client_conn->actual_protocol_version = S2N_TLS12;
@@ -1226,11 +1224,11 @@ int main(int argc, char **argv)
 
     /* Client requests invalid maximum fragment length */
     {
-        struct s2n_connection *client_conn;
-        struct s2n_connection *server_conn;
-        struct s2n_config *server_config;
-        struct s2n_config *client_config;
-        struct s2n_cert_chain_and_key *chain_and_key;
+        struct s2n_connection *client_conn = NULL;
+        struct s2n_connection *server_conn = NULL;
+        struct s2n_config *server_config = NULL;
+        struct s2n_config *client_config = NULL;
+        struct s2n_cert_chain_and_key *chain_and_key = NULL;
 
         EXPECT_NOT_NULL(client_conn = s2n_connection_new(S2N_CLIENT));
         client_conn->actual_protocol_version = S2N_TLS12;
@@ -1276,11 +1274,11 @@ int main(int argc, char **argv)
 
     /* Server ignores client's request of S2N_TLS_MAX_FRAG_LEN_2048 maximum fragment length when accept_mfl is not set*/
     {
-        struct s2n_connection *client_conn;
-        struct s2n_connection *server_conn;
-        struct s2n_config *server_config;
-        struct s2n_config *client_config;
-        struct s2n_cert_chain_and_key *chain_and_key;
+        struct s2n_connection *client_conn = NULL;
+        struct s2n_connection *server_conn = NULL;
+        struct s2n_config *server_config = NULL;
+        struct s2n_config *client_config = NULL;
+        struct s2n_cert_chain_and_key *chain_and_key = NULL;
 
         EXPECT_NOT_NULL(client_conn = s2n_connection_new(S2N_CLIENT));
         client_conn->actual_protocol_version = S2N_TLS12;
