@@ -75,13 +75,12 @@ int main(int argc, char **argv)
 
     /* Test we can successfully receive an sslv2 client hello and set a
      * tls12 connection */
-    for (uint8_t i = 0; i < 2; i++) {
-        if (i == 1) {
-            EXPECT_SUCCESS(s2n_enable_tls13_in_test());
-        }
+    {
+        /* "enable" tls13, to test under default s2n-tls behavior */
+        EXPECT_SUCCESS(s2n_reset_tls13_in_test());
 
         EXPECT_NOT_NULL(server_conn = s2n_connection_new(S2N_SERVER));
-        EXPECT_SUCCESS(s2n_connection_set_config(server_conn, tls12_config));
+        EXPECT_SUCCESS(s2n_connection_set_config(server_conn, tls13_config));
 
         /* Record version and protocol version are in the header for SSLv2 */
         server_conn->client_hello_version = S2N_SSLv2;
@@ -102,7 +101,6 @@ int main(int argc, char **argv)
         EXPECT_SUCCESS(s2n_stuffer_write(&server_conn->handshake.io, &client_hello));
         EXPECT_SUCCESS(s2n_client_hello_recv(server_conn));
 
-        EXPECT_EQUAL(server_conn->server_protocol_version, i == 0 ? S2N_TLS12 : S2N_TLS13);
         EXPECT_EQUAL(server_conn->actual_protocol_version, S2N_TLS12);
         EXPECT_EQUAL(server_conn->client_protocol_version, S2N_TLS12);
         EXPECT_EQUAL(server_conn->client_hello_version, S2N_SSLv2);
