@@ -32,8 +32,8 @@
 
 int main(int argc, char **argv)
 {
-    char *cert_chain;
-    char *private_key;
+    char *cert_chain = NULL;
+    char *private_key = NULL;
     BEGIN_TEST();
     EXPECT_SUCCESS(s2n_disable_tls13_in_test());
 
@@ -43,8 +43,8 @@ int main(int argc, char **argv)
 
     /* Server negotiates TLS1.2 */
     {
-        struct s2n_connection *client_conn;
-        struct s2n_config *client_config;
+        struct s2n_connection *client_conn = NULL;
+        struct s2n_config *client_config = NULL;
         s2n_blocked_status client_blocked;
 
         uint8_t server_hello_message[] = {
@@ -177,8 +177,8 @@ int main(int argc, char **argv)
 
     /* Server negotiates SSLv3 */
     {
-        struct s2n_connection *client_conn;
-        struct s2n_config *client_config;
+        struct s2n_connection *client_conn = NULL;
+        struct s2n_config *client_config = NULL;
         s2n_blocked_status client_blocked;
 
         uint8_t server_hello_message[] = {
