@@ -49,7 +49,7 @@ int main(int argc, char **argv)
 
     uint8_t latest_version = S2N_TLS13;
 
-    struct s2n_config *config;
+    struct s2n_config *config = NULL;
     EXPECT_NOT_NULL(config = s2n_config_new());
 
     const struct s2n_security_policy *security_policy_with_tls13_and_earlier = &security_policy_20190801;
@@ -109,7 +109,7 @@ int main(int argc, char **argv)
 
     /* Client produces a version list that the server can parse */
     {
-        struct s2n_connection *client_conn;
+        struct s2n_connection *client_conn = NULL;
         EXPECT_NOT_NULL(client_conn = s2n_connection_new(S2N_CLIENT));
         EXPECT_SUCCESS(s2n_connection_set_config(client_conn, config));
 
@@ -118,7 +118,7 @@ int main(int argc, char **argv)
         uint16_t expected_length = size_result - S2N_EXTENSION_TYPE_FIELD_LENGTH - S2N_EXTENSION_LENGTH_FIELD_LENGTH;
 
         struct s2n_stuffer extension = { 0 };
-        s2n_stuffer_alloc(&extension, expected_length);
+        EXPECT_SUCCESS(s2n_stuffer_alloc(&extension, expected_length));
 
         EXPECT_SUCCESS(s2n_client_supported_versions_extension.send(client_conn, &extension));
 
@@ -126,7 +126,7 @@ int main(int argc, char **argv)
         EXPECT_EQUAL(expected_length, s2n_stuffer_data_available(&extension));
 
         /* Check that the server can process the version list */
-        struct s2n_connection *server_conn;
+        struct s2n_connection *server_conn = NULL;
         EXPECT_NOT_NULL(server_conn = s2n_connection_new(S2N_SERVER));
         EXPECT_SUCCESS(s2n_connection_set_config(server_conn, config));
 
@@ -143,7 +143,7 @@ int main(int argc, char **argv)
 
     /* Server selects highest supported version shared by client */
     {
-        struct s2n_connection *server_conn;
+        struct s2n_connection *server_conn = NULL;
         EXPECT_NOT_NULL(server_conn = s2n_connection_new(S2N_SERVER));
         EXPECT_SUCCESS(s2n_connection_set_config(server_conn, config));
 
@@ -152,7 +152,7 @@ int main(int argc, char **argv)
         uint8_t supported_version_list_length = sizeof(supported_version_list);
 
         struct s2n_stuffer extension = { 0 };
-        s2n_stuffer_alloc(&extension, supported_version_list_length * 2 + 1);
+        EXPECT_SUCCESS(s2n_stuffer_alloc(&extension, supported_version_list_length * 2 + 1));
 
         EXPECT_SUCCESS(write_test_supported_versions_list(&extension, supported_version_list,
                 supported_version_list_length));
@@ -169,7 +169,7 @@ int main(int argc, char **argv)
     /* Server does not process the extension if using TLS1.2. */
     {
         EXPECT_SUCCESS(s2n_disable_tls13_in_test());
-        struct s2n_connection *server_conn;
+        struct s2n_connection *server_conn = NULL;
         EXPECT_NOT_NULL(server_conn = s2n_connection_new(S2N_SERVER));
         EXPECT_SUCCESS(s2n_connection_set_config(server_conn, config));
 
@@ -178,7 +178,7 @@ int main(int argc, char **argv)
         uint8_t supported_version_list_length = sizeof(supported_version_list);
 
         struct s2n_stuffer extension = { 0 };
-        s2n_stuffer_alloc(&extension, supported_version_list_length * 2 + 1);
+        EXPECT_SUCCESS(s2n_stuffer_alloc(&extension, supported_version_list_length * 2 + 1));
 
         EXPECT_SUCCESS(write_test_supported_versions_list(&extension, supported_version_list,
                 supported_version_list_length));
@@ -195,7 +195,7 @@ int main(int argc, char **argv)
 
     /* Server terminates connection if there are no supported version in the list */
     {
-        struct s2n_connection *server_conn;
+        struct s2n_connection *server_conn = NULL;
         EXPECT_NOT_NULL(server_conn = s2n_connection_new(S2N_SERVER));
         EXPECT_SUCCESS(s2n_connection_set_config(server_conn, config));
 
@@ -203,7 +203,7 @@ int main(int argc, char **argv)
         uint8_t invalid_version_list_length = s2n_array_len(invalid_version_list);
 
         struct s2n_stuffer extension = { 0 };
-        s2n_stuffer_alloc(&extension, invalid_version_list_length * S2N_TLS_PROTOCOL_VERSION_LEN + 1);
+        EXPECT_SUCCESS(s2n_stuffer_alloc(&extension, invalid_version_list_length * S2N_TLS_PROTOCOL_VERSION_LEN + 1));
 
         POSIX_GUARD(s2n_stuffer_write_uint8(&extension, invalid_version_list_length * S2N_TLS_PROTOCOL_VERSION_LEN));
 
@@ -220,7 +220,7 @@ int main(int argc, char **argv)
 
     /* Check grease values for the supported versions */
     {
-        struct s2n_connection *server_conn;
+        struct s2n_connection *server_conn = NULL;
         EXPECT_NOT_NULL(server_conn = s2n_connection_new(S2N_SERVER));
         EXPECT_SUCCESS(s2n_connection_set_config(server_conn, config));
 
@@ -228,7 +228,7 @@ int main(int argc, char **argv)
         uint8_t grease_version_list_length = s2n_array_len(grease_version_list);
 
         struct s2n_stuffer extension = { 0 };
-        s2n_stuffer_alloc(&extension, grease_version_list_length * S2N_TLS_PROTOCOL_VERSION_LEN + 1);
+        EXPECT_SUCCESS(s2n_stuffer_alloc(&extension, grease_version_list_length * S2N_TLS_PROTOCOL_VERSION_LEN + 1));
 
         POSIX_GUARD(s2n_stuffer_write_uint8(&extension, grease_version_list_length * S2N_TLS_PROTOCOL_VERSION_LEN));
 
@@ -247,7 +247,7 @@ int main(int argc, char **argv)
 
     /* Server selects highest supported protocol among list of invalid protocols (that purposefully test our conversion methods) */
     {
-        struct s2n_connection *server_conn;
+        struct s2n_connection *server_conn = NULL;
         EXPECT_NOT_NULL(server_conn = s2n_connection_new(S2N_SERVER));
         EXPECT_SUCCESS(s2n_connection_set_config(server_conn, config));
 
@@ -255,7 +255,7 @@ int main(int argc, char **argv)
         uint8_t invalid_version_list_length = s2n_array_len(invalid_version_list);
 
         struct s2n_stuffer extension = { 0 };
-        s2n_stuffer_alloc(&extension, invalid_version_list_length * S2N_TLS_PROTOCOL_VERSION_LEN + 1);
+        EXPECT_SUCCESS(s2n_stuffer_alloc(&extension, invalid_version_list_length * S2N_TLS_PROTOCOL_VERSION_LEN + 1));
 
         POSIX_GUARD(s2n_stuffer_write_uint8(&extension, invalid_version_list_length * S2N_TLS_PROTOCOL_VERSION_LEN));
 
@@ -274,7 +274,7 @@ int main(int argc, char **argv)
 
     /* Server alerts if no shared supported version found */
     {
-        struct s2n_connection *server_conn;
+        struct s2n_connection *server_conn = NULL;
         EXPECT_NOT_NULL(server_conn = s2n_connection_new(S2N_SERVER));
         EXPECT_SUCCESS(s2n_connection_set_config(server_conn, config));
 
@@ -282,7 +282,7 @@ int main(int argc, char **argv)
         uint8_t supported_version_list_length = sizeof(supported_version_list);
 
         struct s2n_stuffer extension = { 0 };
-        s2n_stuffer_alloc(&extension, supported_version_list_length * 2 + 1);
+        EXPECT_SUCCESS(s2n_stuffer_alloc(&extension, supported_version_list_length * 2 + 1));
 
         EXPECT_SUCCESS(write_test_supported_versions_list(&extension, supported_version_list,
                 supported_version_list_length));
@@ -297,12 +297,12 @@ int main(int argc, char **argv)
 
     /* Server alerts if supported version list is empty */
     {
-        struct s2n_connection *server_conn;
+        struct s2n_connection *server_conn = NULL;
         EXPECT_NOT_NULL(server_conn = s2n_connection_new(S2N_SERVER));
         EXPECT_SUCCESS(s2n_connection_set_config(server_conn, config));
 
         struct s2n_stuffer extension = { 0 };
-        s2n_stuffer_alloc(&extension, 1);
+        EXPECT_SUCCESS(s2n_stuffer_alloc(&extension, 1));
 
         EXPECT_SUCCESS(s2n_stuffer_write_uint8(&extension, 0));
 
@@ -316,12 +316,12 @@ int main(int argc, char **argv)
 
     /* Server alerts if version list size exceeds the extension size */
     {
-        struct s2n_connection *server_conn;
+        struct s2n_connection *server_conn = NULL;
         EXPECT_NOT_NULL(server_conn = s2n_connection_new(S2N_SERVER));
         EXPECT_SUCCESS(s2n_connection_set_config(server_conn, config));
 
         struct s2n_stuffer extension = { 0 };
-        s2n_stuffer_alloc(&extension, 1);
+        EXPECT_SUCCESS(s2n_stuffer_alloc(&extension, 1));
 
         EXPECT_SUCCESS(s2n_stuffer_write_uint8(&extension, 13));
 
@@ -334,12 +334,12 @@ int main(int argc, char **argv)
 
     /* Server alerts if version list size is less than extension size */
     {
-        struct s2n_connection *server_conn;
+        struct s2n_connection *server_conn = NULL;
         EXPECT_NOT_NULL(server_conn = s2n_connection_new(S2N_SERVER));
         EXPECT_SUCCESS(s2n_connection_set_config(server_conn, config));
 
         struct s2n_stuffer extension = { 0 };
-        s2n_stuffer_alloc(&extension, 5);
+        EXPECT_SUCCESS(s2n_stuffer_alloc(&extension, 5));
 
         EXPECT_SUCCESS(s2n_stuffer_write_uint8(&extension, 2));
         EXPECT_SUCCESS(s2n_stuffer_write_uint16(&extension, 0x0302));
@@ -354,12 +354,12 @@ int main(int argc, char **argv)
 
     /* Server alerts if version list size is odd */
     {
-        struct s2n_connection *server_conn;
+        struct s2n_connection *server_conn = NULL;
         EXPECT_NOT_NULL(server_conn = s2n_connection_new(S2N_SERVER));
         EXPECT_SUCCESS(s2n_connection_set_config(server_conn, config));
 
         struct s2n_stuffer extension = { 0 };
-        s2n_stuffer_alloc(&extension, 4);
+        EXPECT_SUCCESS(s2n_stuffer_alloc(&extension, 4));
 
         EXPECT_SUCCESS(s2n_stuffer_write_uint8(&extension, 3));
         EXPECT_SUCCESS(s2n_stuffer_write_uint16(&extension, 0x0302));
@@ -376,13 +376,13 @@ int main(int argc, char **argv)
      * in the client hello, for backwards compatibility the version field
      * should be set to 1.2 even when a higher version is supported. */
     {
-        struct s2n_connection *conn;
+        struct s2n_connection *conn = NULL;
         EXPECT_NOT_NULL(conn = s2n_connection_new(S2N_CLIENT));
         EXPECT_SUCCESS(s2n_client_hello_send(conn));
 
         struct s2n_stuffer client_hello = conn->handshake.io;
         uint8_t version[2];
-        s2n_stuffer_read_bytes(&client_hello, version, 2);
+        EXPECT_SUCCESS(s2n_stuffer_read_bytes(&client_hello, version, 2));
 
         EXPECT_EQUAL(version[0], 0x03);
         EXPECT_EQUAL(version[1], 0x03);
@@ -391,7 +391,7 @@ int main(int argc, char **argv)
     };
 
     /**
-     *= https://tools.ietf.org/rfc/rfc8446#appendix-D.2
+     *= https://www.rfc-editor.org/rfc/rfc8446#appendix-D.2
      *= type=test
      *# A TLS server can also receive a ClientHello indicating a version number smaller than its highest supported
      *# version. If the "supported_versions" extension is present, the server MUST negotiate using that extension as
@@ -418,7 +418,7 @@ int main(int argc, char **argv)
         EXPECT_NOT_NULL(client_conn);
         EXPECT_SUCCESS(s2n_connection_set_config(client_conn, config_with_cert));
 
-        struct s2n_test_io_pair io_pair = { 0 };
+        DEFER_CLEANUP(struct s2n_test_io_pair io_pair = { 0 }, s2n_io_pair_close);
         EXPECT_SUCCESS(s2n_io_pair_init_non_blocking(&io_pair));
         EXPECT_SUCCESS(s2n_connections_set_io_pair(client_conn, server_conn, &io_pair));
 
@@ -443,6 +443,45 @@ int main(int argc, char **argv)
         EXPECT_EQUAL(server_conn->client_hello_version, S2N_TLS10);
     }
 
+    /* Ensure that TLS 1.3 is enforced in the supported versions extension for ClientHellos sent in
+     * response to a HelloRetryRequest
+     */
+    for (uint8_t version = S2N_TLS10; version <= S2N_TLS13; version++) {
+        for (size_t is_hrr = 0; is_hrr <= 1; is_hrr++) {
+            DEFER_CLEANUP(struct s2n_connection *conn = s2n_connection_new(S2N_SERVER), s2n_connection_ptr_free);
+            EXPECT_SUCCESS(s2n_connection_set_cipher_preferences(conn, "test_all"));
+            if (is_hrr) {
+                EXPECT_OK(s2n_handshake_type_set_tls13_flag(conn, HELLO_RETRY_REQUEST));
+                conn->handshake.message_number = 1;
+            }
+
+            uint8_t supported_versions_list[] = { version };
+            uint8_t supported_versions_list_len = sizeof(supported_versions_list);
+
+            DEFER_CLEANUP(struct s2n_stuffer extension = { 0 }, s2n_stuffer_free);
+            EXPECT_SUCCESS(s2n_stuffer_growable_alloc(&extension, 0));
+            EXPECT_SUCCESS(write_test_supported_versions_list(&extension, supported_versions_list,
+                    supported_versions_list_len));
+
+            int ret = s2n_client_supported_versions_extension.recv(conn, &extension);
+
+            if (is_hrr && version < S2N_TLS13) {
+                /* It should not be possible to negotiate a protocol version < TLS 1.3 after a
+                 * HelloRetryRequest.
+                 */
+                EXPECT_FAILURE_WITH_ERRNO(ret, S2N_ERR_PROTOCOL_VERSION_UNSUPPORTED);
+            } else {
+                /* TLS 1.3 should be permitted after a HelloRetryRequest. Also, it should be
+                 * possible to receive < TLS 1.3 in an initial ClientHello.
+                 */
+                EXPECT_SUCCESS(ret);
+
+                EXPECT_EQUAL(conn->client_protocol_version, version);
+                EXPECT_EQUAL(conn->actual_protocol_version, version);
+            }
+        }
+    }
+
     EXPECT_SUCCESS(s2n_config_free(config));
 
     END_TEST();
