@@ -142,9 +142,6 @@ int main(int argc, char **argv)
     uint8_t empty_md5_hash[MD5_DIGEST_LENGTH] = { 0 };
     DEFER_CLEANUP(struct s2n_hash_state md5_hash = { 0 }, s2n_hash_free);
     EXPECT_SUCCESS(s2n_hash_new(&md5_hash));
-    if (s2n_is_in_fips_mode()) {
-        EXPECT_SUCCESS(s2n_hash_allow_md5_for_fips(&md5_hash));
-    }
     EXPECT_SUCCESS(s2n_hash_init(&md5_hash, S2N_HASH_MD5));
     EXPECT_SUCCESS(s2n_hash_digest(&md5_hash, empty_md5_hash, MD5_DIGEST_LENGTH));
 
