@@ -161,7 +161,7 @@ int main(int argc, char **argv)
     EXPECT_SUCCESS(s2n_disable_tls13_in_test());
 
     /* Load the RSA cert */
-    struct s2n_cert_chain_and_key *rsa_cert_chain;
+    struct s2n_cert_chain_and_key *rsa_cert_chain = NULL;
     EXPECT_SUCCESS(s2n_test_cert_chain_and_key_new(&rsa_cert_chain,
             S2N_RSA_2048_PKCS1_CERT_CHAIN, S2N_RSA_2048_PKCS1_KEY));
 
@@ -181,7 +181,11 @@ int main(int argc, char **argv)
         hash_state_new(sign_hash, random_msg);
         hash_state_new(verify_hash, random_msg);
 
-        EXPECT_EQUAL(s2n_is_rsa_pss_signing_supported(), RSA_PSS_SIGNING_SUPPORTED);
+#if defined(S2N_LIBCRYPTO_SUPPORTS_RSA_PSS_SIGNING)
+        EXPECT_EQUAL(s2n_is_rsa_pss_signing_supported(), 1);
+#else
+        EXPECT_EQUAL(s2n_is_rsa_pss_signing_supported(), 0);
+#endif
 
         if (!s2n_is_rsa_pss_signing_supported()) {
             EXPECT_FAILURE_WITH_ERRNO(rsa_public_key.sign(rsa_cert_chain->private_key, S2N_SIGNATURE_RSA_PSS_RSAE, &sign_hash, &result),
@@ -198,7 +202,7 @@ int main(int argc, char **argv)
 
 #if RSA_PSS_CERTS_SUPPORTED
 
-    struct s2n_cert_chain_and_key *rsa_pss_cert_chain;
+    struct s2n_cert_chain_and_key *rsa_pss_cert_chain = NULL;
     EXPECT_SUCCESS(s2n_test_cert_chain_and_key_new(&rsa_pss_cert_chain,
             S2N_RSA_PSS_2048_SHA256_LEAF_CERT, S2N_RSA_PSS_2048_SHA256_LEAF_KEY));
 
@@ -241,14 +245,30 @@ int main(int argc, char **argv)
         /* Test: If they share the same RSA key,
          * an RSA cert and an RSA_PSS cert are equivalent for PSS signatures. */
         {
-            struct s2n_pkey rsa_pss_public_key;
-            s2n_pkey_type rsa_pss_pkey_type;
-            EXPECT_OK(s2n_asn1der_to_public_key_and_type(&rsa_pss_public_key, &rsa_pss_pkey_type, &rsa_pss_cert_chain->cert_chain->head->raw));
-            EXPECT_EQUAL(rsa_pss_pkey_type, S2N_PKEY_TYPE_RSA_PSS);
+            DEFER_CLEANUP(struct s2n_pkey rsa_public_key_as_pss = { 0 }, s2n_pkey_free);
+            s2n_pkey_type rsa_public_key_as_pss_type = S2N_PKEY_TYPE_UNKNOWN;
+            EXPECT_OK(s2n_asn1der_to_public_key_and_type(&rsa_public_key_as_pss, &rsa_public_key_as_pss_type,
+                    &rsa_cert_chain->cert_chain->head->raw));
+            EXPECT_EQUAL(rsa_public_key_as_pss_type, S2N_PKEY_TYPE_RSA);
 
-            /* Set the keys equal. */
-            RSA *rsa_key_copy = EVP_PKEY_get1_RSA(rsa_public_key.pkey);
-            POSIX_GUARD_OSSL(EVP_PKEY_set1_RSA(rsa_pss_public_key.pkey, rsa_key_copy), S2N_ERR_KEY_INIT);
+            DEFER_CLEANUP(struct s2n_pkey rsa_pss_public_key = { 0 }, s2n_pkey_free);
+            s2n_pkey_type rsa_pss_pkey_type_shared = S2N_PKEY_TYPE_UNKNOWN;
+            EXPECT_OK(s2n_asn1der_to_public_key_and_type(&rsa_pss_public_key, &rsa_pss_pkey_type_shared,
+                    &rsa_pss_cert_chain->cert_chain->head->raw));
+            EXPECT_EQUAL(rsa_pss_pkey_type_shared, S2N_PKEY_TYPE_RSA_PSS);
+
+            /* Set the keys equal.
+             *
+             * When EVP signing APIs are enabled, s2n-tls validates the signature algorithm type
+             * against the EVP pkey type, so the EVP pkey type must be RSA_PSS in order to use the
+             * RSA_PSS signature algorithm. However, EVP_PKEY_set1_RSA sets the EVP pkey type to
+             * RSA, even if the EVP pkey type was RSA_PSS (there is no EVP_PKEY_set1_RSA_PSS API).
+             *
+             * To ensure that the RSA_PSS EVP pkey type remains set to RSA_PSS, the RSA key is
+             * overridden instead of the RSA_PSS key, since its pkey type is RSA anyway.
+             */
+            RSA *rsa_key_copy = EVP_PKEY_get1_RSA(rsa_pss_public_key.pkey);
+            POSIX_GUARD_OSSL(EVP_PKEY_set1_RSA(rsa_public_key_as_pss.pkey, rsa_key_copy), S2N_ERR_KEY_INIT);
             RSA_free(rsa_key_copy);
 
             /* RSA signed with PSS, RSA_PSS verified with PSS */
@@ -256,8 +276,10 @@ int main(int argc, char **argv)
                 hash_state_new(sign_hash, random_msg);
                 hash_state_new(verify_hash, random_msg);
 
-                EXPECT_SUCCESS(rsa_public_key.sign(rsa_cert_chain->private_key, S2N_SIGNATURE_RSA_PSS_RSAE, &sign_hash, &result));
-                EXPECT_SUCCESS(rsa_pss_public_key.verify(&rsa_public_key, S2N_SIGNATURE_RSA_PSS_PSS, &verify_hash, &result));
+                EXPECT_SUCCESS(rsa_public_key_as_pss.sign(rsa_pss_cert_chain->private_key, S2N_SIGNATURE_RSA_PSS_RSAE,
+                        &sign_hash, &result));
+                EXPECT_SUCCESS(rsa_pss_public_key.verify(&rsa_pss_public_key, S2N_SIGNATURE_RSA_PSS_PSS,
+                        &verify_hash, &result));
             };
 
             /* RSA_PSS signed with PSS, RSA verified with PSS */
@@ -265,11 +287,11 @@ int main(int argc, char **argv)
                 hash_state_new(sign_hash, random_msg);
                 hash_state_new(verify_hash, random_msg);
 
-                EXPECT_SUCCESS(rsa_pss_public_key.sign(rsa_cert_chain->private_key, S2N_SIGNATURE_RSA_PSS_PSS, &sign_hash, &result));
-                EXPECT_SUCCESS(rsa_public_key.verify(&rsa_public_key, S2N_SIGNATURE_RSA_PSS_RSAE, &verify_hash, &result));
+                EXPECT_SUCCESS(rsa_pss_public_key.sign(rsa_pss_cert_chain->private_key, S2N_SIGNATURE_RSA_PSS_PSS,
+                        &sign_hash, &result));
+                EXPECT_SUCCESS(rsa_public_key_as_pss.verify(&rsa_public_key_as_pss, S2N_SIGNATURE_RSA_PSS_RSAE,
+                        &verify_hash, &result));
             };
-
-            EXPECT_SUCCESS(s2n_pkey_free(&rsa_pss_public_key));
         };
 
         EXPECT_SUCCESS(s2n_pkey_free(&rsa_public_key));
@@ -296,8 +318,8 @@ int main(int argc, char **argv)
         RSA_free(rsa_key_copy);
 
         struct s2n_stuffer message_stuffer = { 0 }, signature_stuffer = { 0 };
-        s2n_stuffer_alloc_ro_from_hex_string(&message_stuffer, test_case.message);
-        s2n_stuffer_alloc_ro_from_hex_string(&signature_stuffer, test_case.signature);
+        POSIX_GUARD_RESULT(s2n_stuffer_alloc_from_hex(&message_stuffer, test_case.message));
+        POSIX_GUARD_RESULT(s2n_stuffer_alloc_from_hex(&signature_stuffer, test_case.signature));
         hash_state_for_alg_new(verify_hash, test_case.hash_alg, message_stuffer.blob);
 
         int ret_val = rsa_public_key.verify(&rsa_public_key, S2N_SIGNATURE_RSA_PSS_RSAE,
