@@ -31,8 +31,8 @@
 
 int mock_client(struct s2n_test_io_pair *io_pair)
 {
-    struct s2n_connection *conn;
-    struct s2n_config *client_config;
+    struct s2n_connection *conn = NULL;
+    struct s2n_config *client_config = NULL;
     s2n_blocked_status blocked;
     int result = 0;
 
@@ -69,11 +69,11 @@ int main(int argc, char **argv)
      */
     {
         s2n_blocked_status blocked;
-        int status;
-        pid_t pid;
-        char *cert_chain_pem;
-        char *private_key_pem;
-        char *dhparams_pem;
+        int status = 0;
+        pid_t pid = 0;
+        char *cert_chain_pem = NULL;
+        char *private_key_pem = NULL;
+        char *dhparams_pem = NULL;
 
         /* For convenience, this test will intentionally try to write to closed pipes during shutdown. Ignore the signal to
         * avoid exiting the process on SIGPIPE.
@@ -131,7 +131,7 @@ int main(int argc, char **argv)
 
         /* Negotiate the handshake. */
         do {
-            int ret;
+            int ret = 0;
 
             ret = s2n_negotiate(conn, &blocked);
             EXPECT_TRUE(ret == 0 || (blocked && (errno == EAGAIN || errno == EWOULDBLOCK)));
@@ -146,7 +146,7 @@ int main(int argc, char **argv)
         /* Shutdown after negotiating */
         uint8_t server_shutdown = 0;
         do {
-            int ret;
+            int ret = 0;
 
             ret = s2n_shutdown(conn, &blocked);
             EXPECT_TRUE(ret == 0 || (blocked && (errno == EAGAIN || errno == EWOULDBLOCK)));
