@@ -81,7 +81,7 @@ int main(int argc, char **argv)
 
     /* Test s2n_extensions_server_key_share_send_size */
     {
-        struct s2n_connection *conn;
+        struct s2n_connection *conn = NULL;
         EXPECT_NOT_NULL(conn = s2n_connection_new(S2N_SERVER));
 
         EXPECT_EQUAL(0, s2n_extensions_server_key_share_send_size(conn));
@@ -200,8 +200,8 @@ int main(int argc, char **argv)
 
         int i = 0;
         do {
-            struct s2n_connection *server_send_conn;
-            struct s2n_connection *client_recv_conn;
+            struct s2n_connection *server_send_conn = NULL;
+            struct s2n_connection *client_recv_conn = NULL;
             EXPECT_NOT_NULL(server_send_conn = s2n_connection_new(S2N_SERVER));
             EXPECT_NOT_NULL(client_recv_conn = s2n_connection_new(S2N_CLIENT));
 
@@ -238,19 +238,20 @@ int main(int argc, char **argv)
         if (s2n_is_evp_apis_supported()) {
             /* Payload values were generated by connecting to openssl */
             const char *key_share_payloads[] = {
-                /* x25519 */
-                "001d00206b24ffd795c496899cd14b7742a5ffbdc453c23085a7f82f0ed1e0296adb9e0e",
                 /* p256 */
                 "001700410474cfd75c0ab7b57247761a277e1c92b5810dacb251bb758f43e9d15aaf292c4a2be43e886425ba55653ebb7a4f32fe368bacce3df00c618645cf1eb646f22552",
+                /* x25519 */
+                "001d00206b24ffd795c496899cd14b7742a5ffbdc453c23085a7f82f0ed1e0296adb9e0e",
                 /* p384 */
                 "00180061040a27264201368540483e97d324a3093e11a5862b0a1be0cf5d8510bc47ec285f5304e9ec3ba01a0c375c3b6fa4bd0ad44aae041bb776aebc7ee92462ad481fe86f8b6e3858d5c41d0f83b0404f711832a4119aec3da2eac86266f424b50aa212"
             };
 
             for (int i = 0; i < 3; i++) {
                 struct s2n_stuffer extension_stuffer = { 0 };
-                struct s2n_connection *client_conn;
+                struct s2n_connection *client_conn = NULL;
 
                 EXPECT_NOT_NULL(client_conn = s2n_connection_new(S2N_CLIENT));
+                EXPECT_SUCCESS(s2n_connection_set_cipher_preferences(client_conn, "20240503"));
 
                 const struct s2n_ecc_preferences *ecc_pref = NULL;
                 EXPECT_SUCCESS(s2n_connection_get_ecc_preferences(client_conn, &ecc_pref));
@@ -259,7 +260,7 @@ int main(int argc, char **argv)
                 const char *payload = key_share_payloads[i];
 
                 EXPECT_NULL(client_conn->kex_params.server_ecc_evp_params.negotiated_curve);
-                EXPECT_SUCCESS(s2n_stuffer_alloc_ro_from_hex_string(&extension_stuffer, payload));
+                EXPECT_OK(s2n_stuffer_alloc_from_hex(&extension_stuffer, payload));
 
                 client_conn->kex_params.client_ecc_evp_params.negotiated_curve = ecc_pref->ecc_curves[i];
                 EXPECT_SUCCESS(s2n_ecc_evp_generate_ephemeral_key(&client_conn->kex_params.client_ecc_evp_params));
@@ -276,7 +277,7 @@ int main(int argc, char **argv)
              * if tls1.3 not enabled */
             {
                 struct s2n_stuffer extension_stuffer = { 0 };
-                struct s2n_connection *client_conn;
+                struct s2n_connection *client_conn = NULL;
 
                 EXPECT_NOT_NULL(client_conn = s2n_connection_new(S2N_CLIENT));
 
@@ -289,7 +290,7 @@ int main(int argc, char **argv)
                 const char *payload = key_share_payloads[0];
 
                 EXPECT_NULL(client_conn->kex_params.server_ecc_evp_params.negotiated_curve);
-                EXPECT_SUCCESS(s2n_stuffer_alloc_ro_from_hex_string(&extension_stuffer, payload));
+                EXPECT_OK(s2n_stuffer_alloc_from_hex(&extension_stuffer, payload));
 
                 client_conn->kex_params.client_ecc_evp_params.negotiated_curve = ecc_pref->ecc_curves[0];
                 EXPECT_SUCCESS(s2n_ecc_evp_generate_ephemeral_key(&client_conn->kex_params.client_ecc_evp_params));
@@ -312,13 +313,13 @@ int main(int argc, char **argv)
         /* Test error handling parsing broken/trancated p256 key share */
         {
             struct s2n_stuffer extension_stuffer = { 0 };
-            struct s2n_connection *client_conn;
+            struct s2n_connection *client_conn = NULL;
 
             EXPECT_NOT_NULL(client_conn = s2n_connection_new(S2N_CLIENT));
             const char *p256 = "001700410474cfd75c0ab7b57247761a277e1c92b5810dacb251bb758f43e9d15aaf292c4a2be43e886425ba55653ebb7a4f32fe368bacce3df00c618645cf1eb6";
 
             EXPECT_NULL(client_conn->kex_params.server_ecc_evp_params.negotiated_curve);
-            EXPECT_SUCCESS(s2n_stuffer_alloc_ro_from_hex_string(&extension_stuffer, p256));
+            EXPECT_OK(s2n_stuffer_alloc_from_hex(&extension_stuffer, p256));
 
             EXPECT_FAILURE_WITH_ERRNO(s2n_server_key_share_extension.recv(client_conn, &extension_stuffer), S2N_ERR_BAD_KEY_SHARE);
 
@@ -329,7 +330,7 @@ int main(int argc, char **argv)
         /* Test failure for receiving p256 key share for client configured p384 key share */
         {
             struct s2n_stuffer extension_stuffer = { 0 };
-            struct s2n_connection *client_conn;
+            struct s2n_connection *client_conn = NULL;
 
             EXPECT_NOT_NULL(client_conn = s2n_connection_new(S2N_CLIENT));
             const struct s2n_ecc_preferences *ecc_pref = NULL;
@@ -339,7 +340,7 @@ int main(int argc, char **argv)
             const char *p256 = "001700410474cfd75c0ab7b57247761a277e1c92b5810dacb251bb758f43e9d15aaf292c4a2be43e886425ba55653ebb7a4f32fe368bacce3df00c618645cf1eb646f22552";
 
             EXPECT_NULL(client_conn->kex_params.server_ecc_evp_params.negotiated_curve);
-            EXPECT_SUCCESS(s2n_stuffer_alloc_ro_from_hex_string(&extension_stuffer, p256));
+            EXPECT_OK(s2n_stuffer_alloc_from_hex(&extension_stuffer, p256));
 
             /* If s2n_is_evp_apis_supported is not supported, the ecc_prefs->ecc_curves contains only p-256, p-384 curves. */
             int p_384_index = s2n_is_evp_apis_supported() ? 2 : 1;
@@ -356,7 +357,7 @@ int main(int argc, char **argv)
 
     /* Test Shared Key Generation */
     {
-        struct s2n_connection *client_conn, *server_conn;
+        struct s2n_connection *client_conn = NULL, *server_conn = NULL;
         struct s2n_stuffer key_share_extension = { 0 };
 
         EXPECT_NOT_NULL(client_conn = s2n_connection_new(S2N_CLIENT));
@@ -424,7 +425,7 @@ int main(int argc, char **argv)
 
     /* Test s2n_server_key_share_extension.send with supported curve not in s2n_ecc_preferences list selected */
     if (s2n_is_evp_apis_supported()) {
-        struct s2n_connection *conn;
+        struct s2n_connection *conn = NULL;
         EXPECT_NOT_NULL(conn = s2n_connection_new(S2N_SERVER));
         EXPECT_NOT_NULL(conn->config);
 
@@ -441,7 +442,7 @@ int main(int argc, char **argv)
 
     /* Test s2n_server_key_share_extension.recv with supported curve not in s2n_ecc_preferences list selected  */
     if (s2n_is_evp_apis_supported()) {
-        struct s2n_connection *conn;
+        struct s2n_connection *conn = NULL;
         EXPECT_NOT_NULL(conn = s2n_connection_new(S2N_CLIENT));
         struct s2n_stuffer *extension_stuffer = &conn->handshake.io;
 
@@ -464,8 +465,8 @@ int main(int argc, char **argv)
         /* For a HelloRetryRequest, we won't have a key share. We just have the server selected group/negotiated curve.
          * Test that s2n_server_key_share_extension.recv obtains the server negotiate curve successfully. */
         {
-            struct s2n_connection *client_conn;
-            struct s2n_connection *server_conn;
+            struct s2n_connection *client_conn = NULL;
+            struct s2n_connection *server_conn = NULL;
 
             EXPECT_NOT_NULL(server_conn = s2n_connection_new(S2N_SERVER));
             EXPECT_NOT_NULL(client_conn = s2n_connection_new(S2N_CLIENT));
@@ -555,11 +556,19 @@ int main(int argc, char **argv)
 
             /* If PQ is disabled, the client will not have sent PQ IDs/keyshares in the ClientHello;
              * if the server responded with a PQ keyshare, we should error. */
-            if (!s2n_pq_is_enabled()) {
-                struct s2n_connection *client_conn = NULL;
-                EXPECT_NOT_NULL(client_conn = s2n_connection_new(S2N_CLIENT));
+            {
+                DEFER_CLEANUP(struct s2n_connection *client_conn = s2n_connection_new(S2N_CLIENT),
+                        s2n_connection_ptr_free);
+                EXPECT_NOT_NULL(client_conn);
                 client_conn->security_policy_override = &test_security_policy;
 
+                /* The connection is set up to be receiving a HelloRetryRequest in order to exit
+                 * s2n_server_key_share_extension.recv earlier for testing the success case.
+                 */
+                client_conn->handshake.state_machine = S2N_STATE_MACHINE_TLS13;
+                EXPECT_SUCCESS(s2n_set_connection_hello_retry_flags(client_conn));
+                EXPECT_TRUE(s2n_is_hello_retry_message(client_conn));
+
                 uint8_t iana_buffer[2];
                 struct s2n_blob iana_blob = { 0 };
                 struct s2n_stuffer iana_stuffer = { 0 };
@@ -567,9 +576,12 @@ int main(int argc, char **argv)
                 EXPECT_SUCCESS(s2n_stuffer_init(&iana_stuffer, &iana_blob));
                 EXPECT_SUCCESS(s2n_stuffer_write_uint16(&iana_stuffer, test_security_policy.kem_preferences->tls13_kem_groups[0]->iana_id));
 
-                EXPECT_FAILURE_WITH_ERRNO(s2n_server_key_share_extension.recv(client_conn, &iana_stuffer), S2N_ERR_NO_SUPPORTED_LIBCRYPTO_API);
-
-                EXPECT_SUCCESS(s2n_connection_free(client_conn));
+                int ret = s2n_server_key_share_extension.recv(client_conn, &iana_stuffer);
+                if (s2n_pq_is_enabled()) {
+                    EXPECT_SUCCESS(ret);
+                } else {
+                    EXPECT_FAILURE_WITH_ERRNO(ret, S2N_ERR_ECDHE_UNSUPPORTED_CURVE);
+                }
             }
 
             /* Test s2n_server_key_share_extension.recv with KAT pq key shares */
@@ -647,7 +659,7 @@ int main(int argc, char **argv)
                     /* In the HRR, the server indicated p256+Kyber as it's choice in the key share extension */
                     const struct s2n_kem_group *kem_group = &s2n_secp256r1_kyber_512_r3;
                     DEFER_CLEANUP(struct s2n_stuffer key_share_payload = { 0 }, s2n_stuffer_free);
-                    EXPECT_SUCCESS(s2n_stuffer_alloc_ro_from_hex_string(&key_share_payload, "2F3A"));
+                    EXPECT_OK(s2n_stuffer_alloc_from_hex(&key_share_payload, "2F3A"));
 
                     /* Client should successfully parse the indicated group */
                     EXPECT_SUCCESS(s2n_server_key_share_extension.recv(client_conn, &key_share_payload));
@@ -682,21 +694,21 @@ int main(int argc, char **argv)
                         if (!s2n_kem_group_is_available(kem_group)) {
                             continue;
                         }
-                        struct s2n_connection *client_conn;
+                        struct s2n_connection *client_conn = NULL;
                         EXPECT_NOT_NULL(client_conn = s2n_connection_new(S2N_CLIENT));
                         client_conn->security_policy_override = &test_security_policy;
 
                         /* Server sends a named group identifier that isn't in the client's KEM preferences */
                         const char *bad_group = "2F2C"; /* IANA ID for secp256r1_threebears-babybear-r2 (not imported into s2n) */
                         DEFER_CLEANUP(struct s2n_stuffer bad_group_stuffer = { 0 }, s2n_stuffer_free);
-                        EXPECT_SUCCESS(s2n_stuffer_alloc_ro_from_hex_string(&bad_group_stuffer, bad_group));
+                        EXPECT_OK(s2n_stuffer_alloc_from_hex(&bad_group_stuffer, bad_group));
                         EXPECT_FAILURE_WITH_ERRNO(s2n_server_key_share_extension.recv(client_conn, &bad_group_stuffer),
                                 S2N_ERR_ECDHE_UNSUPPORTED_CURVE);
 
                         /* Server sends a key share that is in the client's KEM preferences, but client didn't send a key share */
                         const char *wrong_share = "2F1F"; /* Full extension truncated - not necessary */
                         DEFER_CLEANUP(struct s2n_stuffer wrong_share_stuffer = { 0 }, s2n_stuffer_free);
-                        EXPECT_SUCCESS(s2n_stuffer_alloc_ro_from_hex_string(&wrong_share_stuffer, wrong_share));
+                        EXPECT_OK(s2n_stuffer_alloc_from_hex(&wrong_share_stuffer, wrong_share));
                         EXPECT_FAILURE_WITH_ERRNO(s2n_server_key_share_extension.recv(client_conn, &wrong_share_stuffer),
                                 S2N_ERR_ECDHE_UNSUPPORTED_CURVE);
 
@@ -782,7 +794,7 @@ int main(int argc, char **argv)
             EXPECT_NOT_NULL(conn = s2n_connection_new(S2N_SERVER));
 
             if (!s2n_pq_is_enabled()) {
-                EXPECT_FAILURE_WITH_ERRNO(s2n_server_key_share_send_check_pq_hybrid(conn), S2N_ERR_NO_SUPPORTED_LIBCRYPTO_API);
+                EXPECT_FAILURE_WITH_ERRNO(s2n_server_key_share_send_check_pq_hybrid(conn), S2N_ERR_UNIMPLEMENTED);
             }
 
             if (s2n_pq_is_enabled()) {
@@ -874,15 +886,23 @@ int main(int argc, char **argv)
                     S2N_STUFFER_READ_EXPECT_EQUAL(&stuffer, kem_group->iana_id, uint16);
                     S2N_STUFFER_READ_EXPECT_EQUAL(&stuffer, expected_hybrid_share_size, uint16);
 
-                    if (len_prefixed) {
-                        S2N_STUFFER_READ_EXPECT_EQUAL(&stuffer, kem_group->curve->share_size, uint16);
+                    uint16_t expected_first_share_size = kem_group->curve->share_size;
+                    uint16_t expected_second_share_size = kem_group->kem->ciphertext_length;
+
+                    if (kem_group->send_kem_first) {
+                        expected_first_share_size = kem_group->kem->ciphertext_length;
+                        expected_second_share_size = kem_group->curve->share_size;
                     }
-                    EXPECT_SUCCESS(s2n_stuffer_skip_read(&stuffer, kem_group->curve->share_size));
 
                     if (len_prefixed) {
-                        S2N_STUFFER_READ_EXPECT_EQUAL(&stuffer, kem_group->kem->ciphertext_length, uint16);
+                        S2N_STUFFER_READ_EXPECT_EQUAL(&stuffer, expected_first_share_size, uint16);
                     }
-                    S2N_STUFFER_LENGTH_WRITTEN_EXPECT_EQUAL(&stuffer, kem_group->kem->ciphertext_length);
+                    EXPECT_SUCCESS(s2n_stuffer_skip_read(&stuffer, expected_first_share_size));
+
+                    if (len_prefixed) {
+                        S2N_STUFFER_READ_EXPECT_EQUAL(&stuffer, expected_second_share_size, uint16);
+                    }
+                    S2N_STUFFER_LENGTH_WRITTEN_EXPECT_EQUAL(&stuffer, expected_second_share_size);
 
                     EXPECT_NULL(conn->kex_params.server_ecc_evp_params.negotiated_curve);
                     EXPECT_EQUAL(server_params->kem_group, kem_group);
