@@ -449,7 +449,7 @@ int main(int argc, char **argv)
         conn->handshake.handshake_type = NEGOTIATED | FULL_HANDSHAKE | CLIENT_AUTH | NO_CLIENT_CERT | TLS12_PERFECT_FORWARD_SECRECY | OCSP_STATUS | WITH_SESSION_TICKET | WITH_NPN;
         EXPECT_STRING_EQUAL(all_flags_handshake_type_name, s2n_connection_get_handshake_type_name(conn));
 
-        const char *handshake_type_name;
+        const char *handshake_type_name = NULL;
         for (int i = 0; i < valid_tls12_handshakes_size; i++) {
             conn->handshake.handshake_type = valid_tls12_handshakes[i];
 
@@ -482,7 +482,7 @@ int main(int argc, char **argv)
 
         conn->handshake.handshake_type = test_handshake_type;
 
-        for (size_t i = 0; i < sizeof(expected) / sizeof(char *); i++) {
+        for (size_t i = 0; i < s2n_array_len(expected); i++) {
             conn->handshake.message_number = i;
             EXPECT_STRING_EQUAL(expected[i], s2n_connection_get_last_message_name(conn));
         }
@@ -492,12 +492,9 @@ int main(int argc, char **argv)
 
     /* Test: A WITH_NPN form of every valid, negotiated handshake exists */
     {
-        uint32_t handshake_type_original, handshake_type_npn;
-        message_type_t *messages_original, *messages_npn;
-
         for (size_t i = 0; i < valid_tls12_handshakes_size; i++) {
-            handshake_type_original = valid_tls12_handshakes[i];
-            messages_original = handshakes[handshake_type_original];
+            uint32_t handshake_type_original = valid_tls12_handshakes[i];
+            message_type_t *messages_original = handshakes[handshake_type_original];
 
             /* Ignore INITIAL and WITH_NPN handshakes */
             if (!(handshake_type_original & NEGOTIATED) || (handshake_type_original & WITH_NPN)) {
@@ -505,8 +502,8 @@ int main(int argc, char **argv)
             }
 
             /* Get the WITH_NPN form of the handshake */
-            handshake_type_npn = handshake_type_original | WITH_NPN;
-            messages_npn = handshakes[handshake_type_npn];
+            uint32_t handshake_type_npn = handshake_type_original | WITH_NPN;
+            message_type_t *messages_npn = handshakes[handshake_type_npn];
 
             for (size_t j = 0, j_npn = 0; j < S2N_MAX_HANDSHAKE_LENGTH && j_npn < S2N_MAX_HANDSHAKE_LENGTH; j++, j_npn++) {
                 /* The original handshake cannot contain the Next Protocol message */
