@@ -121,8 +121,8 @@ int main(int argc, char **argv)
      *       message and client CCS messages.
      */
     {
-        uint32_t original_handshake_type, early_data_handshake_type;
-        message_type_t *original_messages, *early_data_messages;
+        uint32_t original_handshake_type = 0, early_data_handshake_type = 0;
+        message_type_t *original_messages = NULL, *early_data_messages = NULL;
 
         for (size_t i = 0; i < valid_tls13_handshakes_size; i++) {
             original_handshake_type = valid_tls13_handshakes[i];
@@ -183,8 +183,8 @@ int main(int argc, char **argv)
     /* Test: A MIDDLEBOX_COMPAT form of every valid, negotiated handshake exists
      *       and matches the non-MIDDLEBOX_COMPAT form EXCEPT for CCS messages */
     {
-        uint32_t handshake_type_original, handshake_type_mc;
-        message_type_t *messages_original, *messages_mc;
+        uint32_t handshake_type_original = 0, handshake_type_mc = 0;
+        message_type_t *messages_original = NULL, *messages_mc = NULL;
 
         for (size_t i = 0; i < valid_tls13_handshakes_size; i++) {
             handshake_type_original = valid_tls13_handshakes[i];
@@ -218,8 +218,8 @@ int main(int argc, char **argv)
 
     /* Test: A non-FULL_HANDSHAKE form of every valid, negotiated handshake exists */
     {
-        uint32_t handshake_type_original, handshake_type_fh;
-        message_type_t *messages_original, *messages_fh;
+        uint32_t handshake_type_original = 0, handshake_type_fh = 0;
+        message_type_t *messages_original = NULL, *messages_fh = NULL;
 
         for (size_t i = 0; i < valid_tls13_handshakes_size; i++) {
             handshake_type_original = valid_tls13_handshakes[i];
@@ -264,8 +264,8 @@ int main(int argc, char **argv)
     /* Test: A EARLY_CLIENT_CCS form of every middlebox compatible handshake exists.
      * Any handshake could start with early data, even if that early data is later rejected. */
     {
-        uint32_t handshake_type_original, handshake_type_test;
-        message_type_t *messages_original, *messages_test;
+        uint32_t handshake_type_original = 0, handshake_type_test = 0;
+        message_type_t *messages_original = NULL, *messages_test = NULL;
 
         for (size_t i = 0; i < valid_tls13_handshakes_size; i++) {
             handshake_type_original = valid_tls13_handshakes[i];
@@ -646,7 +646,7 @@ int main(int argc, char **argv)
 
     /* Test: TLS 1.3 MIDDLEBOX_COMPAT handshakes all follow CCS middlebox compatibility rules.
      *
-     *= https://tools.ietf.org/rfc/rfc8446#appendix-D.4
+     *= https://www.rfc-editor.org/rfc/rfc8446#appendix-D.4
      *= type=test
      *# Field measurements [Ben17a] [Ben17b] [Res17a] [Res17b] have found
      *# that a significant number of middleboxes misbehave when a TLS
@@ -655,12 +655,8 @@ int main(int argc, char **argv)
      *# the TLS 1.3 handshake look more like a TLS 1.2 handshake:
      */
     {
-        bool change_cipher_spec_found;
-        uint32_t handshake_type;
-        message_type_t *messages;
-
         /*
-         *= https://tools.ietf.org/rfc/rfc8446#appendix-D.4
+         *= https://www.rfc-editor.org/rfc/rfc8446#appendix-D.4
          *= type=test
          *# If not offering early data, the client sends a dummy
          *# change_cipher_spec record (see the third paragraph of Section 5)
@@ -668,9 +664,9 @@ int main(int argc, char **argv)
          *# its second ClientHello or before its encrypted handshake flight.
          **/
         for (size_t i = 0; i < valid_tls13_handshakes_size; i++) {
-            change_cipher_spec_found = false;
-            handshake_type = valid_tls13_handshakes[i];
-            messages = tls13_handshakes[handshake_type];
+            bool change_cipher_spec_found = false;
+            uint32_t handshake_type = valid_tls13_handshakes[i];
+            message_type_t *messages = tls13_handshakes[handshake_type];
 
             /* Ignore INITIAL and non-MIDDLEBOX_COMPAT handshakes */
             if (!(handshake_type & NEGOTIATED)
@@ -699,14 +695,14 @@ int main(int argc, char **argv)
         }
 
         /**
-         *= https://tools.ietf.org/rfc/rfc8446#appendix-D.4
+         *= https://www.rfc-editor.org/rfc/rfc8446#appendix-D.4
          *= type=test
          *# If offering early data, the record is placed immediately after the
          *# first ClientHello.
          */
         for (size_t i = 0; i < valid_tls13_handshakes_size; i++) {
-            handshake_type = valid_tls13_handshakes[i];
-            messages = tls13_handshakes[handshake_type];
+            uint32_t handshake_type = valid_tls13_handshakes[i];
+            message_type_t *messages = tls13_handshakes[handshake_type];
 
             /* Ignore handshakes where early data did not trigger the change in CCS behavior */
             if (!(handshake_type & EARLY_CLIENT_CCS)) {
@@ -721,16 +717,16 @@ int main(int argc, char **argv)
         }
 
         /**
-         *= https://tools.ietf.org/rfc/rfc8446#appendix-D.4
+         *= https://www.rfc-editor.org/rfc/rfc8446#appendix-D.4
          *= type=test
          *# The server sends a dummy change_cipher_spec record immediately
          *# after its first handshake message.  This may either be after a
          *# ServerHello or a HelloRetryRequest.
          **/
         for (size_t i = 0; i < valid_tls13_handshakes_size; i++) {
-            change_cipher_spec_found = false;
-            handshake_type = valid_tls13_handshakes[i];
-            messages = tls13_handshakes[handshake_type];
+            bool change_cipher_spec_found = false;
+            uint32_t handshake_type = valid_tls13_handshakes[i];
+            message_type_t *messages = tls13_handshakes[handshake_type];
 
             /* Ignore INITIAL and non-MIDDLEBOX_COMPAT handshakes */
             if (!(handshake_type & NEGOTIATED) || !(handshake_type & MIDDLEBOX_COMPAT)) {
@@ -766,7 +762,7 @@ int main(int argc, char **argv)
         conn->session_ticket_status = S2N_NEW_TICKET;
 
         /* Ensure CLIENT_AUTH is set */
-        conn->config->client_cert_auth_type = S2N_CERT_AUTH_REQUIRED;
+        EXPECT_SUCCESS(s2n_connection_set_client_auth_type(conn, S2N_CERT_AUTH_REQUIRED));
 
         /* Ensure TLS12_PERFECT_FORWARD_SECRECY is set by choosing a cipher suite with is_ephemeral=1 on the kex */
         conn->secure->cipher_suite = &s2n_dhe_rsa_with_chacha20_poly1305_sha256;
@@ -937,7 +933,7 @@ int main(int argc, char **argv)
                 | MIDDLEBOX_COMPAT | WITH_EARLY_DATA | EARLY_CLIENT_CCS;
         EXPECT_STRING_EQUAL(all_flags_handshake_type_name, s2n_connection_get_handshake_type_name(conn));
 
-        const char *handshake_type_name;
+        const char *handshake_type_name = NULL;
         for (int i = 0; i < valid_tls13_handshakes_size; i++) {
             conn->handshake.handshake_type = valid_tls13_handshakes[i];
 
