@@ -81,7 +81,7 @@ int s2n_extensions_client_supported_versions_process(struct s2n_connection *conn
     uint8_t minimum_supported_version = s2n_unknown_protocol_version;
     POSIX_GUARD_RESULT(s2n_connection_get_minimum_supported_version(conn, &minimum_supported_version));
 
-    uint8_t size_of_version_list;
+    uint8_t size_of_version_list = 0;
     POSIX_GUARD(s2n_stuffer_read_uint8(extension, &size_of_version_list));
     S2N_ERROR_IF(size_of_version_list != s2n_stuffer_data_available(extension), S2N_ERR_BAD_MESSAGE);
     S2N_ERROR_IF(size_of_version_list % S2N_TLS_PROTOCOL_VERSION_LEN != 0, S2N_ERR_BAD_MESSAGE);
@@ -156,6 +156,13 @@ static int s2n_client_supported_versions_recv(struct s2n_connection *conn, struc
         return S2N_SUCCESS;
     }
 
+    /* A TLS 1.3 state machine flag is used to determine if a HelloRetryRequest is negotiated. A
+     * protocol version of TLS 1.3 must be set in order to query the TLS 1.3 state machine, so
+     * it must be queried before the protocol version is potentially reset due to processing the
+     * extension.
+     */
+    bool is_hrr_handshake = s2n_is_hello_retry_handshake(conn);
+
     s2n_result result = s2n_client_supported_versions_recv_impl(conn, extension);
     if (s2n_result_is_error(result)) {
         conn->client_protocol_version = s2n_unknown_protocol_version;
@@ -166,6 +173,14 @@ static int s2n_client_supported_versions_recv(struct s2n_connection *conn, struc
     }
     POSIX_GUARD_RESULT(result);
 
+    /* When the supported versions extension is received in a ClientHello sent in response to a
+     * HelloRetryRequest, ensure that TLS 1.3 is selected as the protocol version.
+     */
+    if (is_hrr_handshake && conn->handshake.message_number > 0) {
+        POSIX_ENSURE(conn->client_protocol_version == S2N_TLS13, S2N_ERR_PROTOCOL_VERSION_UNSUPPORTED);
+        POSIX_ENSURE(conn->actual_protocol_version == S2N_TLS13, S2N_ERR_PROTOCOL_VERSION_UNSUPPORTED);
+    }
+
     return S2N_SUCCESS;
 }
 
