@@ -34,22 +34,14 @@ const s2n_extension_type s2n_server_key_share_extension = {
     .if_missing = s2n_extension_noop_if_missing,
 };
 
-static int s2n_server_key_share_generate_pq_hybrid(struct s2n_connection *conn, struct s2n_stuffer *out)
+static int s2n_server_key_share_send_hybrid_partial_ecc(struct s2n_connection *conn, struct s2n_stuffer *out)
 {
-    POSIX_ENSURE_REF(out);
     POSIX_ENSURE_REF(conn);
-
-    POSIX_ENSURE(s2n_pq_is_enabled(), S2N_ERR_NO_SUPPORTED_LIBCRYPTO_API);
+    POSIX_ENSURE_REF(out);
 
     struct s2n_kem_group_params *server_kem_group_params = &conn->kex_params.server_kem_group_params;
     struct s2n_kem_params *client_kem_params = &conn->kex_params.client_kem_group_params.kem_params;
 
-    POSIX_ENSURE_REF(server_kem_group_params->kem_group);
-    POSIX_GUARD(s2n_stuffer_write_uint16(out, server_kem_group_params->kem_group->iana_id));
-
-    struct s2n_stuffer_reservation total_share_size = { 0 };
-    POSIX_GUARD(s2n_stuffer_reserve_uint16(out, &total_share_size));
-
     struct s2n_ecc_evp_params *server_ecc_params = &server_kem_group_params->ecc_params;
     POSIX_ENSURE_REF(server_ecc_params->negotiated_curve);
     if (client_kem_params->len_prefixed) {
@@ -58,12 +50,37 @@ static int s2n_server_key_share_generate_pq_hybrid(struct s2n_connection *conn,
     POSIX_GUARD(s2n_ecc_evp_generate_ephemeral_key(server_ecc_params));
     POSIX_GUARD(s2n_ecc_evp_write_params_point(server_ecc_params, out));
 
+    return S2N_SUCCESS;
+}
+
+static int s2n_server_key_share_generate_pq_hybrid(struct s2n_connection *conn, struct s2n_stuffer *out)
+{
+    POSIX_ENSURE_REF(out);
+    POSIX_ENSURE_REF(conn);
+
+    POSIX_ENSURE(s2n_pq_is_enabled(), S2N_ERR_UNIMPLEMENTED);
+
+    struct s2n_kem_group_params *server_kem_group_params = &conn->kex_params.server_kem_group_params;
+    struct s2n_kem_params *client_kem_params = &conn->kex_params.client_kem_group_params.kem_params;
     POSIX_ENSURE_REF(client_kem_params->public_key.data);
+
+    POSIX_ENSURE_REF(server_kem_group_params->kem_group);
+    POSIX_GUARD(s2n_stuffer_write_uint16(out, server_kem_group_params->kem_group->iana_id));
+
+    struct s2n_stuffer_reservation total_share_size = { 0 };
+    POSIX_GUARD(s2n_stuffer_reserve_uint16(out, &total_share_size));
+
     /* s2n_kem_send_ciphertext() will generate the PQ shared secret and use
      * the client's public key to encapsulate; the PQ shared secret will be
      * stored in client_kem_params, and will be used during the hybrid shared
      * secret derivation. */
-    POSIX_GUARD(s2n_kem_send_ciphertext(out, client_kem_params));
+    if (server_kem_group_params->kem_group->send_kem_first) {
+        POSIX_GUARD(s2n_kem_send_ciphertext(out, client_kem_params));
+        POSIX_GUARD(s2n_server_key_share_send_hybrid_partial_ecc(conn, out));
+    } else {
+        POSIX_GUARD(s2n_server_key_share_send_hybrid_partial_ecc(conn, out));
+        POSIX_GUARD(s2n_kem_send_ciphertext(out, client_kem_params));
+    }
 
     POSIX_GUARD(s2n_stuffer_write_vector_size(&total_share_size));
     return S2N_SUCCESS;
@@ -74,7 +91,7 @@ int s2n_server_key_share_send_check_pq_hybrid(struct s2n_connection *conn)
 {
     POSIX_ENSURE_REF(conn);
 
-    POSIX_ENSURE(s2n_pq_is_enabled(), S2N_ERR_NO_SUPPORTED_LIBCRYPTO_API);
+    POSIX_ENSURE(s2n_pq_is_enabled(), S2N_ERR_UNIMPLEMENTED);
 
     POSIX_ENSURE_REF(conn->kex_params.server_kem_group_params.kem_group);
     POSIX_ENSURE_REF(conn->kex_params.server_kem_group_params.kem_params.kem);
@@ -135,7 +152,7 @@ static int s2n_server_key_share_send(struct s2n_connection *conn, struct s2n_stu
     /* Retry requests only require the selected named group, not an actual share.
      * https://tools.ietf.org/html/rfc8446#section-4.2.8 */
     if (s2n_is_hello_retry_message(conn)) {
-        uint16_t named_group_id;
+        uint16_t named_group_id = 0;
         if (curve != NULL) {
             named_group_id = curve->iana_id;
         } else {
@@ -157,6 +174,32 @@ static int s2n_server_key_share_send(struct s2n_connection *conn, struct s2n_stu
     return S2N_SUCCESS;
 }
 
+static int s2n_server_key_share_recv_hybrid_partial_ecc(struct s2n_connection *conn, struct s2n_stuffer *extension)
+{
+    POSIX_ENSURE_REF(conn);
+    POSIX_ENSURE_REF(extension);
+
+    struct s2n_kem_params *client_kem_params = &conn->kex_params.client_kem_group_params.kem_params;
+    struct s2n_kem_group_params *server_kem_group_params = &conn->kex_params.server_kem_group_params;
+    const struct s2n_kem_group *server_kem_group = server_kem_group_params->kem_group;
+    POSIX_ENSURE_REF(server_kem_group);
+    uint16_t expected_ecc_share_size = server_kem_group->curve->share_size;
+
+    /* Parse ECC key share */
+    if (client_kem_params->len_prefixed) {
+        uint16_t actual_ecc_share_size = 0;
+        POSIX_GUARD(s2n_stuffer_read_uint16(extension, &actual_ecc_share_size));
+        POSIX_ENSURE(actual_ecc_share_size == expected_ecc_share_size, S2N_ERR_BAD_KEY_SHARE);
+    }
+
+    struct s2n_blob point_blob = { 0 };
+    POSIX_ENSURE(s2n_ecc_evp_read_params_point(extension, expected_ecc_share_size, &point_blob) == S2N_SUCCESS, S2N_ERR_BAD_KEY_SHARE);
+    POSIX_ENSURE(s2n_ecc_evp_parse_params_point(&point_blob, &server_kem_group_params->ecc_params) == S2N_SUCCESS, S2N_ERR_BAD_KEY_SHARE);
+    POSIX_ENSURE(server_kem_group_params->ecc_params.evp_pkey != NULL, S2N_ERR_BAD_KEY_SHARE);
+
+    return S2N_SUCCESS;
+}
+
 static int s2n_server_key_share_recv_pq_hybrid(struct s2n_connection *conn, uint16_t named_group_iana,
         struct s2n_stuffer *extension)
 {
@@ -165,7 +208,7 @@ static int s2n_server_key_share_recv_pq_hybrid(struct s2n_connection *conn, uint
 
     /* If PQ is disabled, the client should not have sent any PQ IDs
      * in the supported_groups list of the initial ClientHello */
-    POSIX_ENSURE(s2n_pq_is_enabled(), S2N_ERR_NO_SUPPORTED_LIBCRYPTO_API);
+    POSIX_ENSURE(s2n_pq_is_enabled(), S2N_ERR_ECDHE_UNSUPPORTED_CURVE);
 
     const struct s2n_kem_preferences *kem_pref = NULL;
     POSIX_GUARD(s2n_connection_get_kem_preferences(conn, &kem_pref));
@@ -211,23 +254,14 @@ static int s2n_server_key_share_recv_pq_hybrid(struct s2n_connection *conn, uint
     /* Don't need to call s2n_is_tls13_hybrid_kem_length_prefixed() to set client_kem_params->len_prefixed since we are
      * the client, and server-side should auto-detect hybrid share size and match our behavior. */
 
-    /* Parse ECC key share */
-    uint16_t expected_ecc_share_size = server_kem_group_params->kem_group->curve->share_size;
-    if (client_kem_params->len_prefixed) {
-        uint16_t actual_ecc_share_size = 0;
-        POSIX_GUARD(s2n_stuffer_read_uint16(extension, &actual_ecc_share_size));
-        POSIX_ENSURE(actual_ecc_share_size == expected_ecc_share_size, S2N_ERR_BAD_KEY_SHARE);
+    if (!server_kem_group_params->kem_group->send_kem_first) {
+        POSIX_ENSURE(s2n_server_key_share_recv_hybrid_partial_ecc(conn, extension) == S2N_SUCCESS, S2N_ERR_BAD_KEY_SHARE);
+        POSIX_ENSURE(s2n_kem_recv_ciphertext(extension, client_kem_params) == S2N_SUCCESS, S2N_ERR_BAD_KEY_SHARE);
+    } else {
+        POSIX_ENSURE(s2n_kem_recv_ciphertext(extension, client_kem_params) == S2N_SUCCESS, S2N_ERR_BAD_KEY_SHARE);
+        POSIX_ENSURE(s2n_server_key_share_recv_hybrid_partial_ecc(conn, extension) == S2N_SUCCESS, S2N_ERR_BAD_KEY_SHARE);
     }
 
-    struct s2n_blob point_blob = { 0 };
-    POSIX_ENSURE(s2n_ecc_evp_read_params_point(extension, expected_ecc_share_size, &point_blob) == S2N_SUCCESS, S2N_ERR_BAD_KEY_SHARE);
-    POSIX_ENSURE(s2n_ecc_evp_parse_params_point(&point_blob, &server_kem_group_params->ecc_params) == S2N_SUCCESS, S2N_ERR_BAD_KEY_SHARE);
-    POSIX_ENSURE(server_kem_group_params->ecc_params.evp_pkey != NULL, S2N_ERR_BAD_KEY_SHARE);
-
-    /* Parse the PQ KEM key share */
-    POSIX_ENSURE(s2n_kem_recv_ciphertext(extension, client_kem_params) == S2N_SUCCESS,
-            S2N_ERR_BAD_KEY_SHARE);
-
     return S2N_SUCCESS;
 }
 
@@ -258,7 +292,7 @@ static int s2n_server_key_share_recv_ecc(struct s2n_connection *conn, uint16_t n
     const struct s2n_ecc_named_curve *negotiated_curve = ecc_pref->ecc_curves[supported_curve_index];
 
     /**
-     *= https://tools.ietf.org/rfc/rfc8446#4.2.8
+     *= https://www.rfc-editor.org/rfc/rfc8446#4.2.8
      *# If using (EC)DHE key establishment and a HelloRetryRequest containing a
      *# "key_share" extension was received by the client, the client MUST
      *# verify that the selected NamedGroup in the ServerHello is the same as
@@ -291,7 +325,7 @@ static int s2n_server_key_share_recv_ecc(struct s2n_connection *conn, uint16_t n
     POSIX_ENSURE(client_ecc_evp_params->negotiated_curve == server_ecc_evp_params->negotiated_curve, S2N_ERR_BAD_KEY_SHARE);
     POSIX_ENSURE(client_ecc_evp_params->evp_pkey, S2N_ERR_BAD_KEY_SHARE);
 
-    uint16_t share_size;
+    uint16_t share_size = 0;
     S2N_ERROR_IF(s2n_stuffer_data_available(extension) < sizeof(share_size), S2N_ERR_BAD_KEY_SHARE);
     POSIX_GUARD(s2n_stuffer_read_uint16(extension, &share_size));
     S2N_ERROR_IF(s2n_stuffer_data_available(extension) < share_size, S2N_ERR_BAD_KEY_SHARE);
@@ -346,13 +380,13 @@ int s2n_extensions_server_key_share_select(struct s2n_connection *conn)
 {
     POSIX_ENSURE_REF(conn);
 
-    const struct s2n_ecc_preferences *ecc_pref = NULL;
-    POSIX_GUARD(s2n_connection_get_ecc_preferences(conn, &ecc_pref));
-    POSIX_ENSURE_REF(ecc_pref);
+    /* Get the client's preferred groups for the KeyShares that were actually sent by the client */
+    const struct s2n_ecc_named_curve *client_curve = conn->kex_params.client_ecc_evp_params.negotiated_curve;
+    const struct s2n_kem_group *client_kem_group = conn->kex_params.client_kem_group_params.kem_group;
 
-    const struct s2n_kem_preferences *kem_pref = NULL;
-    POSIX_GUARD(s2n_connection_get_kem_preferences(conn, &kem_pref));
-    POSIX_ENSURE_REF(kem_pref);
+    /* Get the server's preferred groups (which may or may not have been sent in the KeyShare by the client) */
+    const struct s2n_ecc_named_curve *server_curve = conn->kex_params.server_ecc_evp_params.negotiated_curve;
+    const struct s2n_kem_group *server_kem_group = conn->kex_params.server_kem_group_params.kem_group;
 
     /* Boolean XOR check. When receiving the supported_groups extension, s2n server
      * should (exclusively) set either server_curve or server_kem_group based on the
@@ -361,37 +395,48 @@ int s2n_extensions_server_key_share_select(struct s2n_connection *conn)
      * groups; key negotiation is not possible and the handshake should be aborted
      * without sending HRR. (The case of both being non-NULL should never occur, and
      * is an error.) */
-    const struct s2n_ecc_named_curve *server_curve = conn->kex_params.server_ecc_evp_params.negotiated_curve;
-    const struct s2n_kem_group *server_kem_group = conn->kex_params.server_kem_group_params.kem_group;
     POSIX_ENSURE((server_curve == NULL) != (server_kem_group == NULL), S2N_ERR_ECDHE_UNSUPPORTED_CURVE);
 
     /* To avoid extra round trips, we prefer to negotiate a group for which we have already
      * received a key share (even if it is different than the group previously chosen). In
      * general, we prefer to negotiate PQ over ECDHE; however, if both client and server
      * support PQ, but the client sent only EC key shares, then we will negotiate ECHDE. */
-    if (conn->kex_params.client_kem_group_params.kem_group) {
+
+    /* Option 1: Select the best mutually supported PQ Hybrid Group that can be negotiated in 1-RTT */
+    if (client_kem_group != NULL) {
         POSIX_ENSURE_REF(conn->kex_params.client_kem_group_params.ecc_params.negotiated_curve);
         POSIX_ENSURE_REF(conn->kex_params.client_kem_group_params.kem_params.kem);
 
         conn->kex_params.server_kem_group_params.kem_group = conn->kex_params.client_kem_group_params.kem_group;
         conn->kex_params.server_kem_group_params.ecc_params.negotiated_curve = conn->kex_params.client_kem_group_params.ecc_params.negotiated_curve;
         conn->kex_params.server_kem_group_params.kem_params.kem = conn->kex_params.client_kem_group_params.kem_params.kem;
-
         conn->kex_params.server_ecc_evp_params.negotiated_curve = NULL;
         return S2N_SUCCESS;
     }
 
-    if (conn->kex_params.client_ecc_evp_params.negotiated_curve) {
-        conn->kex_params.server_ecc_evp_params.negotiated_curve = conn->kex_params.client_ecc_evp_params.negotiated_curve;
+    /* Option 2: Otherwise, if any PQ Hybrid Groups can be negotiated in 2-RTT's select that one. This ensures that
+     * clients who offer PQ (and presumably therefore have concerns about quantum computing impacting the long term
+     * confidentiality of their data), have their choice to offer PQ respected, even if they predict the server-side
+     * supports a different PQ KeyShare algorithms. This ensures clients with PQ support are never downgraded to non-PQ
+     * algorithms. */
+    if (server_kem_group != NULL) {
+        /* Null out any available ECC curves so that they won't be sent in the ClientHelloRetry */
+        conn->kex_params.server_ecc_evp_params.negotiated_curve = NULL;
+        POSIX_GUARD(s2n_set_hello_retry_required(conn));
+        return S2N_SUCCESS;
+    }
 
+    /* Option 3: Otherwise, if there is a mutually supported classical ECDHE-only group can be negotiated in 1-RTT, select that one */
+    if (client_curve) {
+        conn->kex_params.server_ecc_evp_params.negotiated_curve = conn->kex_params.client_ecc_evp_params.negotiated_curve;
         conn->kex_params.server_kem_group_params.kem_group = NULL;
         conn->kex_params.server_kem_group_params.ecc_params.negotiated_curve = NULL;
         conn->kex_params.server_kem_group_params.kem_params.kem = NULL;
         return S2N_SUCCESS;
     }
 
-    /* Server and client have mutually supported groups, but the client did not send key
-     * shares for any of them. Send HRR indicating the server's preference. */
+    /* Option 4: Server and client have at least 1 mutually supported group, but the client did not send key shares for
+     * any of them. Send a HelloRetryRequest indicating the server's preference. */
     POSIX_GUARD(s2n_set_hello_retry_required(conn));
     return S2N_SUCCESS;
 }
