@@ -20,6 +20,19 @@
 #include "tls/s2n_connection.h"
 #include "utils/s2n_safety.h"
 
+/* Chosen based on AWS server recommendations as of 05/24:
+ * - All supported curves
+ * - Prefer p256
+ */
+const struct s2n_ecc_named_curve *const s2n_ecc_pref_list_20240501[] = {
+    &s2n_ecc_curve_secp256r1,
+#if EVP_APIS_SUPPORTED
+    &s2n_ecc_curve_x25519,
+#endif
+    &s2n_ecc_curve_secp384r1,
+    &s2n_ecc_curve_secp521r1,
+};
+
 const struct s2n_ecc_named_curve *const s2n_ecc_pref_list_20140601[] = {
     &s2n_ecc_curve_secp256r1,
     &s2n_ecc_curve_secp384r1,
@@ -57,6 +70,16 @@ const struct s2n_ecc_named_curve *const s2n_ecc_pref_list_20201021[] = {
     &s2n_ecc_curve_secp521r1,
 };
 
+/* Prefer x25519 over p256 for performance */
+const struct s2n_ecc_named_curve *const s2n_ecc_pref_list_20240603[] = {
+#if EVP_APIS_SUPPORTED
+    &s2n_ecc_curve_x25519,
+#endif
+    &s2n_ecc_curve_secp256r1,
+    &s2n_ecc_curve_secp384r1,
+    &s2n_ecc_curve_secp521r1,
+};
+
 const struct s2n_ecc_named_curve *const s2n_ecc_pref_list_20210816[] = {
     &s2n_ecc_curve_secp384r1,
 };
@@ -70,6 +93,11 @@ const struct s2n_ecc_named_curve *const s2n_ecc_pref_list_test_all[] = {
     &s2n_ecc_curve_secp521r1,
 };
 
+const struct s2n_ecc_preferences s2n_ecc_preferences_20240501 = {
+    .count = s2n_array_len(s2n_ecc_pref_list_20240501),
+    .ecc_curves = s2n_ecc_pref_list_20240501,
+};
+
 const struct s2n_ecc_preferences s2n_ecc_preferences_20140601 = {
     .count = s2n_array_len(s2n_ecc_pref_list_20140601),
     .ecc_curves = s2n_ecc_pref_list_20140601,
@@ -100,6 +128,11 @@ const struct s2n_ecc_preferences s2n_ecc_preferences_20210816 = {
     .ecc_curves = s2n_ecc_pref_list_20210816,
 };
 
+const struct s2n_ecc_preferences s2n_ecc_preferences_20240603 = {
+    .count = s2n_array_len(s2n_ecc_pref_list_20240603),
+    .ecc_curves = s2n_ecc_pref_list_20240603,
+};
+
 const struct s2n_ecc_preferences s2n_ecc_preferences_test_all = {
     .count = s2n_array_len(s2n_ecc_pref_list_test_all),
     .ecc_curves = s2n_ecc_pref_list_test_all,
