@@ -23,6 +23,10 @@
 #define S2N_TLS13_FIXED_STATE_SIZE            21
 #define S2N_TLS13_FIXED_EARLY_DATA_STATE_SIZE 3
 
+/* This is used in session ticket validation. This controls how far in the future
+ * the session ticket issue time can be while still being accepted.
+ */
+#define MAX_ALLOWED_CLOCK_SKEW_SEC     3600
 #define S2N_TLS_SESSION_CACHE_TTL      (6 * 60 * 60)
 #define S2N_TICKET_KEY_NAME_LEN        16
 #define S2N_TICKET_AAD_IMPLICIT_LEN    12
@@ -31,8 +35,10 @@
 #define ONE_SEC_IN_NANOS               1000000000
 #define ONE_MILLISEC_IN_NANOS          1000000
 #define ONE_WEEK_IN_SEC                604800
-#define S2N_TLS12_TICKET_SIZE_IN_BYTES (S2N_TICKET_KEY_NAME_LEN + S2N_TLS_GCM_IV_LEN \
-        + S2N_TLS12_STATE_SIZE_IN_BYTES + S2N_TLS_GCM_TAG_LEN)
+#define S2N_TICKET_INFO_SIZE           32
+#define S2N_TICKET_VERSION_SIZE        1
+#define S2N_TLS12_TICKET_SIZE_IN_BYTES (S2N_TICKET_VERSION_SIZE + S2N_TICKET_KEY_NAME_LEN \
+        + S2N_TICKET_INFO_SIZE + S2N_TLS_GCM_IV_LEN + S2N_TLS12_STATE_SIZE_IN_BYTES + S2N_TLS_GCM_TAG_LEN)
 
 #define S2N_TICKET_ENCRYPT_DECRYPT_KEY_LIFETIME_IN_NANOS 7200000000000  /* 2 hours */
 #define S2N_TICKET_DECRYPT_KEY_LIFETIME_IN_NANOS         46800000000000 /* 13 hours */
@@ -71,11 +77,11 @@ struct s2n_session_ticket {
 };
 
 struct s2n_ticket_key *s2n_find_ticket_key(struct s2n_config *config, const uint8_t name[S2N_TICKET_KEY_NAME_LEN]);
-int s2n_encrypt_session_ticket(struct s2n_connection *conn, struct s2n_stuffer *to);
-int s2n_decrypt_session_ticket(struct s2n_connection *conn, struct s2n_stuffer *from);
-int s2n_encrypt_session_cache(struct s2n_connection *conn, struct s2n_stuffer *to);
-int s2n_decrypt_session_cache(struct s2n_connection *conn, struct s2n_stuffer *from);
-int s2n_config_is_encrypt_decrypt_key_available(struct s2n_config *config);
+struct s2n_ticket_key *s2n_get_ticket_encrypt_decrypt_key(struct s2n_config *config);
+S2N_RESULT s2n_resume_encrypt_session_ticket(struct s2n_connection *conn, struct s2n_ticket_key *key, struct s2n_stuffer *to);
+S2N_RESULT s2n_resume_decrypt_session_ticket(struct s2n_connection *conn, struct s2n_stuffer *from);
+S2N_RESULT s2n_resume_decrypt_session_cache(struct s2n_connection *conn, struct s2n_stuffer *from);
+S2N_RESULT s2n_config_is_encrypt_key_available(struct s2n_config *config);
 int s2n_verify_unique_ticket_key(struct s2n_config *config, uint8_t *hash, uint16_t *insert_index);
 int s2n_config_wipe_expired_ticket_crypto_keys(struct s2n_config *config, int8_t expired_key_index);
 int s2n_config_store_ticket_key(struct s2n_config *config, struct s2n_ticket_key *key);
@@ -92,6 +98,15 @@ typedef enum {
     S2N_SERIALIZED_FORMAT_TLS12_V3,
 } s2n_serial_format_version;
 
+/* Used to specify the format of the ticket schema before encryption.
+ *
+ * This makes it easier to make changes to the ticket schema in the future
+ * as it allows us to interpret and parse all ticket schemas.
+ **/
+typedef enum {
+    S2N_PRE_ENCRYPTED_STATE_V1 = 1,
+} s2n_pre_encrypted_state;
+
 int s2n_allowed_to_cache_connection(struct s2n_connection *conn);
 int s2n_resume_from_cache(struct s2n_connection *conn);
 S2N_RESULT s2n_store_to_cache(struct s2n_connection *conn);
