@@ -72,15 +72,61 @@ int s2n_server_nst_recv(struct s2n_connection *conn)
     return S2N_SUCCESS;
 }
 
+static S2N_RESULT s2n_generate_ticket_lifetime(struct s2n_connection *conn, uint64_t key_intro_time,
+        uint32_t *ticket_lifetime)
+{
+    RESULT_ENSURE_REF(conn);
+    RESULT_ENSURE_REF(conn->config);
+    RESULT_ENSURE_MUT(ticket_lifetime);
+
+    uint64_t now = 0;
+    RESULT_GUARD(s2n_config_wall_clock(conn->config, &now));
+
+    /* Calculate ticket key age */
+    RESULT_ENSURE_GTE(now, key_intro_time);
+    uint64_t ticket_key_age_in_nanos = now - key_intro_time;
+
+    /* Calculate remaining key lifetime */
+    uint64_t key_lifetime_in_nanos = conn->config->encrypt_decrypt_key_lifetime_in_nanos + conn->config->decrypt_key_lifetime_in_nanos;
+    RESULT_ENSURE_GTE(key_lifetime_in_nanos, ticket_key_age_in_nanos);
+    uint32_t remaining_key_lifetime = (key_lifetime_in_nanos - ticket_key_age_in_nanos) / ONE_SEC_IN_NANOS;
+
+    uint32_t session_lifetime = conn->config->session_state_lifetime_in_nanos / ONE_SEC_IN_NANOS;
+
+    /* Min of remaining key lifetime and session */
+    uint32_t min_lifetime = MIN(remaining_key_lifetime, session_lifetime);
+
+    /* In TLS1.3 we take into account keying material lifetime */
+    if (conn->actual_protocol_version == S2N_TLS13) {
+        uint32_t key_material_lifetime = conn->server_keying_material_lifetime;
+        struct s2n_psk *chosen_psk = conn->psk_params.chosen_psk;
+        if (chosen_psk) {
+            RESULT_ENSURE_GTE(chosen_psk->keying_material_expiration, now);
+            uint32_t psk_key_material_lifetime = (chosen_psk->keying_material_expiration - now) / ONE_SEC_IN_NANOS;
+            key_material_lifetime = MIN(key_material_lifetime, psk_key_material_lifetime);
+        }
+        min_lifetime = MIN(min_lifetime, key_material_lifetime);
+    }
+
+    /**
+     *= https://www.rfc-editor.org/rfc/rfc8446#section-4.6.1
+     *# Servers MUST NOT use any value greater than
+     *# 604800 seconds (7 days).
+     **/
+    *ticket_lifetime = MIN(min_lifetime, ONE_WEEK_IN_SEC);
+
+    return S2N_RESULT_OK;
+}
+
 int s2n_server_nst_send(struct s2n_connection *conn)
 {
-    uint16_t session_ticket_len = S2N_TLS12_TICKET_SIZE_IN_BYTES;
+    POSIX_ENSURE_REF(conn);
+
     uint8_t data[S2N_TLS12_TICKET_SIZE_IN_BYTES] = { 0 };
-    struct s2n_blob entry = { 0 };
-    POSIX_GUARD(s2n_blob_init(&entry, data, sizeof(data)));
-    struct s2n_stuffer to = { 0 };
-    uint32_t lifetime_hint_in_secs =
-            (conn->config->encrypt_decrypt_key_lifetime_in_nanos + conn->config->decrypt_key_lifetime_in_nanos) / ONE_SEC_IN_NANOS;
+    struct s2n_blob session_ticket = { 0 };
+    POSIX_GUARD(s2n_blob_init(&session_ticket, data, sizeof(data)));
+
+    uint32_t lifetime_hint_in_secs = 0;
 
     /* Send a zero-length ticket in the NewSessionTicket message if the server changes 
      * its mind mid-handshake or if there are no valid encrypt keys currently available. 
@@ -91,22 +137,15 @@ int s2n_server_nst_send(struct s2n_connection *conn)
      *# ServerHello, then it sends a zero-length ticket in the
      *# NewSessionTicket handshake message.
      **/
-    POSIX_GUARD(s2n_stuffer_init(&to, &entry));
-    if (!conn->config->use_tickets || s2n_result_is_error(s2n_resume_encrypt_session_ticket(conn, &to))) {
+    if (s2n_result_is_error(s2n_server_nst_write(conn, &lifetime_hint_in_secs, &session_ticket))) {
         POSIX_GUARD(s2n_stuffer_write_uint32(&conn->handshake.io, 0));
         POSIX_GUARD(s2n_stuffer_write_uint16(&conn->handshake.io, 0));
-
         return S2N_SUCCESS;
     }
 
-    if (!s2n_server_sending_nst(conn)) {
-        POSIX_BAIL(S2N_ERR_SENDING_NST);
-    }
-
     POSIX_GUARD(s2n_stuffer_write_uint32(&conn->handshake.io, lifetime_hint_in_secs));
-    POSIX_GUARD(s2n_stuffer_write_uint16(&conn->handshake.io, session_ticket_len));
-
-    POSIX_GUARD(s2n_stuffer_write(&conn->handshake.io, &to.blob));
+    POSIX_GUARD(s2n_stuffer_write_uint16(&conn->handshake.io, session_ticket.size));
+    POSIX_GUARD(s2n_stuffer_write(&conn->handshake.io, &session_ticket));
 
     /* For parity with TLS1.3, track the single ticket sent.
      * This simplifies s2n_connection_get_tickets_sent.
@@ -115,6 +154,24 @@ int s2n_server_nst_send(struct s2n_connection *conn)
     return S2N_SUCCESS;
 }
 
+S2N_RESULT s2n_server_nst_write(struct s2n_connection *conn, uint32_t *lifetime_hint_in_secs,
+        struct s2n_blob *session_ticket)
+{
+    RESULT_ENSURE_REF(conn);
+    RESULT_ENSURE(s2n_server_sending_nst(conn), S2N_ERR_SENDING_NST);
+
+    struct s2n_stuffer output = { 0 };
+    RESULT_GUARD_POSIX(s2n_stuffer_init(&output, session_ticket));
+
+    struct s2n_ticket_key *key = s2n_get_ticket_encrypt_decrypt_key(conn->config);
+    RESULT_ENSURE(key != NULL, S2N_ERR_NO_TICKET_ENCRYPT_DECRYPT_KEY);
+
+    RESULT_GUARD(s2n_generate_ticket_lifetime(conn, key->intro_timestamp, lifetime_hint_in_secs));
+    RESULT_GUARD(s2n_resume_encrypt_session_ticket(conn, key, &output));
+
+    return S2N_RESULT_OK;
+}
+
 S2N_RESULT s2n_tls13_server_nst_send(struct s2n_connection *conn, s2n_blocked_status *blocked)
 {
     RESULT_ENSURE_REF(conn);
@@ -184,31 +241,6 @@ S2N_RESULT s2n_tls13_server_nst_send(struct s2n_connection *conn, s2n_blocked_st
     return S2N_RESULT_OK;
 }
 
-/** 
- *= https://www.rfc-editor.org/rfc/rfc8446#section-4.6.1
- *# Indicates the lifetime in seconds as a 32-bit
- *# unsigned integer in network byte order from the time of ticket
- *# issuance. 
- **/
-static S2N_RESULT s2n_generate_ticket_lifetime(struct s2n_connection *conn, uint32_t *ticket_lifetime)
-{
-    RESULT_ENSURE_REF(conn);
-    RESULT_ENSURE_MUT(ticket_lifetime);
-
-    uint32_t key_lifetime_in_secs =
-            (conn->config->encrypt_decrypt_key_lifetime_in_nanos + conn->config->decrypt_key_lifetime_in_nanos) / ONE_SEC_IN_NANOS;
-    uint32_t session_lifetime_in_secs = conn->config->session_state_lifetime_in_nanos / ONE_SEC_IN_NANOS;
-    uint32_t key_and_session_min_lifetime = MIN(key_lifetime_in_secs, session_lifetime_in_secs);
-    /** 
-     *= https://www.rfc-editor.org/rfc/rfc8446#section-4.6.1
-     *# Servers MUST NOT use any value greater than
-     *# 604800 seconds (7 days).
-     **/
-    *ticket_lifetime = MIN(key_and_session_min_lifetime, ONE_WEEK_IN_SEC);
-
-    return S2N_RESULT_OK;
-}
-
 /** 
  *= https://www.rfc-editor.org/rfc/rfc8446#section-4.6.1
  *# A per-ticket value that is unique across all tickets
@@ -271,6 +303,9 @@ S2N_RESULT s2n_tls13_server_nst_write(struct s2n_connection *conn, struct s2n_st
     RESULT_ENSURE_REF(conn);
     RESULT_ENSURE_REF(output);
 
+    struct s2n_ticket_key *key = s2n_get_ticket_encrypt_decrypt_key(conn->config);
+    RESULT_ENSURE(key != NULL, S2N_ERR_NO_TICKET_ENCRYPT_DECRYPT_KEY);
+
     struct s2n_ticket_fields *ticket_fields = &conn->tls13_ticket_fields;
 
     /* Write message type because session resumption in TLS13 is a post-handshake message */
@@ -280,7 +315,9 @@ S2N_RESULT s2n_tls13_server_nst_write(struct s2n_connection *conn, struct s2n_st
     RESULT_GUARD_POSIX(s2n_stuffer_reserve_uint24(output, &message_size));
 
     uint32_t ticket_lifetime_in_secs = 0;
-    RESULT_GUARD(s2n_generate_ticket_lifetime(conn, &ticket_lifetime_in_secs));
+    RESULT_GUARD(s2n_generate_ticket_lifetime(conn, key->intro_timestamp, &ticket_lifetime_in_secs));
+
+    RESULT_ENSURE(ticket_lifetime_in_secs > 0, S2N_ERR_ZERO_LIFETIME_TICKET);
     RESULT_GUARD_POSIX(s2n_stuffer_write_uint32(output, ticket_lifetime_in_secs));
 
     /* Get random data to use as ticket_age_add value */
@@ -310,7 +347,7 @@ S2N_RESULT s2n_tls13_server_nst_write(struct s2n_connection *conn, struct s2n_st
     /* Write ticket */
     struct s2n_stuffer_reservation ticket_size = { 0 };
     RESULT_GUARD_POSIX(s2n_stuffer_reserve_uint16(output, &ticket_size));
-    RESULT_GUARD(s2n_resume_encrypt_session_ticket(conn, output));
+    RESULT_GUARD(s2n_resume_encrypt_session_ticket(conn, key, output));
     RESULT_GUARD_POSIX(s2n_stuffer_write_vector_size(&ticket_size));
 
     RESULT_GUARD_POSIX(s2n_extension_list_send(S2N_EXTENSION_LIST_NST, conn, output));
