@@ -24,13 +24,14 @@
 
 struct s2n_signature_scheme {
     uint16_t iana_value;
+    const char *iana_name;
     s2n_hash_algorithm hash_alg;
     s2n_signature_algorithm sig_alg;
     uint8_t minimum_protocol_version;
     uint8_t maximum_protocol_version;
     uint16_t libcrypto_nid;
 
-    /* Curve is only specified for ECDSA Signatures */
+    /* Curve is only defined for TLS1.3 ECDSA Signatures */
     struct s2n_ecc_named_curve const *signature_curve;
 };
 
@@ -51,18 +52,12 @@ extern const struct s2n_signature_scheme s2n_rsa_pkcs1_sha256;
 extern const struct s2n_signature_scheme s2n_rsa_pkcs1_sha384;
 extern const struct s2n_signature_scheme s2n_rsa_pkcs1_sha512;
 
-/* TLS 1.2 Compatible ECDSA Schemes */
 extern const struct s2n_signature_scheme s2n_ecdsa_sha1;
 extern const struct s2n_signature_scheme s2n_ecdsa_sha224;
 extern const struct s2n_signature_scheme s2n_ecdsa_sha256;
 extern const struct s2n_signature_scheme s2n_ecdsa_sha384;
 extern const struct s2n_signature_scheme s2n_ecdsa_sha512;
 
-/* TLS 1.3 Compatible ECDSA Schemes */
-extern const struct s2n_signature_scheme s2n_ecdsa_secp256r1_sha256;
-extern const struct s2n_signature_scheme s2n_ecdsa_secp384r1_sha384;
-extern const struct s2n_signature_scheme s2n_ecdsa_secp521r1_sha512;
-
 /* RSA PSS */
 /*
  * Use RSA-PSS-RSAE instead of RSA-PSS-PSS in order to work with older certificates.
@@ -75,11 +70,13 @@ extern const struct s2n_signature_scheme s2n_rsa_pss_rsae_sha256;
 extern const struct s2n_signature_scheme s2n_rsa_pss_rsae_sha384;
 extern const struct s2n_signature_scheme s2n_rsa_pss_rsae_sha512;
 
+extern const struct s2n_signature_preferences s2n_signature_preferences_20240501;
 extern const struct s2n_signature_preferences s2n_signature_preferences_20230317;
 extern const struct s2n_signature_preferences s2n_signature_preferences_20140601;
 extern const struct s2n_signature_preferences s2n_signature_preferences_20200207;
 extern const struct s2n_signature_preferences s2n_signature_preferences_20201021;
 extern const struct s2n_signature_preferences s2n_signature_preferences_20210816;
+extern const struct s2n_signature_preferences s2n_signature_preferences_20240521;
 extern const struct s2n_signature_preferences s2n_signature_preferences_rfc9151;
 extern const struct s2n_signature_preferences s2n_certificate_signature_preferences_rfc9151;
 extern const struct s2n_signature_preferences s2n_signature_preferences_default_fips;
