@@ -15,7 +15,7 @@
 
 #include "utils/s2n_safety.h"
 
-void *s2n_ensure_memcpy_trace(void *restrict to, const void *restrict from, size_t size)
+void *s2n_ensure_memmove_trace(void *to, const void *from, size_t size)
 {
     PTR_ENSURE_REF(to);
     PTR_ENSURE_REF(from);
