@@ -60,13 +60,13 @@
     #define __S2N_ENSURE_POSTCONDITION(result) (s2n_likely(s2n_result_is_ok(result)) ? S2N_RESULT_OK : S2N_RESULT_ERROR)
 #endif
 
-#define __S2N_ENSURE_SAFE_MEMCPY(d, s, n, guard)                    \
-    do {                                                            \
-        __typeof(n) __tmp_n = (n);                                  \
-        if (s2n_likely(__tmp_n)) {                                  \
-            void *r = s2n_ensure_memcpy_trace((d), (s), (__tmp_n)); \
-            guard(r);                                               \
-        }                                                           \
+#define __S2N_ENSURE_SAFE_MEMMOVE(d, s, n, guard)                    \
+    do {                                                             \
+        __typeof(n) __tmp_n = (n);                                   \
+        if (s2n_likely(__tmp_n)) {                                   \
+            void *r = s2n_ensure_memmove_trace((d), (s), (__tmp_n)); \
+            guard(r);                                                \
+        }                                                            \
     } while (0)
 
 #define __S2N_ENSURE_SAFE_MEMSET(d, c, n, guard) \
@@ -90,23 +90,7 @@
     #define __S2N_ENSURE_CHECKED_RETURN(v) return v
 #endif
 
-/**
- * `restrict` is a part of the c99 standard and will work with any C compiler. If you're trying to
- * compile with a C++ compiler `restrict` is invalid. However some C++ compilers support the behavior
- * of `restrict` using the `__restrict__` keyword. Therefore if the compiler supports `__restrict__`
- * use it.
- *
- * This is helpful for the benchmarks in tests/benchmark which use Google's Benchmark library and
- * are all written in C++.
- *
- * https://gcc.gnu.org/onlinedocs/gcc/Restricted-Pointers.html
- *
- */
-#if defined(S2N___RESTRICT__SUPPORTED)
-void *s2n_ensure_memcpy_trace(void *__restrict__ to, const void *__restrict__ from, size_t size);
-#else
-void *s2n_ensure_memcpy_trace(void *restrict to, const void *restrict from, size_t size);
-#endif
+void *s2n_ensure_memmove_trace(void *to, const void *from, size_t size);
 
 /**
  * These macros should not be used in validate functions.
@@ -136,6 +120,9 @@ void *s2n_ensure_memcpy_trace(void *restrict to, const void *restrict from, size
 #define S2N_OBJECT_PTR_IS_READABLE(ptr) ((ptr) != NULL)
 #define S2N_OBJECT_PTR_IS_WRITABLE(ptr) ((ptr) != NULL)
 
+/**
+ * If `a` is true, then `b` must be true.
+ */
 #define S2N_IMPLIES(a, b) (!(a) || (b))
 /**
  * If and only if (iff) is a biconditional logical connective between statements a and b.
@@ -151,15 +138,19 @@ void *s2n_ensure_memcpy_trace(void *restrict to, const void *restrict from, size
  * Violations of the function contracts are undefined behaviour.
  */
 #ifdef CBMC
+    #define CONTRACT_ASSERT(...)      __CPROVER_assert(__VA_ARGS__)
     #define CONTRACT_ASSIGNS(...)     __CPROVER_assigns(__VA_ARGS__)
     #define CONTRACT_ASSIGNS_ERR(...) CONTRACT_ASSIGNS(__VA_ARGS__, _s2n_debug_info, s2n_errno)
+    #define CONTRACT_ASSUME(...)      __CPROVER_assume(__VA_ARGS__)
     #define CONTRACT_REQUIRES(...)    __CPROVER_requires(__VA_ARGS__)
     #define CONTRACT_ENSURES(...)     __CPROVER_ensures(__VA_ARGS__)
     #define CONTRACT_INVARIANT(...)   __CPROVER_loop_invariant(__VA_ARGS__)
     #define CONTRACT_RETURN_VALUE     (__CPROVER_return_value)
 #else
+    #define CONTRACT_ASSERT(...)
     #define CONTRACT_ASSIGNS(...)
     #define CONTRACT_ASSIGNS_ERR(...)
+    #define CONTRACT_ASSUME(...)
     #define CONTRACT_REQUIRES(...)
     #define CONTRACT_ENSURES(...)
     #define CONTRACT_INVARIANT(...)
